@@ -1,24 +1,24 @@
 from __future__ import print_function, division, absolute_import
 
-import os
-import inspect
 from contextlib import contextmanager
 from collections import namedtuple, defaultdict
-from pprint import pprint
 import sys
 import warnings
 import traceback
 import threading
-from .tracing import trace, event
+from .tracing import event
 
 from numba import (bytecode, interpreter, funcdesc, postproc,
                    typing, typeinfer, lowering, objmode, utils, config,
-                   errors, types, ir, types, rewrites, transforms)
+                   errors, types, ir, rewrites, transforms)
 from numba.targets import cpu, callconv
 from numba.annotations import type_annotations
 from numba.parfor import PreParforPass, ParforPass, Parfor
 from numba.inline_closurecall import InlineClosureCallPass
+from numba.errors import CompilerError
 
+# terminal color markup
+_termcolor = errors.termcolor()
 
 # Lock for the preventing multiple compiler execution
 lock_compiler = threading.RLock()
@@ -45,12 +45,14 @@ class Flags(utils.ConfigOptions):
         'forceinline': False,
         'no_cpython_wrapper': False,
         # Enable automatic parallel optimization, can be fine-tuned by taking
-        # a dictionary of sub-options instead of a boolean, see parfor.py for detail.
+        # a dictionary of sub-options instead of a boolean, see parfor.py for
+        # detail.
         'auto_parallel': cpu.ParallelOptions(False),
         'nrt': False,
         'no_rewrites': False,
         'error_model': 'python',
         'fastmath': False,
+        'noalias': False,
     }
 
 
@@ -143,7 +145,8 @@ def compile_result(**kws):
 def compile_isolated(func, args, return_type=None, flags=DEFAULT_FLAGS,
                      locals={}):
     """
-    Compile the function in an isolated environment (typing and target context).
+    Compile the function in an isolated environment (typing and target
+    context).
     Good for testing.
     """
     from .targets.registry import cpu_target
@@ -221,16 +224,18 @@ class _PipelineManager(object):
         newmsg = "{desc}\n{exc}".format(desc=desc, exc=exc)
 
         # For python2, attach the traceback of the previous exception.
-        if not utils.IS_PY3:
+        if not utils.IS_PY3 and config.FULL_TRACEBACKS:
+            # strip the new message to just print the error string and not
+            # the marked up source etc (this is handled already).
+            stripped = _termcolor.errmsg(newmsg.split('\n')[1])
             fmt = "Caused By:\n{tb}\n{newmsg}"
-            newmsg = fmt.format(tb=traceback.format_exc(), newmsg=newmsg)
+            newmsg = fmt.format(tb=traceback.format_exc(), newmsg=stripped)
 
         exc.args = (newmsg,)
         return exc
 
     def run(self, status):
         assert self._finalized, "PM must be finalized before run()"
-        res = None
         for pipeline_name in self.pipeline_order:
             event(pipeline_name)
             is_final_pipeline = pipeline_name == self.pipeline_order[-1]
@@ -257,7 +262,7 @@ class _PipelineManager(object):
         raise CompilerError("All pipelines have failed")
 
 
-class Pipeline(object):
+class BasePipeline(object):
     """
     Stores and manages states for the compiler pipeline
     """
@@ -446,12 +451,15 @@ class Pipeline(object):
         with self.fallback_context('Function "%s" failed type inference'
                                    % (self.func_id.func_name,)):
             # Type inference
-            self.typemap, self.return_type, self.calltypes = type_inference_stage(
+            typemap, return_type, calltypes = type_inference_stage(
                 self.typingctx,
                 self.func_ir,
                 self.args,
                 self.return_type,
                 self.locals)
+            self.typemap = typemap
+            self.return_type = return_type
+            self.calltypes = calltypes
 
         with self.fallback_context('Function "%s" has invalid return type'
                                    % (self.func_id.func_name,)):
@@ -464,9 +472,10 @@ class Pipeline(object):
         inference.
         """
         assert self.func_ir
-        with self.fallback_context('Internal error in pre-inference rewriting '
-                                   'pass encountered during compilation of '
-                                   'function "%s"' % (self.func_id.func_name,)):
+        msg = ('Internal error in pre-inference rewriting '
+               'pass encountered during compilation of '
+               'function "%s"' % (self.func_id.func_name,))
+        with self.fallback_context(msg):
             rewrites.rewrite_registry.apply('before-inference',
                                             self, self.func_ir)
 
@@ -479,9 +488,10 @@ class Pipeline(object):
         assert self.func_ir
         assert isinstance(getattr(self, 'typemap', None), dict)
         assert isinstance(getattr(self, 'calltypes', None), dict)
-        with self.fallback_context('Internal error in post-inference rewriting '
-                                   'pass encountered during compilation of '
-                                   'function "%s"' % (self.func_id.func_name,)):
+        msg = ('Internal error in post-inference rewriting '
+               'pass encountered during compilation of '
+               'function "%s"' % (self.func_id.func_name,))
+        with self.fallback_context(msg):
             rewrites.rewrite_registry.apply('after-inference',
                                             self, self.func_ir)
 
@@ -491,9 +501,12 @@ class Pipeline(object):
         """
         # Ensure we have an IR and type information.
         assert self.func_ir
-        preparfor_pass = PreParforPass(self.func_ir, self.type_annotation.typemap,
+        preparfor_pass = PreParforPass(
+            self.func_ir,
+            self.type_annotation.typemap,
             self.type_annotation.calltypes, self.typingctx,
-            self.flags.auto_parallel)
+            self.flags.auto_parallel
+            )
         preparfor_pass.run()
 
     def stage_parfor_pass(self):
@@ -504,7 +517,7 @@ class Pipeline(object):
         assert self.func_ir
         parfor_pass = ParforPass(self.func_ir, self.type_annotation.typemap,
             self.type_annotation.calltypes, self.return_type, self.typingctx,
-            self.flags.auto_parallel)
+            self.flags.auto_parallel, self.flags)
         parfor_pass.run()
 
         if config.WARNINGS:
@@ -524,11 +537,12 @@ class Pipeline(object):
                 if not self.func_ir.loc.filename == '<string>':
                     msg = ("parallel=True was specified but no transformation"
                            " for parallel execution was possible.")
-                    warnings.warn_explicit(msg,
-                                errors.NumbaWarning,
-                                self.func_id.filename,
-                                self.func_id.firstlineno)
-
+                    warnings.warn_explicit(
+                        msg,
+                        errors.NumbaWarning,
+                        self.func_id.filename,
+                        self.func_id.firstlineno
+                        )
 
     def stage_inline_pass(self):
         """
@@ -536,7 +550,8 @@ class Pipeline(object):
         """
         # Ensure we have an IR and type information.
         assert self.func_ir
-        inline_pass = InlineClosureCallPass(self.func_ir, self.flags.auto_parallel)
+        inline_pass = InlineClosureCallPass(self.func_ir,
+                                            self.flags.auto_parallel)
         inline_pass.run()
         # Remove all Dels, and re-run postproc
         post_proc = postproc.PostProcessor(self.func_ir)
@@ -547,7 +562,6 @@ class Pipeline(object):
             print(("IR DUMP: %s" % name).center(80, "-"))
             self.func_ir.dump()
 
-
     def stage_annotate_type(self):
         """
         Create type annotation after type inference
@@ -588,8 +602,9 @@ class Pipeline(object):
 
     def backend_nopython_mode(self):
         """Native mode compilation"""
-        with self.fallback_context("Function %s failed at nopython "
-                                   "mode lowering" % (self.func_id.func_name,)):
+        msg = ("Function %s failed at nopython "
+               "mode lowering" % (self.func_id.func_name,))
+        with self.fallback_context(msg):
             return native_lowering_stage(
                 self.targetctx,
                 self.library,
@@ -612,21 +627,22 @@ class Pipeline(object):
 
         lowered = lowerfn()
         signature = typing.signature(self.return_type, *self.args)
-        self.cr = compile_result(typing_context=self.typingctx,
-                                 target_context=self.targetctx,
-                                 entry_point=lowered.cfunc,
-                                 typing_error=self.status.fail_reason,
-                                 type_annotation=self.type_annotation,
-                                 library=self.library,
-                                 call_helper=lowered.call_helper,
-                                 signature=signature,
-                                 objectmode=objectmode,
-                                 interpmode=False,
-                                 lifted=self.lifted,
-                                 fndesc=lowered.fndesc,
-                                 environment=lowered.env,
-                                 has_dynamic_globals=lowered.has_dynamic_globals,
-                                 )
+        self.cr = compile_result(
+            typing_context=self.typingctx,
+            target_context=self.targetctx,
+            entry_point=lowered.cfunc,
+            typing_error=self.status.fail_reason,
+            type_annotation=self.type_annotation,
+            library=self.library,
+            call_helper=lowered.call_helper,
+            signature=signature,
+            objectmode=objectmode,
+            interpmode=False,
+            lifted=self.lifted,
+            fndesc=lowered.fndesc,
+            environment=lowered.env,
+            has_dynamic_globals=lowered.has_dynamic_globals,
+            )
 
     def stage_objectmode_backend(self):
         """
@@ -638,14 +654,18 @@ class Pipeline(object):
         # Warn if compiled function in object mode and force_pyobject not set
         if not self.flags.force_pyobject:
             if len(self.lifted) > 0:
-                warn_msg = 'Function "%s" was compiled in object mode without forceobj=True, but has lifted loops.' % (self.func_id.func_name,)
+                warn_msg = ('Function "%s" was compiled in object mode without'
+                            ' forceobj=True, but has lifted loops.' %
+                            (self.func_id.func_name,))
             else:
-                warn_msg = 'Function "%s" was compiled in object mode without forceobj=True.' % (self.func_id.func_name,)
+                warn_msg = ('Function "%s" was compiled in object mode without'
+                            ' forceobj=True.' % (self.func_id.func_name,))
             warnings.warn_explicit(warn_msg, errors.NumbaWarning,
                                    self.func_id.filename,
                                    self.func_id.firstlineno)
             if self.flags.release_gil:
-                warn_msg = "Code running in object mode won't allow parallel execution despite nogil=True."
+                warn_msg = ("Code running in object mode won't allow parallel"
+                            " execution despite nogil=True.")
                 warnings.warn_explicit(warn_msg, errors.NumbaWarning,
                                        self.func_id.filename,
                                        self.func_id.firstlineno)
@@ -679,48 +699,94 @@ class Pipeline(object):
         Cleanup intermediate results to release resources.
         """
 
+    def define_pipelines(self, pm):
+        """Child classes override this to customize the pipeline.
+        """
+        raise NotImplementedError()
+
+    def add_preprocessing_stage(self, pm):
+        """Add the preprocessing stage that analyzes the bytecode to prepare
+        the Numba IR.
+        """
+        if self.func_ir is None:
+            pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
+        pm.add_stage(self.stage_process_ir, "processing IR")
+
+    def add_pre_typing_stage(self, pm):
+        """Add any stages that go before type-inference.
+        The current stages contain type-agnostic rewrite passes.
+        """
+        if not self.flags.no_rewrites:
+            if self.status.can_fallback:
+                pm.add_stage(self.stage_preserve_ir,
+                             "preserve IR for fallback")
+            pm.add_stage(self.stage_generic_rewrites, "nopython rewrites")
+        pm.add_stage(self.stage_inline_pass,
+                     "inline calls to locally defined closures")
+
+    def add_typing_stage(self, pm):
+        """Add the type-inference stage necessary for nopython mode.
+        """
+        pm.add_stage(self.stage_nopython_frontend, "nopython frontend")
+        pm.add_stage(self.stage_annotate_type, "annotate type")
+
+    def add_optimization_stage(self, pm):
+        """Add optimization stages.
+        """
+        if self.flags.auto_parallel.enabled:
+            pm.add_stage(self.stage_pre_parfor_pass,
+                         "Preprocessing for parfors")
+        if not self.flags.no_rewrites:
+            pm.add_stage(self.stage_nopython_rewrites, "nopython rewrites")
+        if self.flags.auto_parallel.enabled:
+            pm.add_stage(self.stage_parfor_pass, "convert to parfors")
+
+    def add_lowering_stage(self, pm):
+        """Add the lowering (code-generation) stage for nopython-mode
+        """
+        pm.add_stage(self.stage_nopython_backend, "nopython mode backend")
+
+    def add_cleanup_stage(self, pm):
+        """Add the clean-up stage to remove intermediate results.
+        """
+        pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
+
+    def define_nopython_pipeline(self, pm, name='nopython'):
+        """Add the nopython-mode pipeline to the pipeline manager
+        """
+        pm.create_pipeline(name)
+        self.add_preprocessing_stage(pm)
+        self.add_pre_typing_stage(pm)
+        self.add_typing_stage(pm)
+        self.add_optimization_stage(pm)
+        self.add_lowering_stage(pm)
+        self.add_cleanup_stage(pm)
+
+    def define_objectmode_pipeline(self, pm, name='object'):
+        """Add the object-mode pipeline to the pipeline manager
+        """
+        pm.create_pipeline(name)
+        self.add_preprocessing_stage(pm)
+        pm.add_stage(self.stage_objectmode_frontend,
+                     "object mode frontend")
+        pm.add_stage(self.stage_annotate_type, "annotate type")
+        pm.add_stage(self.stage_objectmode_backend, "object mode backend")
+        self.add_cleanup_stage(pm)
+
+    def define_interpreted_pipeline(self, pm, name="interp"):
+        """Add the interpreted-mode (fallback) pipeline to the pipeline manager
+        """
+        pm.create_pipeline(name)
+        pm.add_stage(self.stage_compile_interp_mode,
+                     "compiling with interpreter mode")
+        self.add_cleanup_stage(pm)
+
     def _compile_core(self):
         """
         Populate and run compiler pipeline
         """
         pm = _PipelineManager()
-
-        if not self.flags.force_pyobject:
-            pm.create_pipeline("nopython")
-            if self.func_ir is None:
-                pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
-            pm.add_stage(self.stage_process_ir, "processing IR")
-            if not self.flags.no_rewrites:
-                if self.status.can_fallback:
-                    pm.add_stage(self.stage_preserve_ir, "preserve IR for fallback")
-                pm.add_stage(self.stage_generic_rewrites, "nopython rewrites")
-            pm.add_stage(self.stage_inline_pass, "inline calls to locally defined closures")
-            pm.add_stage(self.stage_nopython_frontend, "nopython frontend")
-            pm.add_stage(self.stage_annotate_type, "annotate type")
-            if self.flags.auto_parallel.enabled:
-                pm.add_stage(self.stage_pre_parfor_pass, "Preprocessing for parfors")
-            if not self.flags.no_rewrites:
-                pm.add_stage(self.stage_nopython_rewrites, "nopython rewrites")
-            if self.flags.auto_parallel.enabled:
-                pm.add_stage(self.stage_parfor_pass, "convert to parfors")
-            pm.add_stage(self.stage_nopython_backend, "nopython mode backend")
-            pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
-
-        if self.status.can_fallback or self.flags.force_pyobject:
-            pm.create_pipeline("object")
-            if self.func_ir is None:
-                pm.add_stage(self.stage_analyze_bytecode, "analyzing bytecode")
-            pm.add_stage(self.stage_process_ir, "processing IR")
-            pm.add_stage(self.stage_objectmode_frontend, "object mode frontend")
-            pm.add_stage(self.stage_annotate_type, "annotate type")
-            pm.add_stage(self.stage_objectmode_backend, "object mode backend")
-            pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
-
-        if self.status.can_giveup:
-            pm.create_pipeline("interp")
-            pm.add_stage(self.stage_compile_interp_mode, "compiling with interpreter mode")
-            pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
-
+        self.define_pipelines(pm)
         pm.finalize()
         res = pm.run(self.status)
         if res is not None:
@@ -745,6 +811,18 @@ class Pipeline(object):
         return self._compile_core()
 
 
+class Pipeline(BasePipeline):
+    """The default compiler pipeline
+    """
+    def define_pipelines(self, pm):
+        if not self.flags.force_pyobject:
+            self.define_nopython_pipeline(pm)
+        if self.status.can_fallback or self.flags.force_pyobject:
+            self.define_objectmode_pipeline(pm)
+        if self.status.can_giveup:
+            self.define_interpreted_pipeline(pm)
+
+
 def _make_subtarget(targetctx, flags):
     """
     Make a new target context from the given target context and flags.
@@ -767,15 +845,31 @@ def _make_subtarget(targetctx, flags):
 
 
 def compile_extra(typingctx, targetctx, func, args, return_type, flags,
-                  locals, library=None):
+                  locals, library=None, pipeline_class=Pipeline):
+    """Compiler entry point
+
+    Parameter
+    ---------
+    typingctx :
+        typing context
+    targetctx :
+        target context
+    func : function
+        the python function to be compiled
+    args : tuple, list
+        argument types
+    return_type :
+        Use ``None`` to indicate void return
+    flags : numba.compiler.Flags
+        compiler flags
+    library : numba.codegen.CodeLibrary
+        Used to store the compiled code.
+        If it is ``None``, a new CodeLibrary is used.
+    pipeline_class : type like numba.compiler.BasePipeline
+        compiler pipeline
     """
-    Args
-    ----
-    - return_type
-        Use ``None`` to indicate
-    """
-    pipeline = Pipeline(typingctx, targetctx, library,
-                        args, return_type, flags, locals)
+    pipeline = pipeline_class(typingctx, targetctx, library,
+                              args, return_type, flags, locals)
     return pipeline.compile_extra(func)
 
 
@@ -819,7 +913,7 @@ def legalize_return_type(return_type, interp, targetctx):
                     retstmts.append(inst.value.name)
                 elif isinstance(inst, ir.Assign):
                     if (isinstance(inst.value, ir.Expr)
-                        and inst.value.op == 'cast'):
+                            and inst.value.op == 'cast'):
                         caststmts[inst.target.name] = inst.value
                     elif isinstance(inst.value, ir.Arg):
                         argvars.add(inst.target.name)
@@ -829,8 +923,8 @@ def legalize_return_type(return_type, interp, targetctx):
         for var in retstmts:
             cast = caststmts.get(var)
             if cast is None or cast.value.name not in argvars:
-                raise TypeError("Only accept returning of array passed into the "
-                                "function as argument")
+                raise TypeError("Only accept returning of array passed into "
+                                "the function as argument")
 
     elif (isinstance(return_type, types.Function) or
             isinstance(return_type, types.Phantom)):
@@ -892,7 +986,7 @@ def native_lowering_stage(targetctx, library, interp, typemap, restype,
     # Lowering
     fndesc = funcdesc.PythonFunctionDescriptor.from_specialized_function(
         interp, typemap, restype, calltypes, mangler=targetctx.mangler,
-        inline=flags.forceinline)
+        inline=flags.forceinline, noalias=flags.noalias)
 
     lower = lowering.Lower(targetctx, library, fndesc, interp)
     lower.lower()
@@ -917,7 +1011,9 @@ def native_lowering_stage(targetctx, library, interp, typemap, restype,
 
 
 def py_lowering_stage(targetctx, library, interp, flags):
-    fndesc = funcdesc.PythonFunctionDescriptor.from_object_mode_function(interp)
+    fndesc = funcdesc.PythonFunctionDescriptor.from_object_mode_function(
+        interp
+        )
     lower = objmode.PyLower(targetctx, library, fndesc, interp)
     lower.lower()
     if not flags.no_cpython_wrapper:
