@@ -9,6 +9,7 @@ import contextlib
 import numpy as np
 
 from .cudadrv import devicearray, devices, driver
+from .args import In, Out, InOut
 
 
 try:
@@ -23,6 +24,50 @@ current_context = devices.get_context
 gpus = devices.gpus
 
 
+@require_context
+def from_cuda_array_interface(desc, owner=None):
+    """Create a DeviceNDArray from a cuda-array-interface description.
+    The *owner* is the owner of the underlying memory.
+    The resulting DeviceNDArray will acquire a reference from it.
+    """
+    shape = desc['shape']
+    strides = desc.get('strides')
+    dtype = np.dtype(desc['typestr'])
+
+    shape, strides, dtype = _prepare_shape_strides_dtype(
+        shape, strides, dtype, order='C')
+
+    devptr = driver.get_devptr_for_active_ctx(desc['data'][0])
+    data = driver.MemoryPointer(
+        current_context(), devptr, size=np.prod(shape) * dtype.itemsize,
+        owner=owner)
+    da = devicearray.DeviceNDArray(shape=shape, strides=strides,
+                                   dtype=dtype, gpu_data=data)
+    return da
+
+
+def as_cuda_array(obj):
+    """Create a DeviceNDArray from any object that implements
+    the cuda-array-interface.
+
+    A view of the underlying GPU buffer is created.  No copying of the data
+    is done.  The resulting DeviceNDArray will acquire a reference from `obj`.
+    """
+    if not is_cuda_array(obj):
+        raise TypeError("*obj* doesn't implement the cuda array interface.")
+    else:
+        return from_cuda_array_interface(obj.__cuda_array_interface__,
+                                         owner=obj)
+
+
+def is_cuda_array(obj):
+    """Test if the object has defined the `__cuda_array_interface__`.
+
+    Does not verify the validity of the interface.
+    """
+    return hasattr(obj, '__cuda_array_interface__')
+
+
 @require_context
 def to_device(obj, stream=0, copy=True, to=None):
     """to_device(obj, stream=0, copy=True, to=None)
@@ -115,6 +160,32 @@ def mapped_array(shape, dtype=np.float, strides=None, order='C', stream=0,
     return mappedview
 
 
+@contextlib.contextmanager
+@require_context
+def open_ipc_array(handle, shape, dtype, strides=None):
+    """
+    A context manager that opens a IPC *handle* (*CUipcMemHandle*) that is
+    represented as a sequence of bytes (e.g. *bytes*, tuple of int)
+    and represent it as an array of the given *shape*, *strides* and *dtype*.
+    The *strides* can be omitted.  In that case, it is assumed to be a 1D
+    C contiguous array.
+
+    Yields a device array.
+
+    The IPC handle is closed automatically when context manager exits.
+    """
+    dtype = np.dtype(dtype)
+    # compute size
+    size = np.prod(shape) * dtype.itemsize
+    # manually recreate the IPC mem handle
+    handle = driver.drvapi.cu_ipc_mem_handle(*handle)
+    # use *IpcHandle* to open the IPC memory
+    ipchandle = driver.IpcHandle(None, handle, size)
+    yield ipchandle.open_array(current_context(), shape=shape,
+                               strides=strides, dtype=dtype)
+    ipchandle.close()
+
+
 def synchronize():
     "Synchronize the current context."
     return current_context().synchronize()
