@@ -7,20 +7,25 @@ from __future__ import print_function, absolute_import, division
 
 import warnings
 import math
+import functools
 import copy
+from numba import six
 from ctypes import c_void_p
 
 import numpy as np
 
+import numba
 from . import driver as _driver
 from . import devices
 from numba import dummyarray, types, numpy_support
 from numba.unsafe.ndarray import to_fixed_tuple
 
 try:
-    long
-except NameError:
-    long = int
+    lru_cache = getattr(functools, 'lru_cache')(None)
+except AttributeError:
+    # Python 3.1 or lower
+    def lru_cache(func):
+        return func
 
 
 def is_cuda_ndarray(obj):
@@ -41,7 +46,7 @@ def verify_cuda_ndarray_interface(obj):
     requires_attr('shape', tuple)
     requires_attr('strides', tuple)
     requires_attr('dtype', np.dtype)
-    requires_attr('size', (int, long))
+    requires_attr('size', six.integer_types)
 
 
 def require_cuda_ndarray(obj):
@@ -75,9 +80,9 @@ class DeviceNDArrayBase(object):
         gpu_data
             user provided device memory for the ndarray data buffer
         """
-        if isinstance(shape, (int, long)):
+        if isinstance(shape, six.integer_types):
             shape = (shape,)
-        if isinstance(strides, (int, long)):
+        if isinstance(strides, six.integer_types):
             strides = (strides,)
         self.ndim = len(shape)
         if len(strides) != self.ndim:
@@ -108,6 +113,16 @@ class DeviceNDArrayBase(object):
         self.__writeback = writeback    # should deprecate the use of this
         self.stream = stream
 
+    @property
+    def __cuda_array_interface__(self):
+        return {
+            'shape': tuple(self.shape),
+            'strides': tuple(self.strides),
+            'data': (self.device_ctypes_pointer.value, False),
+            'typestr': self.dtype.str,
+            'version': 0,
+        }
+
     def bind(self, stream=0):
         """Bind a CUDA stream to this object so that all subsequent operation
         on this array defaults to the given stream.
@@ -320,6 +335,43 @@ class DeviceRecord(DeviceNDArrayBase):
         return numpy_support.from_dtype(self.dtype)
 
 
+@lru_cache
+def _assign_kernel(ndim):
+    """
+    A separate method so we don't need to compile code every assignment (!).
+
+    :param ndim: We need to have static array sizes for cuda.local.array, so
+        bake in the number of dimensions into the kernel
+    """
+    from numba import cuda  # circular!
+
+    @cuda.jit
+    def kernel(lhs, rhs):
+        location = cuda.grid(1)
+
+        n_elements = 1
+        for i in range(lhs.ndim):
+            n_elements *= lhs.shape[i]
+        if location >= n_elements:
+            # bake n_elements into the kernel, better than passing it in
+            # as another argument.
+            return
+
+        # [0, :] is the to-index (into `lhs`)
+        # [1, :] is the from-index (into `rhs`)
+        idx = cuda.local.array(
+            shape=(2, ndim),
+            dtype=types.int64)
+
+        for i in range(ndim - 1, -1, -1):
+            idx[0, i] = location % lhs.shape[i]
+            idx[1, i] = (location % lhs.shape[i]) * (rhs.shape[i] > 1)
+            location //= lhs.shape[i]
+
+        lhs[to_fixed_tuple(idx[0], ndim)] = rhs[to_fixed_tuple(idx[1], ndim)]
+    return kernel
+
+
 class DeviceNDArray(DeviceNDArrayBase):
     '''
     An on-GPU array type
@@ -352,6 +404,9 @@ class DeviceNDArray(DeviceNDArrayBase):
         """
         return self.copy_to_host().__array__(dtype)
 
+    def __len__(self):
+        return self.shape[0]
+
     def reshape(self, *newshape, **kws):
         """
         Reshape the array without changing its contents, similarly to
@@ -478,37 +533,8 @@ class DeviceNDArray(DeviceNDArrayBase):
 
         # (3) do the copy
 
-        from numba import cuda  # circular!
-
         n_elements = np.prod(lhs.shape)
-
-        # need to have static array sizes for cuda.local.array, so bake in the
-        # dimensions
-        ndim = lhs.ndim
-
-        @cuda.jit
-        def kernel(lhs, rhs):
-            location = cuda.grid(1)
-
-            if location >= n_elements:
-                # bake n_elements into the kernel, better than passing it in
-                # as another argument.
-                return
-
-            # [0, :] is the to-index (into `lhs`)
-            # [1, :] is the from-index (into `rhs`)
-            idx = cuda.local.array(
-                shape=(2, ndim),
-                dtype=types.int64)
-
-            for i in range(ndim - 1, -1, -1):
-                idx[0, i] = location % lhs.shape[i]
-                idx[1, i] = (location % lhs.shape[i]) * (rhs.shape[i] > 1)
-                location //= lhs.shape[i]
-
-            lhs[to_fixed_tuple(idx[0], ndim)] = rhs[to_fixed_tuple(idx[1], ndim)]
-
-        kernel.forall(n_elements, stream=stream)(lhs, rhs)
+        _assign_kernel(lhs.ndim).forall(n_elements, stream=stream)(lhs, rhs)
 
 
 
@@ -602,6 +628,8 @@ def auto_device(obj, stream=0, copy=True):
     """
     if _driver.is_device_memory(obj):
         return obj, False
+    elif hasattr(obj, '__cuda_array_interface__'):
+        return numba.cuda.as_cuda_array(obj), False
     else:
         if isinstance(obj, np.void):
             devobj = from_record_like(obj, stream=stream)
