@@ -6,10 +6,12 @@ import threading
 
 import numpy as np
 
+from numba import six
 from numba.six import reraise
-from .array import to_device
+from .cudadrv.devicearray import to_device, auto_device
 from .kernelapi import Dim3, FakeCUDAModule, swapped_cuda_module
 from ..errors import normalize_kernel_dimensions
+from ..args import wrap_arg, ArgHint
 
 
 """
@@ -46,10 +48,11 @@ class FakeCUDAKernel(object):
     Wraps a @cuda.jit-ed function.
     '''
 
-    def __init__(self, fn, device, fastmath=False):
+    def __init__(self, fn, device, fastmath=False, extensions=[]):
         self.fn = fn
         self._device = device
         self._fastmath = fastmath
+        self.extensions = list(extensions) # defensive copy
         # Initial configuration: 1 block, 1 thread, stream 0, no dynamic shared
         # memory.
         self[1, 1, 0, 0]
@@ -64,18 +67,37 @@ class FakeCUDAKernel(object):
         with _push_kernel_context(fake_cuda_module):
             # fake_args substitutes all numpy arrays for FakeCUDAArrays
             # because they implement some semantics differently
-            def fake_arg(arg):
-                if isinstance(arg, np.ndarray) and arg.ndim > 0:
-                    return to_device(arg)
-                return arg
-            fake_args = [fake_arg(arg) for arg in args]
+            retr = []
 
+            def fake_arg(arg):
+                # map the arguments using any extension you've registered
+                _, arg = six.moves.reduce(
+                    lambda ty_val, extension: extension.prepare_args(
+                        *ty_val,
+                        stream=0,
+                        retr=retr),
+                    self.extensions,
+                    (None, arg)
+                )
+
+                if isinstance(arg, np.ndarray) and arg.ndim > 0:
+                    return wrap_arg(arg).to_device(retr)
+                elif isinstance(arg, ArgHint):
+                    return arg.to_device(retr)
+                else:
+                    return arg
+
+            fake_args = [fake_arg(arg) for arg in args]
             with swapped_cuda_module(self.fn, fake_cuda_module):
                 # Execute one block at a time
                 for grid_point in np.ndindex(*self.grid_dim):
                     bm = BlockManager(self.fn, self.grid_dim, self.block_dim)
                     bm.run(grid_point, *fake_args)
 
+            for wb in retr:
+                wb()
+
+
     def __getitem__(self, configuration):
         self.grid_dim, self.block_dim = \
             normalize_kernel_dimensions(*configuration[:2])
@@ -88,6 +110,9 @@ class FakeCUDAKernel(object):
     def bind(self):
         pass
 
+    def forall(self, ntasks, tpb=0, stream=0, sharedmem=0):
+        return self[ntasks, 1, stream, sharedmem]
+
     @property
     def ptx(self):
         '''
@@ -118,6 +143,8 @@ class BlockThread(threading.Thread):
         self.blockIdx = Dim3(*blockIdx)
         self.threadIdx = Dim3(*threadIdx)
         self.exception = None
+        blockDim = Dim3(*self._manager._block_dim)
+        self.thread_id = self.threadIdx.x + blockDim.x * (self.threadIdx.y + blockDim.y * self.threadIdx.z)
 
     def run(self):
         try:
@@ -137,6 +164,27 @@ class BlockThread(threading.Thread):
         self.syncthreads_event.wait()
         self.syncthreads_event.clear()
 
+    def syncthreads_count(self, value):
+        self._manager.block_state[self.threadIdx.x, self.threadIdx.y, self.threadIdx.z] = value
+        self.syncthreads()
+        count = np.count_nonzero(self._manager.block_state)
+        self.syncthreads()
+        return count
+
+    def syncthreads_and(self, value):
+        self._manager.block_state[self.threadIdx.x, self.threadIdx.y, self.threadIdx.z] = value
+        self.syncthreads()
+        test = np.all(self._manager.block_state)
+        self.syncthreads()
+        return 1 if test else 0
+
+    def syncthreads_or(self, value):
+        self._manager.block_state[self.threadIdx.x, self.threadIdx.y, self.threadIdx.z] = value
+        self.syncthreads()
+        test = np.any(self._manager.block_state)
+        self.syncthreads()
+        return 1 if test else 0
+
     def __str__(self):
         return 'Thread <<<%s, %s>>>' % (self.blockIdx, self.threadIdx)
 
@@ -163,6 +211,7 @@ class BlockManager(object):
         self._grid_dim = grid_dim
         self._block_dim = block_dim
         self._f = f
+        self.block_state = np.zeros(block_dim, dtype=np.bool)
 
     def run(self, grid_point, *args):
         # Create all threads
