@@ -5,15 +5,17 @@ from llvmlite.llvmpy.core import (Type, Builder, LINKAGE_INTERNAL,
 import llvmlite.llvmpy.core as lc
 import llvmlite.binding as ll
 
-from numba import typing, types, cgutils
+from numba import typing, types, cgutils, debuginfo, dispatcher
 from numba.utils import cached_property
 from numba.targets.base import BaseContext
 from numba.targets.callconv import MinimalCallConv
 from numba.targets import cmathimpl, operatorimpl
 from numba.typing import cmathdecl, operatordecl
-from numba.funcdesc import transform_arg_name
+
+from numba import itanium_mangler
 from .cudadrv import nvvm
 from . import codegen, nvvmutils
+from .decorators import jitdevice
 
 
 # -----------------------------------------------------------------------------
@@ -29,6 +31,25 @@ class CUDATypingContext(typing.BaseContext):
         self.install_registry(cmathdecl.registry)
         self.install_registry(operatordecl.registry)
 
+    def resolve_value_type(self, val):
+        # treat dispatcher object as another device function
+        if isinstance(val, dispatcher.Dispatcher):
+            try:
+                # use cached device function
+                val = val.__cudajitdevice
+            except AttributeError:
+                if not val._can_compile:
+                    raise ValueError('using cpu function on device '
+                                     'but its compilation is disabled')
+                jd = jitdevice(val, debug=val.targetoptions.get('debug'))
+                # cache the device function for future use and to avoid
+                # duplicated copy of the same function.
+                val.__cudajitdevice = jd
+                val = jd
+
+        # continue with parent logic
+        return super(CUDATypingContext, self).resolve_value_type(val)
+
 # -----------------------------------------------------------------------------
 # Implementation
 
@@ -38,6 +59,7 @@ VALID_CHARS = re.compile(r'[^a-z0-9]', re.I)
 class CUDATargetContext(BaseContext):
     implement_powi_as_math_call = True
     strict_alignment = True
+    DIBuilder = debuginfo.NvvmDIBuilder
 
     # Overrides
     def create_module(self, name):
@@ -66,23 +88,10 @@ class CUDATargetContext(BaseContext):
     def call_conv(self):
         return CUDACallConv(self)
 
-    @classmethod
-    def mangle_name(cls, name):
-        """
-        Mangle the given string
-        """
-        def repl(m):
-            ch = m.group(0)
-            return "_%X_" % ord(ch)
-
-        return VALID_CHARS.sub(repl, name)
-
     def mangler(self, name, argtypes):
-        qualified = name + '.' + '.'.join(transform_arg_name(a)
-                                          for a in argtypes)
-        return self.mangle_name(qualified)
+        return itanium_mangler.mangle(name, argtypes)
 
-    def prepare_cuda_kernel(self, codelib, fname, argtypes):
+    def prepare_cuda_kernel(self, codelib, fname, argtypes, debug):
         """
         Adapt a code library ``codelib`` with the numba compiled CUDA kernel
         with name ``fname`` and arguments ``argtypes`` for NVVM.
@@ -93,11 +102,12 @@ class CUDATargetContext(BaseContext):
         """
         library = self.codegen().create_library('')
         library.add_linking_library(codelib)
-        wrapper = self.generate_kernel_wrapper(library, fname, argtypes)
+        wrapper = self.generate_kernel_wrapper(library, fname, argtypes,
+                                               debug=debug)
         nvvm.fix_data_layout(library._final_module)
         return library, wrapper
 
-    def generate_kernel_wrapper(self, library, fname, argtypes):
+    def generate_kernel_wrapper(self, library, fname, argtypes, debug):
         """
         Generate the kernel wrapper in the given ``library``.
         The function being wrapped have the name ``fname`` and argument types
@@ -111,7 +121,8 @@ class CUDATargetContext(BaseContext):
                              [self.call_conv.get_return_type(types.pyobject)] + argtys)
         func = wrapper_module.add_function(fnty, name=fname)
 
-        wrapfn = wrapper_module.add_function(wrapfnty, name="cudaPy_" + func.name)
+        prefixed = itanium_mangler.prepend_namespace(func.name, ns='cudapy')
+        wrapfn = wrapper_module.add_function(wrapfnty, name=prefixed)
         builder = Builder(wrapfn.append_basic_block(''))
 
         # Define error handling variables
@@ -132,35 +143,37 @@ class CUDATargetContext(BaseContext):
         status, _ = self.call_conv.call_function(
             builder, func, types.void, argtypes, callargs)
 
-        # Check error status
-        with cgutils.if_likely(builder, status.is_ok):
-            builder.ret_void()
 
-        with builder.if_then(builder.not_(status.is_python_exc)):
-            # User exception raised
-            old = Constant.null(gv_exc.type.pointee)
+        if debug:
+            # Check error status
+            with cgutils.if_likely(builder, status.is_ok):
+                builder.ret_void()
 
-            # Use atomic cmpxchg to prevent rewriting the error status
-            # Only the first error is recorded
+            with builder.if_then(builder.not_(status.is_python_exc)):
+                # User exception raised
+                old = Constant.null(gv_exc.type.pointee)
 
-            casfnty = lc.Type.function(old.type, [gv_exc.type, old.type,
-                                                  old.type])
+                # Use atomic cmpxchg to prevent rewriting the error status
+                # Only the first error is recorded
 
-            casfn = wrapper_module.add_function(casfnty,
-                                                name="___numba_cas_hack")
-            xchg = builder.call(casfn, [gv_exc, old, status.code])
-            changed = builder.icmp(ICMP_EQ, xchg, old)
+                casfnty = lc.Type.function(old.type, [gv_exc.type, old.type,
+                                                    old.type])
 
-            # If the xchange is successful, save the thread ID.
-            sreg = nvvmutils.SRegBuilder(builder)
-            with builder.if_then(changed):
-                for dim, ptr, in zip("xyz", gv_tid):
-                    val = sreg.tid(dim)
-                    builder.store(val, ptr)
+                casfn = wrapper_module.add_function(casfnty,
+                                                    name="___numba_cas_hack")
+                xchg = builder.call(casfn, [gv_exc, old, status.code])
+                changed = builder.icmp(ICMP_EQ, xchg, old)
 
-                for dim, ptr, in zip("xyz", gv_ctaid):
-                    val = sreg.ctaid(dim)
-                    builder.store(val, ptr)
+                # If the xchange is successful, save the thread ID.
+                sreg = nvvmutils.SRegBuilder(builder)
+                with builder.if_then(changed):
+                    for dim, ptr, in zip("xyz", gv_tid):
+                        val = sreg.tid(dim)
+                        builder.store(val, ptr)
+
+                    for dim, ptr, in zip("xyz", gv_ctaid):
+                        val = sreg.ctaid(dim)
+                        builder.store(val, ptr)
 
         builder.ret_void()
 
@@ -186,7 +199,8 @@ class CUDATargetContext(BaseContext):
         addrspace.
         """
         text = Constant.stringz(string)
-        name = '.'.join(["__conststring__", self.mangle_name(string)])
+        name = '$'.join(["__conststring__",
+                         itanium_mangler.mangle_identifier(string)])
         # Try to reuse existing global
         gv = mod.globals.get(name)
         if gv is None:
