@@ -1,10 +1,10 @@
 from __future__ import absolute_import, print_function
 import numpy as np
 from numba import cuda
-from numba.cuda.testing import unittest
+from numba.cuda.testing import unittest, SerialMixin
 
 
-class TestCudaEvent(unittest.TestCase):
+class TestCudaEvent(SerialMixin, unittest.TestCase):
     def test_event_elapsed(self):
         N = 32
         dary = cuda.device_array(N, dtype=np.double)
@@ -16,7 +16,8 @@ class TestCudaEvent(unittest.TestCase):
         evtend.record()
         evtend.wait()
         evtend.synchronize()
-        print(evtstart.elapsed_time(evtend))
+        # Exercise the code path
+        evtstart.elapsed_time(evtend)
 
     def test_event_elapsed_stream(self):
         N = 32
@@ -30,7 +31,8 @@ class TestCudaEvent(unittest.TestCase):
         evtend.record(stream=stream)
         evtend.wait(stream=stream)
         evtend.synchronize()
-        print(evtstart.elapsed_time(evtend))
+        # Exercise the code path
+        evtstart.elapsed_time(evtend)
 
 if __name__ == '__main__':
     unittest.main()
