@@ -2,12 +2,14 @@ from __future__ import print_function, absolute_import, division
 import os.path
 import numpy as np
 from numba.cuda.testing import unittest
+from numba.cuda.testing import skip_on_cudasim
+from numba.cuda.testing import SerialMixin
 from numba.cuda.cudadrv.driver import Linker
 from numba.cuda import require_context
 from numba import cuda
 
-
-class TestLinker(unittest.TestCase):
+@skip_on_cudasim('Linking unsupported in the simulator')
+class TestLinker(SerialMixin, unittest.TestCase):
 
     @require_context
     def test_linker_basic(self):
@@ -35,5 +37,6 @@ class TestLinker(unittest.TestCase):
 
         self.assertTrue(A[0] == 123 + 2 * 321)
 
+
 if __name__ == '__main__':
     unittest.main()
