@@ -21,6 +21,7 @@ class DataFlowAnalysis(object):
         self.bytecode = cfa.bytecode
         # { block offset -> BlockInfo }
         self.infos = {}
+        self.edge_process = {}
 
     def run(self):
         for blk in self.cfa.iterliveblocks():
@@ -28,7 +29,8 @@ class DataFlowAnalysis(object):
 
     def run_on_block(self, blk):
         incoming_blocks = []
-        info = BlockInfo(blk.offset, incoming_blocks)
+        info = BlockInfo(blk, blk.offset, incoming_blocks)
+        edge_callbacks = []
 
         for ib, pops in self.cfa.incoming_blocks(blk):
             # By nature of Python bytecode, there will be no incoming
@@ -38,6 +40,8 @@ class DataFlowAnalysis(object):
                 continue
             ib = self.infos[ib.offset]
             incoming_blocks.append(ib)
+            if (ib.offset, blk.offset) in self.edge_process:
+                edge_callbacks.append(self.edge_process[(ib.offset, blk.offset)])
 
             # Compute stack offset at block entry
             # The stack effect of our predecessors should be known
@@ -66,6 +70,9 @@ class DataFlowAnalysis(object):
             info.syntax_blocks = []
         info.stack_effect = 0
 
+        for callback in edge_callbacks:
+            callback(info)
+
         for offset in blk:
             inst = self.bytecode[offset]
             self.dispatch(info, inst)
@@ -77,9 +84,14 @@ class DataFlowAnalysis(object):
 
     def dispatch(self, info, inst):
         fname = "op_%s" % inst.opname.replace('+', '_')
-        fn = getattr(self, fname)
+        fn = getattr(self, fname, self.handle_unknown_opcode)
         fn(info, inst)
 
+    def handle_unknown_opcode(self, info, inst):
+        msg = "Use of unknown opcode {} at line {} of {}"
+        raise NotImplementedError(msg.format(inst.opname, inst.lineno,
+                                             self.bytecode.func_id.filename))
+
     def dup_topx(self, info, inst, count):
         orig = [info.pop() for _ in range(count)]
         orig.reverse()
@@ -167,6 +179,18 @@ class DataFlowAnalysis(object):
         info.append(inst, items=items, res=lst)
         info.push(lst)
 
+    def op_LIST_APPEND(self, info, inst):
+        value = info.pop()
+        # Python 2.7+ added an argument to LIST_APPEND.
+        if sys.version_info[:2] == (2, 6):
+            target = info.pop()
+        else:
+            index = inst.arg
+            target = info.peek(index)
+        appendvar = info.make_temp()
+        res = info.make_temp()
+        info.append(inst, target=target, value=value, appendvar=appendvar, res=res)
+
     def op_BUILD_MAP(self, info, inst):
         dct = info.make_temp()
         count = inst.arg
@@ -181,7 +205,8 @@ class DataFlowAnalysis(object):
 
     def op_BUILD_SET(self, info, inst):
         count = inst.arg
-        items = [info.pop() for _ in range(count)]
+        # Note: related python bug http://bugs.python.org/issue26020
+        items = list(reversed([info.pop() for _ in range(count)]))
         res = info.make_temp()
         info.append(inst, items=items, res=res)
         info.push(res)
@@ -208,6 +233,10 @@ class DataFlowAnalysis(object):
         dct = info.tos
         info.append(inst, dct=dct, key=key, value=value)
 
+    def op_STORE_DEREF(self, info, inst):
+        value = info.pop()
+        info.append(inst, value=value)
+
     def op_LOAD_FAST(self, info, inst):
         name = self.bytecode.co_varnames[inst.arg]
         res = info.make_temp(name)
@@ -266,32 +295,85 @@ class DataFlowAnalysis(object):
         pred = info.make_temp()
         info.append(inst, iterator=iterator, pair=pair, indval=indval, pred=pred)
         info.push(indval)
+        # Setup for stack POP (twice) at loop exit (before processing instruction at jump target)
+        def pop_info(info):
+            info.pop()
+            info.pop()
+        self.edge_process[(info.block.offset, inst.get_jump_target())] = pop_info
 
-    def _op_call_function(self, info, inst, has_vararg):
-        narg = inst.arg & 0xff
-        nkws = (inst.arg >> 8) & 0xff
+    if utils.PYVERSION < (3, 6):
 
-        def pop_kws():
-            val = info.pop()
-            key = info.pop()
-            return key, val
+        def _op_call_function(self, info, inst, has_vararg):
+            narg = inst.arg & 0xff
+            nkws = (inst.arg >> 8) & 0xff
 
-        vararg = info.pop() if has_vararg else None
-        kws = list(reversed([pop_kws() for _ in range(nkws)]))
-        args = list(reversed([info.pop() for _ in range(narg)]))
-        func = info.pop()
+            def pop_kws():
+                val = info.pop()
+                key = info.pop()
+                return key, val
 
+            vararg = info.pop() if has_vararg else None
+            kws = list(reversed([pop_kws() for _ in range(nkws)]))
+            args = list(reversed([info.pop() for _ in range(narg)]))
+            func = info.pop()
+
+            res = info.make_temp()
+            info.append(inst, func=func, args=args, kws=kws, res=res,
+                        vararg=vararg)
+            info.push(res)
+
+        def op_CALL_FUNCTION(self, info, inst):
+            self._op_call_function(info, inst, has_vararg=False)
+
+        def op_CALL_FUNCTION_VAR(self, info, inst):
+            self._op_call_function(info, inst, has_vararg=True)
+
+    else:
+        def op_CALL_FUNCTION(self, info, inst):
+            narg = inst.arg
+            args = list(reversed([info.pop() for _ in range(narg)]))
+            func = info.pop()
+
+            res = info.make_temp()
+            info.append(inst, func=func, args=args, res=res)
+            info.push(res)
+
+        def op_CALL_FUNCTION_KW(self, info, inst):
+            narg = inst.arg
+            names = info.pop()  # tuple of names
+            args = list(reversed([info.pop() for _ in range(narg)]))
+            func = info.pop()
+
+            res = info.make_temp()
+            info.append(inst, func=func, args=args, names=names, res=res)
+            info.push(res)
+
+        def op_CALL_FUNCTION_EX(self, info, inst):
+            if inst.arg & 1:
+                errmsg = 'CALL_FUNCTION_EX with **kwargs not supported'
+                raise NotImplementedError(errmsg)
+            vararg = info.pop()
+            func = info.pop()
+            res = info.make_temp()
+            info.append(inst, func=func, vararg=vararg, res=res)
+            info.push(res)
+
+    def op_BUILD_TUPLE_UNPACK_WITH_CALL(self, info, inst):
+        # Builds tuple from other tuples on the stack
+        tuples = list(reversed([info.pop() for _ in range(inst.arg)]))
+        temps = [info.make_temp() for _ in range(len(tuples) - 1)]
+        info.append(inst, tuples=tuples, temps=temps)
+        # The result is in the last temp var
+        info.push(temps[-1])
+
+    def op_BUILD_CONST_KEY_MAP(self, info, inst):
+        keys = info.pop()
+        vals = list(reversed([info.pop() for _ in range(inst.arg)]))
+        keytmps = [info.make_temp() for _ in range(inst.arg)]
         res = info.make_temp()
-        info.append(inst, func=func, args=args, kws=kws, res=res,
-                    vararg=vararg)
+        info.append(inst, keys=keys, keytmps=keytmps, values=vals, res=res)
         info.push(res)
 
-    def op_CALL_FUNCTION(self, info, inst):
-        self._op_call_function(info, inst, has_vararg=False)
-
-    def op_CALL_FUNCTION_VAR(self, info, inst):
-        self._op_call_function(info, inst, has_vararg=True)
-
     def op_PRINT_ITEM(self, info, inst):
         warnings.warn("Python2 style print partially supported.  Please use "
                       "Python3 style print.", RuntimeWarning)
@@ -599,6 +681,73 @@ class DataFlowAnalysis(object):
             raise ValueError("Multiple argument raise is not supported.")
         info.append(inst, exc=exc)
 
+    def op_MAKE_FUNCTION(self, info, inst, MAKE_CLOSURE=False):
+        if utils.PYVERSION == (2, 7):
+            name = None
+        else:
+            name = info.pop()
+        code = info.pop()
+        closure = annotations = kwdefaults = defaults = None
+        if utils.PYVERSION < (3, 0):
+            if MAKE_CLOSURE:
+                closure = info.pop()
+            num_posdefaults = inst.arg
+            if num_posdefaults > 0:
+                defaults = []
+                for i in range(num_posdefaults):
+                    defaults.append(info.pop())
+                defaults = tuple(defaults)
+        elif utils.PYVERSION >= (3, 0) and utils.PYVERSION < (3, 6):
+            num_posdefaults = inst.arg & 0xff
+            num_kwdefaults = (inst.arg >> 8) & 0xff
+            num_annotations = (inst.arg >> 16) & 0x7fff
+            if MAKE_CLOSURE:
+                closure = info.pop()
+            if num_annotations > 0:
+                annotations = info.pop() 
+            if num_kwdefaults > 0:
+                kwdefaults = []
+                for i in range(num_kwdefaults):
+                    v = info.pop()
+                    k = info.pop()
+                    kwdefaults.append((k,v))
+                kwdefaults = tuple(kwdefaults)
+            if num_posdefaults:
+                defaults = []
+                for i in range(num_posdefaults):
+                    defaults.append(info.pop())
+                defaults = tuple(defaults)
+        else:
+            if inst.arg & 0x8:
+                closure = info.pop()
+            if inst.arg & 0x4:
+                annotations = info.pop()
+            if inst.arg & 0x2:
+                kwdefaults = info.pop()
+            if inst.arg & 0x1:
+                defaults = info.pop()
+        res = info.make_temp()
+        info.append(inst, name=name, code=code, closure=closure, annotations=annotations, 
+                    kwdefaults=kwdefaults, defaults=defaults, res=res)
+        info.push(res)
+
+    def op_MAKE_CLOSURE(self, info, inst):
+        self.op_MAKE_FUNCTION(info, inst, MAKE_CLOSURE=True)
+
+    def op_LOAD_CLOSURE(self, info, inst):
+        res = info.make_temp()
+        info.append(inst, res=res)
+        info.push(res)
+
+    #NOTE: Please see notes in `interpreter.py` surrounding the implementation
+    # of LOAD_METHOD and CALL_METHOD.
+
+    def op_LOAD_METHOD(self, *args, **kws):
+        self.op_LOAD_ATTR(*args, **kws)
+
+    def op_CALL_METHOD(self, *args, **kws):
+        self.op_CALL_FUNCTION(*args, **kws)
+
     def _ignored(self, info, inst):
         pass
 
@@ -611,7 +760,8 @@ class LoopBlock(object):
 
 
 class BlockInfo(object):
-    def __init__(self, offset, incoming_blocks):
+    def __init__(self, block, offset, incoming_blocks):
+        self.block = block
         self.offset = offset
         # The list of incoming BlockInfo objects (obtained by control
         # flow analysis).
@@ -661,6 +811,18 @@ class BlockInfo(object):
             self.stack_effect -= 1
             return self.stack.pop()
 
+    def peek(self, k):
+        """
+        Return the k'th element back from the top of the stack.
+        peek(1) is the top of the stack.
+        """
+        num_pops = k
+        top_k = [self.pop() for _ in range(num_pops)]
+        r = top_k[-1]
+        for i in range(num_pops - 1, -1, -1):
+            self.push(top_k[i])
+        return r
+
     def make_incoming(self):
         """
         Create an incoming variable (due to not enough values being
