@@ -322,10 +322,6 @@ class MemInfoModel(OpaqueModel):
         return True
 
     def get_nrt_meminfo(self, builder, value):
-        for tp in self.inner_types():
-            if self._dmm.lookup(tp).has_nrt_meminfo():
-                raise NotImplementedError(
-                    "unsupported nested memory-managed object")
         return value
 
 
@@ -1151,7 +1147,8 @@ class GeneratorModel(CompositeModel):
 class ArrayCTypesModel(StructModel):
     def __init__(self, dmm, fe_type):
         # ndim = fe_type.ndim
-        members = [('data', types.CPointer(fe_type.dtype))]
+        members = [('data', types.CPointer(fe_type.dtype)),
+                   ('meminfo', types.MemInfoPointer(fe_type.dtype))]
         super(ArrayCTypesModel, self).__init__(dmm, fe_type, members)
 
 
