@@ -5,8 +5,9 @@ import dis
 import sys
 from copy import copy
 
-from . import config, ir, controlflow, dataflow, utils, errors
-from .utils import builtins
+from . import config, ir, controlflow, dataflow, utils, errors, six
+from .utils import builtins, PYVERSION
+from .errors import NotDefinedError
 
 
 class Assigner(object):
@@ -171,6 +172,7 @@ class Interpreter(object):
                                                       loc=self.loc)
             stmt = ir.Assign(value=self.get(varname), target=target,
                              loc=self.loc)
+            self.definitions[target.name].append(stmt.value)
             if not self.current_block.is_terminated:
                 self.current_block.append(stmt)
             else:
@@ -213,6 +215,10 @@ class Interpreter(object):
     def code_names(self):
         return self.bytecode.co_names
 
+    @property
+    def code_cellvars(self):
+        return self.bytecode.co_cellvars 
+
     @property
     def code_freevars(self):
         return self.bytecode.co_freevars
@@ -229,8 +235,15 @@ class Interpreter(object):
                 return fn(inst, **kws)
             except errors.NotDefinedError as e:
                 if e.loc is None:
-                    e.loc = self.loc
-                raise e
+                    loc = self.loc
+                else:
+                    loc = e.loc
+
+                err = errors.NotDefinedError(e.name, loc=loc)
+                if not config.FULL_TRACEBACKS:
+                    six.raise_from(err, None)
+                else:
+                    raise err
 
 
     # --- Scope operations ---
@@ -241,7 +254,8 @@ class Interpreter(object):
         (a str object).
         """
         if redefine or self.current_block_offset in self.cfa.backbone:
-            target = self.current_scope.redefine(name, loc=self.loc)
+            rename = not (name in self.code_cellvars)
+            target = self.current_scope.redefine(name, loc=self.loc, rename=rename)
         else:
             target = self.current_scope.get_or_define(name, loc=self.loc)
         if isinstance(value, ir.Var):
@@ -254,6 +268,11 @@ class Interpreter(object):
         """
         Get the variable (a Var instance) with the given *name*.
         """
+        # Implicit argument for comprehension starts with '.'
+        # See Parameter class in inspect.py (from Python source)
+        if name[0] == '.' and name[1:].isdigit():
+            name = 'implicit{}'.format(name[1:])
+ 
         # Try to simplify the variable lookup by returning an earlier
         # variable assigned to *name*.
         var = self.assigner.get_assignment_source(name)
@@ -568,42 +587,115 @@ class Interpreter(object):
         self.store(gl, res)
 
     def op_LOAD_DEREF(self, inst, res):
-        name = self.code_freevars[inst.arg]
-        value = self.get_closure_value(inst.arg)
-        gl = ir.FreeVar(inst.arg, name, value, loc=self.loc)
+        n_cellvars = len(self.code_cellvars)
+        if inst.arg < n_cellvars:
+            name = self.code_cellvars[inst.arg]
+            gl = self.get(name)
+        else:
+            idx = inst.arg - n_cellvars
+            name = self.code_freevars[idx]
+            value = self.get_closure_value(idx)
+            gl = ir.FreeVar(idx, name, value, loc=self.loc)
         self.store(gl, res)
 
+    def op_STORE_DEREF(self, inst, value):
+        n_cellvars = len(self.code_cellvars)
+        if inst.arg < n_cellvars:
+            dstname = self.code_cellvars[inst.arg]
+        else:
+            dstname = self.code_freevars[inst.arg - n_cellvars]
+        value = self.get(value)
+        self.store(value=value, name=dstname)
+
     def op_SETUP_LOOP(self, inst):
         assert self.blocks[inst.offset] is self.current_block
         loop = ir.Loop(inst.offset, exit=(inst.next + inst.arg))
         self.syntax_blocks.append(loop)
 
-    def op_CALL_FUNCTION(self, inst, func, args, kws, res, vararg):
-        func = self.get(func)
-        args = [self.get(x) for x in args]
-        if vararg is not None:
-            vararg = self.get(vararg)
+    if PYVERSION < (3, 6):
 
-        # Process keywords
-        keyvalues = []
-        removethese = []
-        for k, v in kws:
-            k, v = self.get(k), self.get(v)
+        def op_CALL_FUNCTION(self, inst, func, args, kws, res, vararg):
+            func = self.get(func)
+            args = [self.get(x) for x in args]
+            if vararg is not None:
+                vararg = self.get(vararg)
+
+            # Process keywords
+            keyvalues = []
+            removethese = []
+            for k, v in kws:
+                k, v = self.get(k), self.get(v)
+                for inst in self.current_block.body:
+                    if isinstance(inst, ir.Assign) and inst.target is k:
+                        removethese.append(inst)
+                        keyvalues.append((inst.value.value, v))
+
+            # Remove keyword constant statements
+            for inst in removethese:
+                self.current_block.remove(inst)
+
+            expr = ir.Expr.call(func, args, keyvalues, loc=self.loc,
+                                vararg=vararg)
+            self.store(expr, res)
+
+        op_CALL_FUNCTION_VAR = op_CALL_FUNCTION
+    else:
+        def op_CALL_FUNCTION(self, inst, func, args, res):
+            func = self.get(func)
+            args = [self.get(x) for x in args]
+            expr = ir.Expr.call(func, args, (), loc=self.loc)
+            self.store(expr, res)
+
+        def op_CALL_FUNCTION_KW(self, inst, func, args, names, res):
+            func = self.get(func)
+            args = [self.get(x) for x in args]
+            # Find names const
+            names = self.get(names)
             for inst in self.current_block.body:
-                if isinstance(inst, ir.Assign) and inst.target is k:
-                    removethese.append(inst)
-                    keyvalues.append((inst.value.value, v))
+                if isinstance(inst, ir.Assign) and inst.target is names:
+                    self.current_block.remove(inst)
+                    keys = inst.value.value
+                    break
 
-        # Remove keyword constant statements
-        for inst in removethese:
-            self.current_block.remove(inst)
+            nkeys = len(keys)
+            posvals = args[:-nkeys]
+            kwvals = args[-nkeys:]
+            keyvalues = list(zip(keys, kwvals))
 
-        expr = ir.Expr.call(func, args, keyvalues, loc=self.loc,
-                            vararg=vararg)
+            expr = ir.Expr.call(func, posvals, keyvalues, loc=self.loc)
+            self.store(expr, res)
+
+        def op_CALL_FUNCTION_EX(self, inst, func, vararg, res):
+            func = self.get(func)
+            vararg = self.get(vararg)
+            expr = ir.Expr.call(func, [], [], loc=self.loc, vararg=vararg)
+            self.store(expr, res)
+
+    def op_BUILD_TUPLE_UNPACK_WITH_CALL(self, inst, tuples, temps):
+        first = self.get(tuples[0])
+        for other, tmp in zip(map(self.get, tuples[1:]), temps):
+            out = ir.Expr.binop(fn='+', lhs=first, rhs=other, loc=self.loc)
+            self.store(out, tmp)
+            first = tmp
+
+    def op_BUILD_CONST_KEY_MAP(self, inst, keys, keytmps, values, res):
+        # Unpack the constant key-tuple and reused build_map which takes
+        # a sequence of (key, value) pair.
+        keyvar = self.get(keys)
+        # TODO: refactor this pattern. occurred several times.
+        for inst in self.current_block.body:
+            if isinstance(inst, ir.Assign) and inst.target is keyvar:
+                self.current_block.remove(inst)
+                keytup = inst.value.value
+                break
+        assert len(keytup) == len(values)
+        keyconsts = [ir.Const(value=x, loc=self.loc) for x in keytup]
+        for kval, tmp in zip(keyconsts, keytmps):
+            self.store(kval, tmp)
+        items = list(zip(map(self.get, keytmps), map(self.get, values)))
+        expr = ir.Expr.build_map(items=items, size=2, loc=self.loc)
         self.store(expr, res)
 
-    op_CALL_FUNCTION_VAR = op_CALL_FUNCTION
-
     def op_GET_ITER(self, inst, value, res):
         expr = ir.Expr.getiter(value=self.get(value), loc=self.loc)
         self.store(expr, res)
@@ -859,3 +951,59 @@ class Interpreter(object):
         index = None
         inst = ir.Yield(value=self.get(value), index=index, loc=self.loc)
         return self.store(inst, res)
+
+    def op_MAKE_FUNCTION(self, inst, name, code, closure, annotations, kwdefaults, defaults, res):
+        if annotations != None:
+            raise NotImplementedError("op_MAKE_FUNCTION with annotations is not implemented")
+        if kwdefaults != None:
+            raise NotImplementedError("op_MAKE_FUNCTION with kwdefaults is not implemented")
+        if isinstance(defaults, tuple):
+            defaults = tuple([self.get(name) for name in defaults])
+        fcode = self.definitions[code][0].value
+        if name:
+            name = self.get(name)
+        if closure:
+            closure = self.get(closure)
+        expr = ir.Expr.make_function(name, fcode, closure, defaults, self.loc)
+        self.store(expr, res)
+
+    def op_MAKE_CLOSURE(self, inst, name, code, closure, annotations, kwdefaults, defaults, res):
+        self.op_MAKE_FUNCTION(inst, name, code, closure, annotations, kwdefaults, defaults, res)
+    
+    def op_LOAD_CLOSURE(self, inst, res):
+        n_cellvars = len(self.code_cellvars)
+        if inst.arg < n_cellvars:
+            name = self.code_cellvars[inst.arg]
+            try:
+                gl = self.get(name)
+            except NotDefinedError as e:
+                raise NotImplementedError("Unsupported use of op_LOAD_CLOSURE encountered")
+        else:
+            idx = inst.arg - n_cellvars
+            name = self.code_freevars[idx]
+            value = self.get_closure_value(idx)
+            gl = ir.FreeVar(idx, name, value, loc=self.loc)
+        self.store(gl, res)
+
+    def op_LIST_APPEND(self, inst, target, value, appendvar, res):
+        target = self.get(target)
+        value = self.get(value)
+        appendattr = ir.Expr.getattr(target, 'append', loc=self.loc)
+        self.store(value=appendattr, name=appendvar)
+        appendinst = ir.Expr.call(self.get(appendvar), (value,), (), loc=self.loc)
+        self.store(value=appendinst, name=res)
+
+
+    # NOTE: The LOAD_METHOD opcode is implemented as a LOAD_ATTR for ease,
+    # however this means a new object (the bound-method instance) could be
+    # created. Conversely, using a pure LOAD_METHOD no intermediary is present
+    # and it is essentially like a pointer grab and forward to CALL_METHOD. The
+    # net outcome is that the implementation in Numba produces the same result,
+    # but in object mode it may be that it runs more slowly than it would if
+    # run in CPython.
+
+    def op_LOAD_METHOD(self, *args, **kws):
+        self.op_LOAD_ATTR(*args, **kws)
+
+    def op_CALL_METHOD(self, *args, **kws):
+        self.op_CALL_FUNCTION(*args, **kws)
