@@ -2,15 +2,20 @@ from __future__ import print_function, division, absolute_import
 
 from collections import defaultdict
 import copy
+import itertools
 import os
+import linecache
 import pprint
 import sys
+import warnings
+from numba import config, errors
 
 from . import utils
 from .errors import (NotDefinedError, RedefinedError, VerificationError,
                      ConstantInferenceError)
-from numba import config
 
+# terminal color markup
+_termcolor = errors.termcolor()
 
 class Loc(object):
     """Source location
@@ -36,16 +41,62 @@ class Loc(object):
         else:
             return "%s (%s)" % (self.filename, self.line)
 
-    def strformat(self):
+    def strformat(self, nlines_up=2):
         try:
             # Try to get a relative path
+            # ipython/jupyter input just returns as self.filename
             path = os.path.relpath(self.filename)
         except ValueError:
-            # Fallback to absolute path if error occured in getting the
+            # Fallback to absolute path if error occurred in getting the
             # relative path.
             # This may happen on windows if the drive is different
             path = os.path.abspath(self.filename)
-        return 'File "%s", line %d' % (path, self.line)
+
+        lines = linecache.getlines(path)
+
+        ret = [] # accumulates output
+        if lines and self.line:
+
+            def count_spaces(string):
+                spaces = 0
+                for x in itertools.takewhile(str.isspace, str(string)):
+                    spaces += 1
+                return spaces
+
+            selected = lines[self.line - nlines_up:self.line]
+            # see if selected contains a definition
+            def_found = False
+            for x in selected:
+                if 'def ' in x:
+                    def_found = True
+
+            # no definition found, try and find one
+            if not def_found:
+                # try and find a def, go backwards from error line
+                fn_name = None
+                for x in reversed(lines[:self.line - 1]):
+                    if 'def ' in x:
+                        fn_name = x
+                        break
+                if fn_name:
+                    ret.append(fn_name)
+                    spaces = count_spaces(x)
+                    ret.append(' '*(4 + spaces) + '<source elided>\n')
+
+            ret.extend(selected[:-1])
+            ret.append(_termcolor.highlight(selected[-1]))
+
+            # point at the problem with a caret
+            spaces = count_spaces(selected[-1])
+            ret.append(' '*(spaces) + _termcolor.indicate("^"))
+
+        # if in the REPL source may not be available
+        if not ret:
+            ret = "<source missing, REPL in use?>"
+
+        err = _termcolor.filename('\nFile "%s", line %d:')+'\n%s'
+        tmp = err % (path, self.line, _termcolor.code(''.join(ret)))
+        return tmp
 
     def with_lineno(self, line, col=None):
         """
@@ -292,7 +343,7 @@ class Expr(Inst):
         return self._rec_list_vars(self._kws)
 
     def infer_constant(self):
-        raise ConstantInferenceError("cannot make a constant of %s" % (self,))
+        raise ConstantInferenceError('%s' % self, loc=self.loc)
 
 
 class SetItem(Stmt):
@@ -523,7 +574,7 @@ class Arg(object):
         return 'arg(%d, name=%s)' % (self.index, self.name)
 
     def infer_constant(self):
-        raise ConstantInferenceError("cannot make a constant of %s" % (self,))
+        raise ConstantInferenceError('%s' % self, loc=self.loc)
 
 
 class Const(object):
@@ -754,6 +805,16 @@ class Block(object):
             if isinstance(inst, cls):
                 yield inst
 
+    def find_variable_assignment(self, name):
+        """
+        Returns the assignment inst associated with variable "name", None if
+        it cannot be found.
+        """
+        for x in self.find_insts(cls=Assign):
+            if x.target.name == name:
+                return x
+        return None
+
     def prepend(self, inst):
         assert isinstance(inst, Stmt)
         self.body.insert(0, inst)
@@ -859,7 +920,6 @@ class FunctionIR(object):
         Post-processing will have to be run again on the new IR.
         """
         firstblock = blocks[min(blocks)]
-        is_generator = self.is_generator and not force_non_generator
 
         new_ir = copy.copy(self)
         new_ir.blocks = blocks
@@ -871,7 +931,8 @@ class FunctionIR(object):
         if arg_names is not None:
             new_ir.arg_names = arg_names
         new_ir._reset_analysis_variables()
-
+        # Make fresh func_id
+        new_ir.func_id = new_ir.func_id.derive()
         return new_ir
 
     def copy(self):
