@@ -5,6 +5,7 @@
 import numpy
 
 import types as pytypes
+import collections
 
 from llvmlite import ir as lir
 
@@ -481,20 +482,21 @@ def remove_args(blocks):
     return
 
 
-def remove_dead(blocks, args, typemap=None, alias_map=None, arg_aliases=None):
+def remove_dead(blocks, args, func_ir, typemap=None, alias_map=None, arg_aliases=None):
     """dead code elimination using liveness and CFG info.
     Returns True if something has been removed, or False if nothing is removed.
     """
     cfg = compute_cfg_from_blocks(blocks)
     usedefs = compute_use_defs(blocks)
     live_map = compute_live_map(cfg, blocks, usedefs.usemap, usedefs.defmap)
+    call_table, _ = get_call_table(blocks)
     if alias_map is None or arg_aliases is None:
-        alias_map, arg_aliases = find_potential_aliases(blocks, args, typemap)
+        alias_map, arg_aliases = find_potential_aliases(blocks, args, typemap,
+                                                        func_ir)
     if config.DEBUG_ARRAY_OPT == 1:
         print("alias map:", alias_map)
     # keep set for easier search
     alias_set = set(alias_map.keys())
-    call_table, _ = get_call_table(blocks)
 
     removed = False
     for label, block in blocks.items():
@@ -503,7 +505,8 @@ def remove_dead(blocks, args, typemap=None, alias_map=None, arg_aliases=None):
         # find live variables at the end of block
         for out_blk, _data in cfg.successors(label):
             lives |= live_map[out_blk]
-        removed |= remove_dead_block(block, lives, call_table, arg_aliases, alias_map, alias_set, typemap)
+        removed |= remove_dead_block(block, lives, call_table, arg_aliases,
+                                     alias_map, alias_set, func_ir, typemap)
     return removed
 
 
@@ -512,7 +515,8 @@ def remove_dead(blocks, args, typemap=None, alias_map=None, arg_aliases=None):
 remove_dead_extensions = {}
 
 
-def remove_dead_block(block, lives, call_table, arg_aliases, alias_map, alias_set, typemap):
+def remove_dead_block(block, lives, call_table, arg_aliases, alias_map,
+                                                  alias_set, func_ir, typemap):
     """remove dead code using liveness info.
     Mutable arguments (e.g. arrays) that are not definitely assigned are live
     after return of function.
@@ -527,13 +531,14 @@ def remove_dead_block(block, lives, call_table, arg_aliases, alias_map, alias_se
     for stmt in reversed(block.body[:-1]):
         # aliases of lives are also live
         alias_lives = set()
-        init_alias_lives = (lives | arg_aliases) & alias_set
+        init_alias_lives = lives & alias_set
         for v in init_alias_lives:
             alias_lives |= alias_map[v]
+        lives_n_aliases = lives | alias_lives | arg_aliases
         # let external calls handle stmt if type matches
         if type(stmt) in remove_dead_extensions:
             f = remove_dead_extensions[type(stmt)]
-            stmt = f(stmt, lives, arg_aliases, alias_map, typemap)
+            stmt = f(stmt, lives, arg_aliases, alias_map, func_ir, typemap)
             if stmt is None:
                 removed = True
                 continue
@@ -542,7 +547,7 @@ def remove_dead_block(block, lives, call_table, arg_aliases, alias_map, alias_se
             lhs = stmt.target
             rhs = stmt.value
             if lhs.name not in lives and has_no_side_effect(
-                    rhs, lives, call_table):
+                    rhs, lives_n_aliases, call_table):
                 removed = True
                 continue
             if isinstance(rhs, ir.Var) and lhs.name == rhs.name:
@@ -551,7 +556,7 @@ def remove_dead_block(block, lives, call_table, arg_aliases, alias_map, alias_se
             # TODO: remove other nodes like SetItem etc.
         if isinstance(stmt, ir.SetItem):
             name = stmt.target.name
-            if not (name in lives or name in alias_lives or name in arg_aliases):
+            if name not in lives_n_aliases:
                 continue
 
         if type(stmt) in analysis.ir_extension_usedefs:
@@ -574,7 +579,7 @@ remove_call_handlers = []
 
 def remove_dead_random_call(rhs, lives, call_list):
     if len(call_list) == 3 and call_list[1:] == ['random', numpy]:
-        return True
+        return call_list[0] != 'seed'
     return False
 
 remove_call_handlers.append(remove_dead_random_call)
@@ -630,7 +635,8 @@ def is_pure(rhs, lives, call_table):
             return False
         call_list = call_table[func_name]
         if (call_list == [slice] or
-            call_list == ['log', numpy]):
+            call_list == ['log', numpy] or
+            call_list == ['empty', numpy]):
             return True
         for f in is_pure_extensions:
             if f(rhs, lives, call_list):
@@ -642,18 +648,24 @@ def is_pure(rhs, lives, call_table):
 
 alias_analysis_extensions = {}
 
-def find_potential_aliases(blocks, args, typemap, alias_map=None, arg_aliases=None):
+def find_potential_aliases(blocks, args, typemap, func_ir, alias_map=None,
+                                                            arg_aliases=None):
     "find all array aliases and argument aliases to avoid remove as dead"
     if alias_map is None:
         alias_map = {}
     if arg_aliases is None:
         arg_aliases = set(a for a in args if not is_immutable_type(a, typemap))
 
+    # update definitions since they are not guaranteed to be up-to-date
+    # FIXME keep definitions up-to-date to avoid the need for rebuilding
+    func_ir._definitions = build_definitions(func_ir.blocks)
+    np_alias_funcs = ['ravel', 'transpose', 'reshape']
+
     for bl in blocks.values():
         for instr in bl.body:
             if type(instr) in alias_analysis_extensions:
                 f = alias_analysis_extensions[type(instr)]
-                f(instr, args, typemap, alias_map, arg_aliases)
+                f(instr, args, typemap, func_ir, alias_map, arg_aliases)
             if isinstance(instr, ir.Assign):
                 expr = instr.value
                 lhs = instr.target.name
@@ -666,6 +678,24 @@ def find_potential_aliases(blocks, args, typemap, alias_map=None, arg_aliases=No
                 if (isinstance(expr, ir.Expr) and (expr.op == 'cast' or
                     expr.op in ['getitem', 'static_getitem'])):
                     _add_alias(lhs, expr.value.name, alias_map, arg_aliases)
+                # array attributes like A.T
+                if (isinstance(expr, ir.Expr) and expr.op == 'getattr'
+                        and expr.attr in ['T', 'ctypes', 'flat']):
+                    _add_alias(lhs, expr.value.name, alias_map, arg_aliases)
+                # calls that can create aliases such as B = A.ravel()
+                if isinstance(expr, ir.Expr) and expr.op == 'call':
+                    fdef = guard(find_callname, func_ir, expr, typemap)
+                    # TODO: sometimes gufunc backend creates duplicate code
+                    # causing find_callname to fail. Example: test_argmax
+                    # ignored here since those cases don't create aliases
+                    # but should be fixed in general
+                    if fdef is None:
+                        continue
+                    fname, fmod = fdef
+                    if fmod == 'numpy' and fname in np_alias_funcs:
+                        _add_alias(lhs, expr.args[0].name, alias_map, arg_aliases)
+                    if isinstance(fmod, ir.Var) and fname in np_alias_funcs:
+                        _add_alias(lhs, fmod.name, alias_map, arg_aliases)
 
     # copy to avoid changing size during iteration
     old_alias_map = copy.deepcopy(alias_map)
@@ -843,7 +873,6 @@ apply_copy_propagate_extensions = {}
 
 
 def apply_copy_propagate(blocks, in_copies, name_var_table, typemap, calltypes,
-                         ext_func=lambda a, b, c, d:None, ext_data=None,
                          save_copies=None):
     """apply copy propagation to IR: replace variables when copies available"""
     # save_copies keeps an approximation of the copies that were applied, so
@@ -856,10 +885,9 @@ def apply_copy_propagate(blocks, in_copies, name_var_table, typemap, calltypes,
         var_dict = {l: name_var_table[r] for l, r in in_copies[label]}
         # assignments as dict to replace with latest value
         for stmt in block.body:
-            ext_func(label, stmt, var_dict, ext_data)
             if type(stmt) in apply_copy_propagate_extensions:
                 f = apply_copy_propagate_extensions[type(stmt)]
-                f(stmt, var_dict, name_var_table, ext_func, ext_data,
+                f(stmt, var_dict, name_var_table,
                     typemap, calltypes, save_copies)
             # only rhs of assignments should be replaced
             # e.g. if x=y is available, x in x=z shouldn't be replaced
@@ -1282,7 +1310,7 @@ def simplify(func_ir, typemap, calltypes):
         calltypes)
     restore_copy_var_names(func_ir.blocks, save_copies, typemap)
     # remove dead code to enable fusion
-    remove_dead(func_ir.blocks, func_ir.arg_names, typemap)
+    remove_dead(func_ir.blocks, func_ir.arg_names, func_ir, typemap)
     func_ir.blocks = simplify_CFG(func_ir.blocks)
     if config.DEBUG_ARRAY_OPT == 1:
         dprint_func_ir(func_ir, "after simplify")
@@ -1318,35 +1346,31 @@ def get_definition(func_ir, name, **kwargs):
     except KeyError:
         raise GuardException
 
-def build_definitions(blocks=None, func_ir=None):
+def build_definitions(blocks, definitions=None):
     """Build the definitions table of the given blocks by scanning
     through all blocks and instructions, useful when the definitions
     table is out-of-sync.
-    Must give at least one argument, and the new definitions table is
-    returned. If func_ir is not None, func_ir._definitions will be
-    updated with new entries. So If we also want to get rid of old
-    definitions table, we must do:
-      func_ir._definitions = build_definitions(blocks=func_ir.blocks)
+    Will return a new definition table if one is not passed.
     """
-    if func_ir == None:
-        definitions = dict()
-    else:
-        definitions = func_ir._definitions
-
-    if blocks == None:
-        assert(func_ir != None)
-        blocks = func_ir.blocks
+    if definitions is None:
+        definitions = collections.defaultdict(list)
 
     for block in blocks.values():
-        for inst in block.find_insts(ir.Assign):
-            name = inst.target.name
-            definition = definitions.get(name, [])
-            if definition == []:
-                definitions[name] = definition
-            definition.append(inst.value)
+        for inst in block.body:
+            if isinstance(inst, ir.Assign):
+                name = inst.target.name
+                definition = definitions.get(name, [])
+                if definition == []:
+                    definitions[name] = definition
+                definition.append(inst.value)
+            if type(inst) in build_defs_extensions:
+                f = build_defs_extensions[type(inst)]
+                f(inst, definitions)
 
     return definitions
 
+build_defs_extensions = {}
+
 def find_callname(func_ir, expr, typemap=None, definition_finder=get_definition):
     """Check if a call expression is calling a numpy function, and
     return the callee's function name and module name (both are strings),
@@ -1358,6 +1382,7 @@ def find_callname(func_ir, expr, typemap=None, definition_finder=get_definition)
     callee = expr.func
     callee_def = definition_finder(func_ir, callee)
     attrs = []
+    obj = None
     while True:
         if isinstance(callee_def, (ir.Global, ir.FreeVar)):
             # require(callee_def.value == numpy)
@@ -1372,20 +1397,26 @@ def find_callname(func_ir, expr, typemap=None, definition_finder=get_definition)
             if not value:
                 raise GuardException
             attrs.append(value)
-            if hasattr(callee_def.value, '__module__'):
-                mod_name = callee_def.value.__module__
+            def_val = callee_def.value
+            # get the underlying definition of Intrinsic object to be able to
+            # find the module effectively.
+            # Otherwise, it will return numba.extending
+            if isinstance(def_val, numba.extending._Intrinsic):
+                def_val = def_val._defn
+            if hasattr(def_val, '__module__'):
+                mod_name = def_val.__module__
                 # it might be a numpy function imported directly
                 if (hasattr(numpy, value)
-                        and callee_def.value == getattr(numpy, value)):
+                        and def_val == getattr(numpy, value)):
                     attrs += ['numpy']
                 # it might be a np.random function imported directly
                 elif (hasattr(numpy.random, value)
-                        and callee_def.value == getattr(numpy.random, value)):
+                        and def_val == getattr(numpy.random, value)):
                     attrs += ['random', 'numpy']
                 elif mod_name is not None:
                     attrs.append(mod_name)
             else:
-                class_name = callee_def.value.__class__.__name__
+                class_name = def_val.__class__.__name__
                 if class_name == 'builtin_function_or_method':
                     class_name = 'builtin'
                 if class_name != 'module':
@@ -1400,6 +1431,9 @@ def find_callname(func_ir, expr, typemap=None, definition_finder=get_definition)
                     return attrs[0], obj
             callee_def = definition_finder(func_ir, obj)
         else:
+            # obj.func calls where obj is not np array
+            if obj is not None:
+                return '.'.join(reversed(attrs)), obj
             raise GuardException
     return attrs[0], '.'.join(reversed(attrs[1:]))
 
@@ -1585,3 +1619,66 @@ def dump_blocks(blocks):
         print(label, ":")
         for stmt in block.body:
             print("    ", stmt)
+
+def is_get_setitem(stmt):
+    """stmt is getitem assignment or setitem (and static cases)"""
+    return is_getitem(stmt) or is_setitem(stmt)
+
+
+def is_getitem(stmt):
+    """true if stmt is a getitem or static_getitem assignment"""
+    return (isinstance(stmt, ir.Assign)
+            and isinstance(stmt.value, ir.Expr)
+            and stmt.value.op in ['getitem', 'static_getitem'])
+
+def is_setitem(stmt):
+    """true if stmt is a SetItem or StaticSetItem node"""
+    return isinstance(stmt, (ir.SetItem, ir.StaticSetItem))
+
+def index_var_of_get_setitem(stmt):
+    """get index variable for getitem/setitem nodes (and static cases)"""
+    if is_getitem(stmt):
+        if stmt.value.op == 'getitem':
+            return stmt.value.index
+        else:
+            return stmt.value.index_var
+
+    if is_setitem(stmt):
+        if isinstance(stmt, ir.SetItem):
+            return stmt.index
+        else:
+            return stmt.index_var
+
+    return None
+
+def set_index_var_of_get_setitem(stmt, new_index):
+    if is_getitem(stmt):
+        if stmt.value.op == 'getitem':
+            stmt.value.index = new_index
+        else:
+            stmt.value.index_var = new_index
+    elif is_setitem(stmt):
+        if isinstance(stmt, ir.SetItem):
+            stmt.index = new_index
+        else:
+            stmt.index_var = new_index
+    else:
+        raise ValueError("getitem or setitem node expected but received {}".format(
+                     stmt))
+
+def is_namedtuple_class(c):
+    """check if c is a namedtuple class"""
+    if not isinstance(c, type):
+        return False
+    # should have only tuple as superclass
+    bases = c.__bases__
+    if len(bases) != 1 or bases[0] != tuple:
+        return False
+    # should have _make method
+    if not hasattr(c, '_make'):
+        return False
+    # should have _fields that is all string
+    fields = getattr(c, '_fields', None)
+    if not isinstance(fields, tuple):
+        return False
+    return all(isinstance(f, str) for f in fields)
