@@ -5,13 +5,12 @@ import contextlib
 import pickle
 
 from llvmlite import ir
-import llvmlite.binding as ll
-from llvmlite.llvmpy.core import Type, Constant, LLVMException
+from llvmlite.llvmpy.core import Type, Constant
 import llvmlite.llvmpy.core as lc
 
 from numba.config import PYVERSION
 import numba.ctypes_support as ctypes
-from numba import numpy_support
+from numba import config
 from numba import types, utils, cgutils, lowering, _helperlib
 
 
@@ -29,13 +28,13 @@ class _Registry(object):
             return func
         return decorator
 
-    def lookup(self, typeclass):
+    def lookup(self, typeclass, default=None):
         assert issubclass(typeclass, types.Type)
         for cls in typeclass.__mro__:
             func = self.functions.get(cls)
             if func is not None:
                 return func
-        return None
+        return default
 
 # Registries of boxing / unboxing implementations
 _boxers = _Registry()
@@ -746,21 +745,6 @@ class PythonAPI(object):
         fn = self._get_function(fnty, name="PyList_SetSlice")
         return self.builder.call(fn, (lst, start, stop, obj))
 
-    def list_get_private_data(self, lst):
-        fnty = Type.function(self.voidptr, [self.pyobj])
-        fn = self._get_function(fnty, name="numba_get_list_private_data")
-        return self.builder.call(fn, (lst,))
-
-    def list_set_private_data(self, lst, ptr):
-        fnty = Type.function(Type.void(), [self.pyobj, self.voidptr])
-        fn = self._get_function(fnty, name="numba_set_list_private_data")
-        return self.builder.call(fn, (lst, ptr))
-
-    def list_reset_private_data(self, lst):
-        fnty = Type.function(Type.void(), [self.pyobj])
-        fn = self._get_function(fnty, name="numba_reset_list_private_data")
-        return self.builder.call(fn, (lst,))
-
 
     #
     # Concrete tuple API
@@ -1293,7 +1277,7 @@ class PythonAPI(object):
         # First make the array constant
         data = pickle.dumps(obj, protocol=-1)
         assert len(data) < 2**31
-        name = ".const.pickledata.%s" % (id(obj))
+        name = ".const.pickledata.%s" % (id(obj) if config.DIFF_IR == 0 else "DIFF_IR")
         bdata = cgutils.make_bytearray(data)
         arr = self.context.insert_unique_const(self.module, name, bdata)
         # Then populate the structure constant
@@ -1313,7 +1297,7 @@ class PythonAPI(object):
             gv = self.module.__serialized[obj]
         except KeyError:
             struct = self.serialize_uncached(obj)
-            name = ".const.picklebuf.%s" % (id(obj))
+            name = ".const.picklebuf.%s" % (id(obj) if config.DIFF_IR == 0 else "DIFF_IR")
             gv = self.context.insert_unique_const(self.module, name, struct)
             # Make the id() (and hence the name) unique while populating the module.
             self.module.__serialized[obj] = gv
@@ -1327,10 +1311,9 @@ class PythonAPI(object):
         Unbox the Python object as the given Numba type.
         A NativeValue instance is returned.
         """
-        impl = _unboxers.lookup(typ.__class__)
-        if impl is None:
-            raise NotImplementedError("cannot convert %s to native value" % (typ,))
+        from numba.targets.boxing import unbox_unsupported
 
+        impl = _unboxers.lookup(typ.__class__, unbox_unsupported)
         c = _UnboxContext(self.context, self.builder, self)
         return impl(typ, obj, c)
 
@@ -1347,9 +1330,9 @@ class PythonAPI(object):
         pointer is returned (NULL if an error occurred).
         This method steals any native (NRT) reference embedded in *val*.
         """
-        impl = _boxers.lookup(typ.__class__)
-        if impl is None:
-            raise NotImplementedError("cannot convert native %s to Python object" % (typ,))
+        from numba.targets.boxing import box_unsupported
+
+        impl = _boxers.lookup(typ.__class__, box_unsupported)
 
         c = _BoxContext(self.context, self.builder, self, env_manager)
         return impl(typ, val, c)
