@@ -163,10 +163,13 @@ declmethod(MemInfo_alloc_safe_aligned);
 declmethod(MemInfo_alloc_dtor_safe);
 declmethod(MemInfo_call_dtor);
 declmethod(MemInfo_new_varsize);
+declmethod(MemInfo_new_varsize_dtor);
 declmethod(MemInfo_varsize_alloc);
 declmethod(MemInfo_varsize_free);
 declmethod(MemInfo_varsize_realloc);
 declmethod(MemInfo_release);
+declmethod(Allocate);
+declmethod(Free);
 
 
 #undef declmethod
