@@ -237,10 +237,11 @@ class MinimalCallConv(BaseCallConv):
         fnty = ir.FunctionType(errcode_t, [resptr] + argtypes)
         return fnty
 
-    def decorate_function(self, fn, args, fe_argtypes):
+    def decorate_function(self, fn, args, fe_argtypes, noalias=False):
         """
         Set names and attributes of function arguments.
         """
+        assert not noalias
         arginfo = self._get_arg_packer(fe_argtypes)
         arginfo.assign_names(self.get_arguments(fn),
                              ['arg.' + a for a in args])
@@ -253,11 +254,10 @@ class MinimalCallConv(BaseCallConv):
         """
         return func.args[1:]
 
-    def call_function(self, builder, callee, resty, argtys, args, env=None):
+    def call_function(self, builder, callee, resty, argtys, args):
         """
         Call the Numba-compiled *callee*.
         """
-        assert env is None
         retty = callee.args[0].type.pointee
         retvaltmp = cgutils.alloca_once(builder, retty)
         # initialize return value
@@ -305,7 +305,7 @@ class CPUCallConv(BaseCallConv):
     The calling convention for CPU targets.
     The implemented function signature is:
 
-        retcode_t (<Python return type>*, excinfo **, env *, ... <Python arguments>)
+        retcode_t (<Python return type>*, excinfo **, ... <Python arguments>)
 
     The return code will be one of the RETCODE_* constants.
     If RETCODE_USEREXC, the exception info pointer will be filled with
@@ -314,9 +314,6 @@ class CPUCallConv(BaseCallConv):
     Caller is responsible for allocating slots for the return value
     and the exception info pointer (passed as first and second arguments,
     respectively).
-
-    The third argument (env *) is a _dynfunc.Environment object, used
-    only for object mode functions.
     """
     _status_ids = itertools.count(1)
 
@@ -386,11 +383,11 @@ class CPUCallConv(BaseCallConv):
         argtypes = list(arginfo.argument_types)
         resptr = self.get_return_type(restype)
         fnty = ir.FunctionType(errcode_t,
-                               [resptr, ir.PointerType(excinfo_ptr_t), PYOBJECT]
+                               [resptr, ir.PointerType(excinfo_ptr_t)]
                                + argtypes)
         return fnty
 
-    def decorate_function(self, fn, args, fe_argtypes):
+    def decorate_function(self, fn, args, fe_argtypes, noalias=False):
         """
         Set names of function arguments, and add useful attributes to them.
         """
@@ -405,23 +402,20 @@ class CPUCallConv(BaseCallConv):
         excarg.name = "excinfo"
         excarg.add_attribute("nocapture")
         excarg.add_attribute("noalias")
-        envarg = self.get_env_argument(fn)
-        envarg.name = "env"
-        envarg.add_attribute("nocapture")
-        envarg.add_attribute("noalias")
+
+        if noalias:
+            args = self.get_arguments(fn)
+            for a in args:
+                if isinstance(a.type, ir.PointerType):
+                    a.add_attribute("nocapture")
+                    a.add_attribute("noalias")
         return fn
 
     def get_arguments(self, func):
         """
         Get the Python-level arguments of LLVM *func*.
         """
-        return func.args[3:]
-
-    def get_env_argument(self, func):
-        """
-        Get the environment argument of LLVM *func*.
-        """
-        return func.args[2]
+        return func.args[2:]
 
     def _get_return_argument(self, func):
         return func.args[0]
@@ -429,16 +423,14 @@ class CPUCallConv(BaseCallConv):
     def _get_excinfo_argument(self, func):
         return func.args[1]
 
-    def call_function(self, builder, callee, resty, argtys, args, env=None):
+    def call_function(self, builder, callee, resty, argtys, args):
         """
         Call the Numba-compiled *callee*.
         """
-        if env is None:
-            # This only works with functions that don't use the environment
-            # (nopython functions).
-            env = cgutils.get_null_value(PYOBJECT)
-        is_generator_function = isinstance(resty, types.Generator)
-        retty = self._get_return_argument(callee).type.pointee
+        # XXX better fix for callees that are not function values
+        #     (pointers to function; thus have no `.args` attribute)
+        retty = self._get_return_argument(callee.function_type).pointee
+
         retvaltmp = cgutils.alloca_once(builder, retty)
         # initialize return value to zeros
         builder.store(cgutils.get_null_value(retty), retvaltmp)
@@ -448,14 +440,11 @@ class CPUCallConv(BaseCallConv):
 
         arginfo = self._get_arg_packer(argtys)
         args = list(arginfo.as_arguments(builder, args))
-        realargs = [retvaltmp, excinfoptr, env] + args
+        realargs = [retvaltmp, excinfoptr] + args
         code = builder.call(callee, realargs)
         status = self._get_return_status(builder, code,
                                          builder.load(excinfoptr))
-        if is_generator_function:
-            retval = retvaltmp
-        else:
-            retval = builder.load(retvaltmp)
+        retval = builder.load(retvaltmp)
         out = self.context.get_returned_value(builder, resty, retval)
         return status, out
 
