@@ -551,6 +551,27 @@ def vdot(context, builder, sig, args):
         return dot_2_vv(context, builder, sig, args, conjugate=True)
 
 
+def dot_3_vm_check_args(a, b, out):
+    m, = a.shape
+    _m, n = b.shape
+    if m != _m:
+        raise ValueError("incompatible array sizes for "
+                         "np.dot(a, b) (vector * matrix)")
+    if out.shape != (n,):
+        raise ValueError("incompatible output array size for "
+                         "np.dot(a, b, out) (vector * matrix)")
+
+
+def dot_3_mv_check_args(a, b, out):
+    m, _n = a.shape
+    n, = b.shape
+    if n != _n:
+        raise ValueError("incompatible array sizes for np.dot(a, b) "
+                         "(matrix * vector)")
+    if out.shape != (m,):
+        raise ValueError("incompatible output array size for "
+                         "np.dot(a, b, out) (matrix * vector)")
+
 def dot_3_vm(context, builder, sig, args):
     """
     np.dot(vector, matrix, out)
@@ -573,16 +594,9 @@ def dot_3_vm(context, builder, sig, args):
         m_shapes = y_shapes
         do_trans = yty.layout == 'F'
         m_data, v_data = y.data, x.data
+        check_args = dot_3_vm_check_args
+
 
-        def check_args(a, b, out):
-            m, = a.shape
-            _m, n = b.shape
-            if m != _m:
-                raise ValueError("incompatible array sizes for "
-                                 "np.dot(a, b) (vector * matrix)")
-            if out.shape != (n,):
-                raise ValueError("incompatible output array size for "
-                                 "np.dot(a, b, out) (vector * matrix)")
     else:
         # Matrix * vector
         # We will compute x * y
@@ -590,16 +604,8 @@ def dot_3_vm(context, builder, sig, args):
         m_shapes = x_shapes
         do_trans = xty.layout == 'C'
         m_data, v_data = x.data, y.data
+        check_args = dot_3_mv_check_args
 
-        def check_args(a, b, out):
-            m, _n = a.shape
-            n, = b.shape
-            if n != _n:
-                raise ValueError("incompatible array sizes for np.dot(a, b) "
-                                 "(matrix * vector)")
-            if out.shape != (m,):
-                raise ValueError("incompatible output array size for "
-                                 "np.dot(a, b, out) (matrix * vector)")
 
     context.compile_internal(builder, check_args,
                              signature(types.none, *sig.args), args)
@@ -719,16 +725,25 @@ def _check_finite_matrix(a):
                 "Array must not contain infs or NaNs.")
 
 
-def _check_linalg_matrix(a, func_name):
+def _check_linalg_matrix(a, func_name, la_prefix=True):
+    # la_prefix is present as some functions, e.g. np.trace()
+    # are documented under "linear algebra" but aren't in the
+    # module
+    prefix = "np.linalg" if la_prefix else "np"
+    interp = (prefix, func_name)
+    # Unpack optional type
+    if isinstance(a, types.Optional):
+        a = a.type
     if not isinstance(a, types.Array):
-        raise TypingError("np.linalg.%s() only supported for array types"
-                          % func_name)
+        msg = "%s.%s() only supported for array types" % interp
+        raise TypingError(msg, highlighting=False)
     if not a.ndim == 2:
-        raise TypingError("np.linalg.%s() only supported on 2-D arrays."
-                          % func_name)
+        msg = "%s.%s() only supported on 2-D arrays." % interp
+        raise TypingError(msg, highlighting=False)
     if not isinstance(a.dtype, (types.Float, types.Complex)):
-        raise TypingError("np.linalg.%s() only supported on "
-                          "float and complex arrays." % func_name)
+        msg = "%s.%s() only supported on "\
+            "float and complex arrays." % interp
+        raise TypingError(msg, highlighting=False)
 
 
 def _check_homogeneous_types(func_name, *types):
@@ -736,7 +751,7 @@ def _check_homogeneous_types(func_name, *types):
     for t in types[1:]:
         if t.dtype != t0:
             msg = "np.linalg.%s() only supports inputs that have homogeneous dtypes." % func_name
-            raise TypingError(msg)
+            raise TypingError(msg, highlighting=False)
 
 
 @register_jitable
@@ -749,6 +764,11 @@ def _inv_err_handler(r):
             raise np.linalg.LinAlgError(
                 "Matrix is singular to machine precision.")
 
+@register_jitable
+def _dummy_liveness_func(a):
+    """pass a list of variables to be preserved through dead code elimination"""
+    return a[0]
+
 
 @overload(np.linalg.inv)
 def inv_impl(a):
@@ -777,6 +797,9 @@ def inv_impl(a):
         else:
             acpy = np.asfortranarray(a)
 
+        if n == 0:
+            return acpy
+
         ipiv = np.empty(n, dtype=F_INT_nptype)
 
         r = numba_xxgetrf(kind, n, n, acpy.ctypes, n, ipiv.ctypes)
@@ -786,8 +809,7 @@ def inv_impl(a):
         _inv_err_handler(r)
 
         # help liveness analysis
-        acpy.size
-        ipiv.size
+        _dummy_liveness_func([acpy.size, ipiv.size])
         return acpy
 
     return inv_impl
@@ -803,17 +825,22 @@ def _handle_err_maybe_convergence_problem(r):
             raise ValueError("Internal algorithm failed to converge.")
 
 
-def _check_linalg_1_or_2d_matrix(a, func_name):
+def _check_linalg_1_or_2d_matrix(a, func_name, la_prefix=True):
+    # la_prefix is present as some functions, e.g. np.trace()
+    # are documented under "linear algebra" but aren't in the
+    # module
+    prefix = "np.linalg" if la_prefix else "np"
+    interp = (prefix, func_name)
     # checks that a matrix is 1 or 2D
     if not isinstance(a, types.Array):
-        raise TypingError("np.linalg.%s() only supported for array types "
-                          % func_name)
+        raise TypingError("%s.%s() only supported for array types "
+                          % interp)
     if not a.ndim <= 2:
-        raise TypingError("np.linalg.%s() only supported on 1 and 2-D arrays "
-                          % func_name)
+        raise TypingError("%s.%s() only supported on 1 and 2-D arrays "
+                          % interp)
     if not isinstance(a.dtype, (types.Float, types.Complex)):
-        raise TypingError("np.linalg.%s() only supported on "
-                          "float and complex arrays." % func_name)
+        raise TypingError("%s.%s() only supported on "
+                          "float and complex arrays." % interp)
 
 if numpy_version >= (1, 8):
 
@@ -837,6 +864,10 @@ if numpy_version >= (1, 8):
 
             # The output is allocated in C order
             out = a.copy()
+
+            if n == 0:
+                return out
+
             # Pass UP since xxpotrf() operates in F order
             # The semantics ensure this works fine
             # (out is really its Hermitian in F order, but UP instructs
@@ -896,6 +927,9 @@ if numpy_version >= (1, 8):
             vl = np.empty((n, ldvl), dtype=a.dtype)
             vr = np.empty((n, ldvr), dtype=a.dtype)
 
+            if n == 0:
+                return (wr, vr.T)
+
             r = numba_ez_rgeev(kind,
                                JOBVL,
                                JOBVR,
@@ -926,11 +960,7 @@ if numpy_version >= (1, 8):
 
             # put these in to help with liveness analysis,
             # `.ctypes` doesn't keep the vars alive
-            acpy.size
-            vl.size
-            vr.size
-            wr.size
-            wi.size
+            _dummy_liveness_func([acpy.size, vl.size, vr.size, wr.size, wi.size])
             return (wr, vr.T)
 
         def cmplx_eig_impl(a):
@@ -955,6 +985,9 @@ if numpy_version >= (1, 8):
             vl = np.empty((n, ldvl), dtype=a.dtype)
             vr = np.empty((n, ldvr), dtype=a.dtype)
 
+            if n == 0:
+                return (w, vr.T)
+
             r = numba_ez_cgeev(kind,
                                JOBVL,
                                JOBVR,
@@ -970,10 +1003,7 @@ if numpy_version >= (1, 8):
 
             # put these in to help with liveness analysis,
             # `.ctypes` doesn't keep the vars alive
-            acpy.size
-            vl.size
-            vr.size
-            w.size
+            _dummy_liveness_func([acpy.size, vl.size, vr.size, w.size])
             return (w, vr.T)
 
         if isinstance(a.dtype, types.scalars.Complex):
@@ -1016,6 +1046,10 @@ if numpy_version >= (1, 8):
             ldvl = 1
             ldvr = 1
             wr = np.empty(n, dtype=a.dtype)
+
+            if n == 0:
+                return wr
+
             wi = np.empty(n, dtype=a.dtype)
 
             # not referenced but need setting for MKL null check
@@ -1052,11 +1086,7 @@ if numpy_version >= (1, 8):
 
             # put these in to help with liveness analysis,
             # `.ctypes` doesn't keep the vars alive
-            acpy.size
-            vl.size
-            vr.size
-            wr.size
-            wi.size
+            _dummy_liveness_func([acpy.size, vl.size, vr.size, wr.size, wi.size])
             return wr
 
         def cmplx_eigvals_impl(a):
@@ -1078,6 +1108,10 @@ if numpy_version >= (1, 8):
             ldvl = 1
             ldvr = 1
             w = np.empty(n, dtype=a.dtype)
+
+            if n == 0:
+                return w
+
             vl = np.empty((1), dtype=a.dtype)
             vr = np.empty((1), dtype=a.dtype)
 
@@ -1096,10 +1130,7 @@ if numpy_version >= (1, 8):
 
             # put these in to help with liveness analysis,
             # `.ctypes` doesn't keep the vars alive
-            acpy.size
-            vl.size
-            vr.size
-            w.size
+            _dummy_liveness_func([acpy.size, vl.size, vr.size, w.size])
             return w
 
         if isinstance(a.dtype, types.scalars.Complex):
@@ -1142,6 +1173,9 @@ if numpy_version >= (1, 8):
 
             w = np.empty(n, dtype=w_dtype)
 
+            if n == 0:
+                return (w, acpy)
+
             r = numba_ez_xxxevd(kind,  # kind
                                 JOBZ,  # jobz
                                 UPLO,  # uplo
@@ -1153,9 +1187,7 @@ if numpy_version >= (1, 8):
             _handle_err_maybe_convergence_problem(r)
 
             # help liveness analysis
-            acpy.size
-            w.size
-
+            _dummy_liveness_func([acpy.size, w.size])
             return (w, acpy)
 
         return eigh_impl
@@ -1195,6 +1227,9 @@ if numpy_version >= (1, 8):
 
             w = np.empty(n, dtype=w_dtype)
 
+            if n == 0:
+                return w
+
             r = numba_ez_xxxevd(kind,  # kind
                                 JOBZ,  # jobz
                                 UPLO,  # uplo
@@ -1206,9 +1241,7 @@ if numpy_version >= (1, 8):
             _handle_err_maybe_convergence_problem(r)
 
             # help liveness analysis
-            acpy.size
-            w.size
-
+            _dummy_liveness_func([acpy.size, w.size])
             return w
 
         return eigvalsh_impl
@@ -1236,6 +1269,9 @@ if numpy_version >= (1, 8):
             n = a.shape[-1]
             m = a.shape[-2]
 
+            if n == 0 or m == 0:
+                raise np.linalg.LinAlgError("Arrays cannot be empty")
+
             _check_finite_matrix(a)
 
             if F_layout:
@@ -1275,11 +1311,7 @@ if numpy_version >= (1, 8):
             _handle_err_maybe_convergence_problem(r)
 
             # help liveness analysis
-            acpy.size
-            vt.size
-            u.size
-            s.size
-
+            _dummy_liveness_func([acpy.size, vt.size, u.size, s.size])
             return (u.T, s, vt.T)
 
         return svd_impl
@@ -1308,6 +1340,9 @@ def qr_impl(a):
         n = a.shape[-1]
         m = a.shape[-2]
 
+        if n == 0 or m == 0:
+            raise np.linalg.LinAlgError("Arrays cannot be empty")
+
         _check_finite_matrix(a)
 
         # copy A as it will be destroyed
@@ -1358,9 +1393,7 @@ def qr_impl(a):
         _handle_err_maybe_convergence_problem(ret)
 
         # help liveness analysis
-        tau.size
-        q.size
-
+        _dummy_liveness_func([tau.size, q.size])
         return (q[:, :minmn], r)
 
     return qr_impl
@@ -1436,6 +1469,35 @@ def _system_check_dimensionally_valid_impl(a, b):
         return twoD_impl
 
 
+def _system_check_non_empty(a, b):
+    """
+    Check that AX=B style system input is not empty.
+    """
+    raise NotImplementedError
+
+
+@overload(_system_check_non_empty)
+def _system_check_non_empty_impl(a, b):
+    ndim = b.ndim
+    if ndim == 1:
+        def oneD_impl(a, b):
+            am = a.shape[-2]
+            an = a.shape[-1]
+            bm = b.shape[-1]
+            if am == 0 or bm == 0 or an == 0:
+                raise np.linalg.LinAlgError('Arrays cannot be empty')
+        return oneD_impl
+    else:
+        def twoD_impl(a, b):
+            am = a.shape[-2]
+            an = a.shape[-1]
+            bm = b.shape[-2]
+            bn = b.shape[-1]
+            if am == 0 or bm == 0 or an == 0 or bn == 0:
+                raise np.linalg.LinAlgError('Arrays cannot be empty')
+        return twoD_impl
+
+
 def _lstsq_residual(b, n, rhs):
     """
     Compute the residual from the 'b' scratch space.
@@ -1541,7 +1603,10 @@ def lstsq_impl(a, b, rcond=-1.0):
         _check_finite_matrix(a)
         _check_finite_matrix(b)
 
-        # check the systems is dimensionally valid
+        # check the system is not empty
+        _system_check_non_empty(a, b)
+
+        # check the systems are dimensionally valid
         _system_check_dimensionally_valid(a, b)
 
         minmn = min(m, n)
@@ -1592,11 +1657,7 @@ def lstsq_impl(a, b, rcond=-1.0):
         x = _lstsq_solution(b, bcpy, n)
 
         # help liveness analysis
-        acpy.size
-        bcpy.size
-        s.size
-        rank_ptr.size
-
+        _dummy_liveness_func([acpy.size, bcpy.size, s.size, rank_ptr.size])
         return (x, res, rank, s[:minmn])
 
     return lstsq_impl
@@ -1661,6 +1722,9 @@ def solve_impl(a, b):
 
         # b is overwritten on exit with the solution, copy allocate
         bcpy = np.empty((nrhs, n), dtype=np_dt).T
+        if n == 0:
+            return _solve_compute_return(b, bcpy)
+
         # specialised copy in due to b being 1 or 2D
         _system_copy_in_b(bcpy, b, nrhs)
 
@@ -1680,10 +1744,7 @@ def solve_impl(a, b):
         _inv_err_handler(r)
 
         # help liveness analysis
-        acpy.size
-        bcpy.size
-        ipiv.size
-
+        _dummy_liveness_func([acpy.size, bcpy.size, ipiv.size])
         return _solve_compute_return(b, bcpy)
 
     return solve_impl
@@ -1766,6 +1827,9 @@ def pinv_impl(a, rcond=1.e-15):
         else:
             acpy = np.asfortranarray(a)
 
+        if m == 0 or n == 0:
+            return acpy.T.ravel().reshape(a.shape).T
+
         minmn = min(m, n)
 
         u = np.empty((minmn, m), dtype=a.dtype)
@@ -1842,13 +1906,14 @@ def pinv_impl(a, rcond=1.e-15):
         )
 
         # help liveness analysis
-        acpy.size
-        vt.size
-        u.size
-        s.size
-        one.size
-        zero.size
-
+        #acpy.size
+        #vt.size
+        #u.size
+        #s.size
+        #one.size
+        #zero.size
+        _dummy_liveness_func([acpy.size, vt.size, u.size, s.size, one.size,
+            zero.size])
         return acpy.T.ravel().reshape(a.shape).T
 
     return pinv_impl
@@ -1904,12 +1969,18 @@ def slogdet_impl(a):
 
     diag_walker = _get_slogdet_diag_walker(a)
 
+    ONE = a.dtype(1)
+    ZERO = getattr(a.dtype, "underlying_float", a.dtype)(0)
+
     def slogdet_impl(a):
         n = a.shape[-1]
         if a.shape[-2] != n:
             msg = "Last 2 dimensions of the array must be square."
             raise np.linalg.LinAlgError(msg)
 
+        if n == 0:
+            return (ONE, ZERO)
+
         _check_finite_matrix(a)
 
         if F_layout:
@@ -1942,7 +2013,7 @@ def slogdet_impl(a):
             sgn = -1
 
         # help liveness analysis
-        ipiv.size
+        _dummy_liveness_func([ipiv.size])
         return diag_walker(n, acpy, sgn)
 
     return slogdet_impl
@@ -1985,7 +2056,7 @@ def _compute_singular_values_impl(a):
     np_ret_type = np_support.as_dtype(nb_ret_type)
     np_dtype = np_support.as_dtype(a.dtype)
 
-    # This are not referenced in the computation but must be set
+    # These are not referenced in the computation but must be set
     # for MKL.
     u = np.empty((1, 1), dtype=np_dtype)
     vt = np.empty((1, 1), dtype=np_dtype)
@@ -2000,9 +2071,12 @@ def _compute_singular_values_impl(a):
         # call LAPACK to shortcut doing the "reconstruct
         # singular vectors from reflectors" step and just
         # get back the singular values.
-        _check_finite_matrix(a)
         n = a.shape[-1]
         m = a.shape[-2]
+        if m == 0 or n == 0:
+            raise np.linalg.LinAlgError('Arrays cannot be empty')
+        _check_finite_matrix(a)
+
         ldu = m
         minmn = min(m, n)
 
@@ -2010,8 +2084,6 @@ def _compute_singular_values_impl(a):
         ucol = 1
         ldvt = 1
 
-        _check_finite_matrix(a)
-
         if F_layout:
             acpy = np.copy(a)
         else:
@@ -2038,11 +2110,7 @@ def _compute_singular_values_impl(a):
         _handle_err_maybe_convergence_problem(r)
 
         # help liveness analysis
-        acpy.size
-        vt.size
-        u.size
-        s.size
-
+        _dummy_liveness_func([acpy.size, vt.size, u.size, s.size])
         return s
 
     return sv_function
@@ -2083,9 +2151,9 @@ def _oneD_norm_2_impl(a):
             assert 0   # unreachable
 
         # help liveness analysis
-        ret.size
-        a.size
-
+        #ret.size
+        #a.size
+        _dummy_liveness_func([ret.size, a.size])
         return ret[0]
 
     return impl
@@ -2451,6 +2519,14 @@ def matrix_power_impl(a, n):
                 A[k, k] = 1.
             return A
 
+        am, an = a.shape[-1], a.shape[-2]
+        if am != an:
+            raise ValueError('input must be a square array')
+
+        # empty, return a copy
+        if am == 0:
+            return a.copy()
+
         # note: to be consistent over contiguousness, C order is
         # returned as that is what dot() produces and the most common
         # paths through matrix_power will involve that. Therefore
@@ -2493,3 +2569,227 @@ def matrix_power_impl(a, n):
             return ret
 
     return matrix_power_impl
+
+# This is documented under linalg despite not being in the module
+
+
+@overload(np.trace)
+def matrix_trace_impl(a, offset=types.int_):
+    """
+    Computes the trace of an array.
+    """
+
+    _check_linalg_matrix(a, "trace", la_prefix=False)
+
+    if not isinstance(offset, types.Integer):
+        raise TypeError("integer argument expected, got %s" % offset)
+
+    def matrix_trace_impl(a, offset=0):
+        rows, cols = a.shape
+        k = offset
+        if k < 0:
+            rows = rows + k
+        if k > 0:
+            cols = cols - k
+        n = max(min(rows, cols), 0)
+        ret = 0
+        if k >= 0:
+            for i in range(n):
+                ret += a[i, k + i]
+        else:
+            for i in range(n):
+                ret += a[i - k, i]
+        return ret
+
+    return matrix_trace_impl
+
+
+def _check_scalar_or_lt_2d_mat(a, func_name, la_prefix=True):
+    prefix = "np.linalg" if la_prefix else "np"
+    interp = (prefix, func_name)
+    # checks that a matrix is 1 or 2D
+    if isinstance(a, types.Array):
+        if not a.ndim <= 2:
+            raise TypingError("%s.%s() only supported on 1 and 2-D arrays "
+                              % interp, highlighting=False)
+
+
+def _get_as_array(x):
+    if not isinstance(x, types.Array):
+        @register_jitable
+        def asarray(x):
+            return np.array((x,))
+        return asarray
+    else:
+        @register_jitable
+        def asarray(x):
+            return x
+        return asarray
+
+
+def _get_outer_impl(a, b, out):
+    a_arr = _get_as_array(a)
+    b_arr = _get_as_array(b)
+
+    if out in (None, types.none):
+        @register_jitable
+        def outer_impl(a, b, out):
+            aa = a_arr(a)
+            bb = b_arr(b)
+            return np.multiply(aa.ravel().reshape((aa.size, 1)),
+                               bb.ravel().reshape((1, bb.size)))
+        return outer_impl
+    else:
+        @register_jitable
+        def outer_impl(a, b, out):
+            aa = a_arr(a)
+            bb = b_arr(b)
+            np.multiply(aa.ravel().reshape((aa.size, 1)),
+                        bb.ravel().reshape((1, bb.size)),
+                        out)
+            return out
+        return outer_impl
+
+
+if numpy_version >= (1, 9):
+    @overload(np.outer)
+    def outer_impl(a, b, out=None):
+
+        _check_scalar_or_lt_2d_mat(a, "outer", la_prefix=False)
+        _check_scalar_or_lt_2d_mat(b, "outer", la_prefix=False)
+
+        impl = _get_outer_impl(a, b, out)
+
+        def outer_impl(a, b, out=None):
+            return impl(a, b, out)
+
+        return outer_impl
+else:
+    @overload(np.outer)
+    def outer_impl(a, b):
+
+        _check_scalar_or_lt_2d_mat(a, "outer", la_prefix=False)
+        _check_scalar_or_lt_2d_mat(b, "outer", la_prefix=False)
+
+        impl = _get_outer_impl(a, b, None)
+
+        def outer_impl(a, b):
+            return impl(a, b, None)
+
+        return outer_impl
+
+
+def _kron_normaliser_impl(x):
+    # makes x into a 2d array
+    if isinstance(x, types.Array):
+        if x.ndim == 2:
+            @register_jitable
+            def nrm_shape(x):
+                xn = x.shape[-1]
+                xm = x.shape[-2]
+                return x.reshape(xm, xn)
+            return nrm_shape
+        else:
+            @register_jitable
+            def nrm_shape(x):
+                xn = x.shape[-1]
+                return x.reshape(1, xn)
+            return nrm_shape
+    else:  # assume its a scalar
+        @register_jitable
+        def nrm_shape(x):
+            a = np.empty((1, 1), type(x))
+            a[0] = x
+            return a
+        return nrm_shape
+
+
+def _kron_return(a, b):
+    # transforms c into something that kron would return
+    # based on the shapes of a and b
+    a_is_arr = isinstance(a, types.Array)
+    b_is_arr = isinstance(b, types.Array)
+    if a_is_arr and b_is_arr:
+        if a.ndim == 2 or b.ndim == 2:
+            @register_jitable
+            def ret(a, b, c):
+                return c
+            return ret
+        else:
+            @register_jitable
+            def ret(a, b, c):
+                return c.reshape(c.size)
+            return ret
+    else:  # at least one of (a, b) is a scalar
+        if a_is_arr:
+            @register_jitable
+            def ret(a, b, c):
+                return c.reshape(a.shape)
+            return ret
+        elif b_is_arr:
+            @register_jitable
+            def ret(a, b, c):
+                return c.reshape(b.shape)
+            return ret
+        else:  # both scalars
+            @register_jitable
+            def ret(a, b, c):
+                return c[0]
+            return ret
+
+
+@overload(np.kron)
+def kron_impl(a, b):
+
+    _check_scalar_or_lt_2d_mat(a, "kron", la_prefix=False)
+    _check_scalar_or_lt_2d_mat(b, "kron", la_prefix=False)
+
+    fix_a = _kron_normaliser_impl(a)
+    fix_b = _kron_normaliser_impl(b)
+    ret_c = _kron_return(a, b)
+
+    # this is fine because the ufunc for the Hadamard product
+    # will reject differing dtypes in a and b.
+    dt = getattr(a, 'dtype', a)
+
+    def kron_impl(a, b):
+
+        aa = fix_a(a)
+        bb = fix_b(b)
+
+        am = aa.shape[-2]
+        an = aa.shape[-1]
+        bm = bb.shape[-2]
+        bn = bb.shape[-1]
+
+        cm = am * bm
+        cn = an * bn
+
+        # allocate c
+        C = np.empty((cm, cn), dtype=dt)
+
+        # In practice this is runs quicker than the more obvious
+        # `each element of A multiplied by B and assigned to
+        # a block in C` like alg.
+
+        # loop over rows of A
+        for i in range(am):
+            # compute the column offset into C
+            rjmp = i * bm
+            # loop over rows of B
+            for k in range(bm):
+                # compute row the offset into C
+                irjmp = rjmp + k
+                # slice a given row of B
+                slc = bb[k, :]
+                # loop over columns of A
+                for j in range(an):
+                    # vectorized assignment of an element of A
+                    # multiplied by the current row of B into
+                    # a slice of a row of C
+                    cjmp = j * bn
+                    C[irjmp, cjmp:cjmp + bn] = aa[i, j] * slc
+
+        return ret_c(a, b, C)
+
+    return kron_impl
