@@ -2,11 +2,13 @@
 Implementation of operations on numpy timedelta64.
 """
 
+import numpy as np
+
 from llvmlite.llvmpy.core import Type, Constant
 import llvmlite.llvmpy.core as lc
 
 from numba import npdatetime, types, cgutils
-from numba.targets.imputils import lower_builtin, impl_ret_untracked
+from .imputils import lower_builtin, lower_constant, impl_ret_untracked
 
 
 # datetime64 and timedelta64 use the same internal representation
@@ -49,7 +51,7 @@ def scale_timedelta(context, builder, val, srcty, destty):
 def normalize_timedeltas(context, builder, left, right, leftty, rightty):
     """
     Scale either *left* or *right* to the other's unit, in order to have
-    homogenous units.
+    homogeneous units.
     """
     factor = npdatetime.get_timedelta_conversion_factor(leftty.unit, rightty.unit)
     if factor is not None:
@@ -105,6 +107,13 @@ normal_year_months_acc = make_constant_array(
 leap_year_months_acc = make_constant_array(
     [0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335])
 
+
+@lower_constant(types.NPDatetime)
+@lower_constant(types.NPTimedelta)
+def datetime_constant(context, builder, ty, pyval):
+    return DATETIME64(pyval.astype(np.int64))
+
+
 # Arithmetic operators on timedelta64
 
 @lower_builtin('+', types.NPTimedelta)
