@@ -1,5 +1,5 @@
 """
-Support for native homogenous sets.
+Support for native homogeneous sets.
 """
 
 from __future__ import print_function, absolute_import, division
@@ -167,7 +167,7 @@ class _SetPayload(object):
                                                ref=entry_ptr)
         return entry
 
-    def _lookup(self, item, h):
+    def _lookup(self, item, h, for_insert=False):
         """
         Lookup the *item* with the given hash values in the entries.
 
@@ -175,7 +175,7 @@ class _SetPayload(object):
         - If found is true, <entry index> points to the entry containing
           the item.
         - If found is false, <entry index> points to the empty entry that
-          the item can be written to.
+          the item can be written to (only if *for_insert* is true)
         """
         context = self._context
         builder = self._builder
@@ -195,6 +195,10 @@ class _SetPayload(object):
         # The index of the entry being considered: start with (hash & mask)
         index = cgutils.alloca_once_value(builder,
                                           builder.and_(h, mask))
+        if for_insert:
+            # The index of the first deleted entry in the lookup chain
+            free_index_sentinel = mask.type(-1)  # highest unsigned index
+            free_index = cgutils.alloca_once_value(builder, free_index_sentinel)
 
         bb_body = builder.append_basic_block("lookup.body")
         bb_found = builder.append_basic_block("lookup.found")
@@ -218,6 +222,14 @@ class _SetPayload(object):
             with builder.if_then(is_hash_empty(context, builder, entry_hash)):
                 builder.branch(bb_not_found)
 
+            if for_insert:
+                # Memorize the index of the first deleted entry
+                with builder.if_then(is_hash_deleted(context, builder, entry_hash)):
+                    j = builder.load(free_index)
+                    j = builder.select(builder.icmp_unsigned('==', j, free_index_sentinel),
+                                       i, j)
+                    builder.store(j, free_index)
+
         # First linear probing.  When the number of collisions is small,
         # the lineary probing loop achieves better cache locality and
         # is also slightly cheaper computationally.
@@ -249,6 +261,15 @@ class _SetPayload(object):
             builder.branch(bb_body)
 
         with builder.goto_block(bb_not_found):
+            if for_insert:
+                # Not found => for insertion, return the index of the first
+                # deleted entry (if any), to avoid creating an infinite
+                # lookup chain (issue #1913).
+                i = builder.load(index)
+                j = builder.load(free_index)
+                i = builder.select(builder.icmp_unsigned('==', j, free_index_sentinel),
+                                   i, j)
+                builder.store(i, index)
             builder.branch(bb_end)
 
         with builder.goto_block(bb_found):
@@ -345,7 +366,7 @@ class SetInstance(object):
         context = self._context
         builder = self._builder
 
-        ptr = self._context.nrt_meminfo_data(builder, self.meminfo)
+        ptr = self._context.nrt.meminfo_data(builder, self.meminfo)
         return _SetPayload(context, builder, self._ty, ptr)
 
     @property
@@ -398,7 +419,7 @@ class SetInstance(object):
         context = self._context
         builder = self._builder
 
-        found, i = payload._lookup(item, h)
+        found, i = payload._lookup(item, h, for_insert=True)
         not_found = builder.not_(found)
 
         with builder.if_then(not_found):
@@ -579,7 +600,7 @@ class SetInstance(object):
             h = loop.entry.hash
             # We must reload our payload as it may be resized during the loop
             payload = self.payload
-            found, i = payload._lookup(key, h)
+            found, i = payload._lookup(key, h, for_insert=True)
             entry = payload.get_entry(i)
             with builder.if_else(found) as (if_common, if_not_common):
                 with if_common:
@@ -716,7 +737,7 @@ class SetInstance(object):
         self = cls(context, builder, set_type, None)
         self._set.meminfo = meminfo
         self._set.parent = context.get_constant_null(types.pyobject)
-        context.nrt_incref(builder, set_type, self.value)
+        context.nrt.incref(builder, set_type, self.value)
         # Payload is part of the meminfo, no need to touch it
         return self
 
@@ -933,11 +954,11 @@ class SetInstance(object):
         with builder.if_then(builder.load(ok), likely=True):
             if realloc:
                 meminfo = self._set.meminfo
-                ptr = context.nrt_meminfo_varsize_alloc(builder, meminfo,
+                ptr = context.nrt.meminfo_varsize_alloc(builder, meminfo,
                                                         size=allocsize)
                 alloc_ok = cgutils.is_null(builder, ptr)
             else:
-                meminfo = context.nrt_meminfo_new_varsize(builder, size=allocsize)
+                meminfo = context.nrt.meminfo_new_varsize(builder, size=allocsize)
                 alloc_ok = cgutils.is_null(builder, meminfo)
 
             with builder.if_else(cgutils.is_null(builder, meminfo),
@@ -968,7 +989,7 @@ class SetInstance(object):
         """
         Free an allocated old payload at *ptr*.
         """
-        self._context.nrt_meminfo_varsize_free(self._builder, self.meminfo, ptr)
+        self._context.nrt.meminfo_varsize_free(self._builder, self.meminfo, ptr)
 
     def _copy_payload(self, src_payload):
         """
@@ -1000,7 +1021,7 @@ class SetInstance(object):
                                             nentries))
 
         with builder.if_then(builder.load(ok), likely=True):
-            meminfo = context.nrt_meminfo_new_varsize(builder, size=allocsize)
+            meminfo = context.nrt.meminfo_new_varsize(builder, size=allocsize)
             alloc_ok = cgutils.is_null(builder, meminfo)
 
             with builder.if_else(cgutils.is_null(builder, meminfo),
@@ -1033,7 +1054,7 @@ class SetIterInstance(object):
         self._builder = builder
         self._ty = iter_type
         self._iter = context.make_helper(builder, iter_type, iter_val)
-        ptr = self._context.nrt_meminfo_data(builder, self.meminfo)
+        ptr = self._context.nrt.meminfo_data(builder, self.meminfo)
         self._payload = _SetPayload(context, builder, self._ty.container, ptr)
 
     @classmethod
