@@ -1,6 +1,6 @@
 from __future__ import division
 
-from itertools import product, cycle
+from itertools import product, cycle, permutations
 import sys
 
 import numpy as np
@@ -12,6 +12,7 @@ from numba.errors import TypingError, LoweringError
 from numba.numpy_support import (as_dtype, strict_ufunc_typing,
                                  version as numpy_version)
 from .support import TestCase, CompilationCache, MemoryLeak, MemoryLeakMixin, tag
+from .matmul_usecase import needs_blas
 
 
 def np_around_array(arr, decimals, out):
@@ -171,6 +172,26 @@ def array_cumsum_kws(a, axis):
     return a.cumsum(axis=axis)
 
 
+def array_real(a):
+    return np.real(a)
+
+
+def array_imag(a):
+    return np.imag(a)
+
+
+def np_unique(a):
+    return np.unique(a)
+
+
+def array_dot(a, b):
+    return a.dot(b)
+
+
+def array_dot_chain(a, b):
+    return a.dot(b).dot(b)
+
+
 class TestArrayMethods(MemoryLeakMixin, TestCase):
     """
     Test various array methods and array-related functions.
@@ -456,6 +477,7 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         arr = np.array([0]).reshape(())
         check_arr(arr)
 
+
     def test_array_transpose(self):
         self.check_layout_dependent_func(array_transpose)
 
@@ -732,7 +754,6 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         with self.assertRaises(TypingError):
             cfunc(a, axis=1)
 
-
     def test_take(self):
         pyfunc = array_take
         cfunc = jit(nopython=True)(pyfunc)
@@ -798,7 +819,6 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         #exceptions leak refs
         self.disable_leak_check()
 
-
     def test_fill(self):
         pyfunc = array_fill
         cfunc = jit(nopython=True)(pyfunc)
@@ -827,6 +847,57 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
         for x in [np.float64, np.complex128, np.bool_]:
             check(A, x(10))
 
+    def test_real(self):
+        pyfunc = array_real
+        cfunc = jit(nopython=True)(pyfunc)
+
+        x = np.linspace(-10, 10)
+        np.testing.assert_equal(pyfunc(x), cfunc(x))
+
+        x, y = np.meshgrid(x, x)
+        z = x + 1j*y
+        np.testing.assert_equal(pyfunc(z), cfunc(z))
+
+    def test_imag(self):
+        pyfunc = array_imag
+        cfunc = jit(nopython=True)(pyfunc)
+
+        x = np.linspace(-10, 10)
+        np.testing.assert_equal(pyfunc(x), cfunc(x))
+
+        x, y = np.meshgrid(x, x)
+        z = x + 1j*y
+        np.testing.assert_equal(pyfunc(z), cfunc(z))
+
+    def test_unique(self):
+        pyfunc = np_unique
+        cfunc = jit(nopython=True)(pyfunc)
+
+        def check(a):
+            np.testing.assert_equal(pyfunc(a), cfunc(a))
+
+        check(np.array([[1, 1, 3], [3, 4, 5]]))
+        check(np.array(np.zeros(5)))
+        check(np.array([[3.1, 3.1], [1.7, 2.29], [3.3, 1.7]]))
+        check(np.array([]))
+
+    @needs_blas
+    def test_array_dot(self):
+        # just ensure that the dot impl dispatches correctly, do
+        # not test dot itself, this is done in test_linalg.
+        pyfunc = array_dot
+        cfunc = jit(nopython=True)(pyfunc)
+        a = np.arange(20.).reshape(4, 5)
+        b = np.arange(5.)
+        np.testing.assert_equal(pyfunc(a, b), cfunc(a, b))
+
+        # check that chaining works
+        pyfunc = array_dot_chain
+        cfunc = jit(nopython=True)(pyfunc)
+        a = np.arange(16.).reshape(4, 4)
+        np.testing.assert_equal(pyfunc(a, a), cfunc(a, a))
+
+
 class TestArrayComparisons(TestCase):
 
     def test_identity(self):
