@@ -20,6 +20,9 @@ forceobj_flags.set("force_pyobject")
 
 no_pyobj_flags = Flags()
 
+nrt_no_pyobj_flags = Flags()
+nrt_no_pyobj_flags.set("nrt")
+
 
 def abs_usecase(x):
     return abs(x)
@@ -182,6 +185,17 @@ def zip_3_usecase():
     return result
 
 
+def zip_first_exhausted():
+    iterable = range(7)
+    n = 3
+    it = iter(iterable)
+    # 1st iterator is shorter
+    front = list(zip(range(n), it))
+    # Make sure that we didn't skip one in `it`
+    back = list(it)
+    return front, back
+
+
 def pow_op_usecase(x, y):
     return x ** y
 
@@ -681,7 +695,7 @@ class TestBuiltins(TestCase):
         cfunc(1, [1])
 
     def test_max_1_invalid_types(self):
-        # Heterogenous ordering is valid in Python 2
+        # Heterogeneous ordering is valid in Python 2
         if utils.IS_PY3:
             with self.assertRaises(TypeError):
                 self.check_min_max_invalid_types(max_usecase1)
@@ -693,7 +707,7 @@ class TestBuiltins(TestCase):
             self.check_min_max_invalid_types(max_usecase1, flags=no_pyobj_flags)
 
     def test_min_1_invalid_types(self):
-        # Heterogenous ordering is valid in Python 2
+        # Heterogeneous ordering is valid in Python 2
         if utils.IS_PY3:
             with self.assertRaises(TypeError):
                 self.check_min_max_invalid_types(min_usecase1)
@@ -903,6 +917,17 @@ class TestBuiltins(TestCase):
     def test_zip_0_npm(self):
         self.test_zip_0(flags=no_pyobj_flags)
 
+    def test_zip_first_exhausted(self, flags=forceobj_flags):
+        """
+        Test side effect to the input iterators when a left iterator has been
+        exhausted before the ones on the right.
+        """
+        self.run_nullary_func(zip_first_exhausted, flags)
+
+    @tag('important')
+    def test_zip_first_exhausted_npm(self):
+        self.test_zip_first_exhausted(flags=nrt_no_pyobj_flags)
+
     def test_pow_op_usecase(self):
         args = [
             (2, 3),
