@@ -45,6 +45,13 @@ class TestCFFI(TestCase):
     def test_sin_function(self):
         self._test_function(mod.use_cffi_sin)
 
+    def test_bool_function_ool(self):
+        pyfunc = mod.use_cffi_boolean_true
+        cres = compile_isolated(pyfunc, (), flags=no_pyobj_flags)
+        cfunc = cres.entry_point
+        self.assertEqual(pyfunc(), True)
+        self.assertEqual(cfunc(), True)
+
     @tag('important')
     def test_sin_function_npm(self):
         self._test_function(mod.use_cffi_sin, flags=no_pyobj_flags)
@@ -117,7 +124,6 @@ class TestCFFI(TestCase):
         imag_cfunc(x, y)
         np.testing.assert_equal(x.imag, y)
 
-
     @unittest.skipIf(sys.version_info < (3,),
                      "buffer protocol on array.array needs Python 3+")
     def test_from_buffer_pyarray(self):
@@ -138,6 +144,28 @@ class TestCFFI(TestCase):
         self.assertIn("from_buffer() unsupported on non-contiguous buffers",
                       str(raises.exception))
 
+    def test_indirect_multiple_use(self):
+        """
+        Issue #2263
+
+        Linkage error due to multiple definition of global tracking symbol.
+        """
+        my_sin = mod.cffi_sin
+
+        # Use two jit functions that references `my_sin` to ensure multiple
+        # modules
+        @jit(nopython=True)
+        def inner(x):
+            return my_sin(x)
+
+        @jit(nopython=True)
+        def foo(x):
+            return inner(x) + my_sin(x + 1)
+
+        # Error occurs when foo is being compiled
+        x = 1.123
+        self.assertEqual(foo(x), my_sin(x) + my_sin(x + 1))
+
 
 if __name__ == '__main__':
     unittest.main()
