@@ -1,7 +1,10 @@
 from __future__ import print_function
+
 import numpy as np
-import numba.unittest_support as unittest
+
 import numba
+from .support import TestCase
+from numba.six import exec_
 
 
 class Issue455(object):
@@ -20,7 +23,7 @@ class Issue455(object):
                 x[i] = 1.
         """
         d = {}
-        exec(code.strip(), d)
+        exec_(code.strip(), d)
         self.f.append(numba.jit("void(f8[:])", nopython=True)(d['f']))
 
     def call_f(self):
@@ -30,12 +33,13 @@ class Issue455(object):
         return a
 
 
-class TestDynFunc(unittest.TestCase):
+class TestDynFunc(TestCase):
+
     def test_issue_455(self):
         inst = Issue455()
         inst.create_f()
         a = inst.call_f()
-        self.assertTrue(np.all(a == np.ones(a.size)))
+        self.assertPreciseEqual(a, np.ones_like(a))
 
 
 if __name__ == '__main__':
