@@ -1,33 +1,47 @@
 from __future__ import print_function, division, absolute_import
 
-from collections import namedtuple
 import math
 import sys
 import pickle
+import multiprocessing
+import ctypes
+from distutils.version import LooseVersion
+import re
 
 import numpy as np
 
 from numba import unittest_support as unittest
-from numba import jit, types, errors, typeof, numpy_support, cgutils
+from numba import jit, types, errors, typing
 from numba.compiler import compile_isolated
-from .support import TestCase, captured_stdout, tag
+from .support import (TestCase, captured_stdout, tag, temp_directory,
+                      override_config)
 
 from numba.extending import (typeof_impl, type_callable,
                              lower_builtin, lower_cast,
                              overload, overload_attribute,
+                             overload_method,
                              models, register_model,
                              box, unbox, NativeValue,
                              make_attribute_wrapper,
                              intrinsic, _Intrinsic,
                              register_jitable,
+                             get_cython_function_address
                              )
 from numba.typing.templates import (
     ConcreteTemplate, signature, infer)
-from numba.targets.imputils import impl_ret_borrowed
 
 # Pandas-like API implementation
 from .pdlike_usecase import Index, Series
 
+try:
+    import scipy
+    if LooseVersion(scipy.__version__) < '0.19':
+        sc = None
+    else:
+        import scipy.special.cython_special as sc
+except ImportError:
+    sc = None
+
 
 # -----------------------------------------------------------------------
 # Define a custom type and an implicit cast on it
@@ -208,6 +222,17 @@ def overload_len_dummy(arg):
         return len_impl
 
 
+@overload_method(MyDummyType, 'length')
+def overload_method_length(arg):
+    def imp(arg):
+        return len(arg)
+    return imp
+
+
+def cache_overload_method_usecase(x):
+    return x.length()
+
+
 def call_func1_nullary():
     return func1()
 
@@ -454,13 +479,70 @@ class TestHighLevelExtending(TestCase):
         np.testing.assert_equal(expect, got)
         # Verify that the Module type cannot be returned to CPython
         bad_cfunc = jit(nopython=True)(non_boxable_bad_usecase)
-        with self.assertRaises(NotImplementedError) as raises:
+        with self.assertRaises(TypeError) as raises:
             bad_cfunc()
         errmsg = str(raises.exception)
         expectmsg = "cannot convert native Module"
         self.assertIn(expectmsg, errmsg)
 
 
+def _assert_cache_stats(cfunc, expect_hit, expect_misses):
+    hit = cfunc._cache_hits[cfunc.signatures[0]]
+    if hit != expect_hit:
+        raise AssertionError('cache not used')
+    miss = cfunc._cache_misses[cfunc.signatures[0]]
+    if miss != expect_misses:
+        raise AssertionError('cache not used')
+
+
+class TestOverloadMethodCaching(TestCase):
+    # Nested multiprocessing.Pool raises AssertionError:
+    # "daemonic processes are not allowed to have children"
+    _numba_parallel_test_ = False
+
+    def test_caching_overload_method(self):
+        self._cache_dir = temp_directory(self.__class__.__name__)
+        with override_config('CACHE_DIR', self._cache_dir):
+            self.run_caching_overload_method()
+
+    def run_caching_overload_method(self):
+        cfunc = jit(nopython=True, cache=True)(cache_overload_method_usecase)
+        self.assertPreciseEqual(cfunc(MyDummy()), 13)
+        _assert_cache_stats(cfunc, 0, 1)
+        llvmir = cfunc.inspect_llvm((mydummy_type,))
+        # Ensure the inner method is not a declaration
+        decls = [ln for ln in llvmir.splitlines()
+                 if ln.startswith('declare') and 'overload_method_length' in ln]
+        self.assertEqual(len(decls), 0)
+        # Test in a separate process
+        try:
+            ctx = multiprocessing.get_context('spawn')
+        except AttributeError:
+            ctx = multiprocessing
+        q = ctx.Queue()
+        p = ctx.Process(target=run_caching_overload_method,
+                        args=(q, self._cache_dir))
+        p.start()
+        q.put(MyDummy())
+        p.join()
+        # Ensure subprocess exited normally
+        self.assertEqual(p.exitcode, 0)
+        res = q.get(timeout=1)
+        self.assertEqual(res, 13)
+
+
+def run_caching_overload_method(q, cache_dir):
+    """
+    Used by TestOverloadMethodCaching.test_caching_overload_method
+    """
+    with override_config('CACHE_DIR', cache_dir):
+        arg = q.get()
+        cfunc = jit(nopython=True, cache=True)(cache_overload_method_usecase)
+        res = cfunc(arg)
+        q.put(res)
+        # Check cache stat
+        _assert_cache_stats(cfunc, 1, 0)
+
 class TestIntrinsic(TestCase):
     def test_ll_pointer_cast(self):
         """
@@ -474,6 +556,7 @@ class TestIntrinsic(TestCase):
 
             @intrinsic
             def unsafe_cast(typingctx, src):
+                self.assertIsInstance(typingctx, typing.Context)
                 if isinstance(src, types.Integer):
                     sig = result_type(types.uintp)
 
@@ -519,7 +602,6 @@ class TestIntrinsic(TestCase):
 
         # Test
         arr = np.arange(10, dtype=np.float32)
-        foo(arr)
         with captured_stdout() as buf:
             foo(arr)
             got = buf.getvalue().splitlines()
@@ -639,5 +721,30 @@ class TestRegisterJitable(unittest.TestCase):
         self.assertIn(msg, str(raises.exception))
 
 
+class TestImportCythonFunction(unittest.TestCase):
+    @unittest.skipIf(sc is None, "Only run if SciPy >= 0.19 is installed")
+    def test_getting_function(self):
+        addr = get_cython_function_address("scipy.special.cython_special", "j0")
+        functype = ctypes.CFUNCTYPE(ctypes.c_double, ctypes.c_double)
+        _j0 = functype(addr)
+        j0 = jit(nopython=True)(lambda x: _j0(x))
+        self.assertEqual(j0(0), 1)
+
+    def test_missing_module(self):
+        with self.assertRaises(ImportError) as raises:
+            addr = get_cython_function_address("fakemodule", "fakefunction")
+        # The quotes are not there in Python 2
+        msg = "No module named '?fakemodule'?"
+        match = re.match(msg, str(raises.exception))
+        self.assertIsNotNone(match)
+
+    @unittest.skipIf(sc is None, "Only run if SciPy >= 0.19 is installed")
+    def test_missing_function(self):
+        with self.assertRaises(ValueError) as raises:
+            addr = get_cython_function_address("scipy.special.cython_special", "foo")
+        msg = "No function 'foo' found in __pyx_capi__ of 'scipy.special.cython_special'"
+        self.assertEqual(msg, str(raises.exception))
+
+
 if __name__ == '__main__':
     unittest.main()
