@@ -11,6 +11,7 @@ import numpy as np
 
 from numba import unittest_support as unittest
 from numba import jit, errors
+from numba.numpy_support import version as numpy_version
 from .support import TestCase, tag
 from .matmul_usecase import matmul_usecase, needs_matmul, needs_blas
 
@@ -373,6 +374,26 @@ def matrix_power_matrix(a, n):
     return np.linalg.matrix_power(a, n)
 
 
+def trace_matrix(a, offset=0):
+    return np.trace(a, offset)
+
+
+def trace_matrix_no_offset(a):
+    return np.trace(a)
+
+
+if numpy_version >= (1, 9):
+    def outer_matrix(a, b, out=None):
+        return np.outer(a, b, out=out)
+else:
+    def outer_matrix(a, b):
+        return np.outer(a, b)
+
+
+def kron_matrix(a, b):
+    return np.kron(a, b)
+
+
 class TestLinalgBase(TestCase):
     """
     Provides setUp and common data/error modes for testing np.linalg functions.
@@ -478,6 +499,20 @@ class TestLinalgBase(TestCase):
 
         return Q
 
+    def shape_with_0_input(self, *args):
+        """
+        returns True if an input argument has a dimension that is zero
+        and Numpy version is < 1.13, else False. This is due to behaviour
+        changes in handling dimension zero arrays:
+        https://github.com/numpy/numpy/issues/10573
+        """
+        if numpy_version < (1, 13):
+            for x in args:
+                if isinstance(x, np.ndarray):
+                    if 0 in x.shape:
+                        return True
+        return False
+
     def assert_error(self, cfunc, args, msg, err=ValueError):
         with self.assertRaises(err) as raises:
             cfunc(*args)
@@ -491,8 +526,9 @@ class TestLinalgBase(TestCase):
         msg = "np.linalg.%s() only supported on float and complex arrays" % name
         self.assert_error(cfunc, args, msg, errors.TypingError)
 
-    def assert_wrong_dimensions(self, name, cfunc, args):
-        msg = "np.linalg.%s() only supported on 2-D arrays" % name
+    def assert_wrong_dimensions(self, name, cfunc, args, la_prefix=True):
+        prefix = "np.linalg" if la_prefix else "np"
+        msg = "%s.%s() only supported on 2-D arrays" % (prefix, name)
         self.assert_error(cfunc, args, msg, errors.TypingError)
 
     def assert_no_nan_or_inf(self, cfunc, args):
@@ -563,6 +599,10 @@ class TestLinalgBase(TestCase):
         msg = "Invalid norm order for matrices."
         self.assert_error(cfunc, args, msg, ValueError)
 
+    def assert_raise_on_empty(self, cfunc, args):
+        msg = 'Arrays cannot be empty'
+        self.assert_error(cfunc, args, msg, np.linalg.LinAlgError)
+
 
 class TestTestLinalgBase(TestCase):
     """
@@ -711,6 +751,9 @@ class TestLinalgInv(TestLinalgBase):
             a = self.specific_sample_matrix((n, n), dtype, order)
             check(a)
 
+        # 0 dimensioned matrix
+        check(np.empty((0, 0)))
+
         # Non square matrix
         self.assert_non_square(cfunc, (np.ones((2, 3)),))
 
@@ -755,6 +798,11 @@ class TestLinalgCholesky(TestLinalgBase):
         cfunc = jit(nopython=True)(cholesky_matrix)
 
         def check(a):
+            if self.shape_with_0_input(a):
+                # has shape with 0 on input, numpy will fail,
+                # just make sure Numba runs without error
+                cfunc(a)
+                return
             expected = cholesky_matrix(a)
             got = cfunc(a)
             use_reconstruction = False
@@ -788,6 +836,9 @@ class TestLinalgCholesky(TestLinalgBase):
             a = self.sample_matrix(n, dtype, order)
             check(a)
 
+        # 0 dimensioned matrix
+        check(np.empty((0, 0)))
+
         rn = "cholesky"
         # Non square matrices
         self.assert_non_square(cfunc, (np.ones((2, 3), dtype=np.float64),))
@@ -834,6 +885,11 @@ class TestLinalgEigenSystems(TestLinalgBase):
         cfunc = jit(nopython=True)(func)
 
         def check(a):
+            if self.shape_with_0_input(a):
+                # has shape with 0 on input, numpy will fail,
+                # just make sure Numba runs without error
+                cfunc(a)
+                return
             expected = func(a)
             got = cfunc(a)
             # check that the returned tuple is same length
@@ -876,8 +932,8 @@ class TestLinalgEigenSystems(TestLinalgBase):
                     # modify 'a' if hermitian eigensystem functionality is
                     # being tested. 'L' for use lower part is default and
                     # the only thing used at present so we conjugate transpose
-                    # the lower part into the upper for use in the 
-                    # reconstruction. By construction the sample matrix is 
+                    # the lower part into the upper for use in the
+                    # reconstruction. By construction the sample matrix is
                     # tridiag so this is just a question of copying the lower
                     # diagonal into the upper and conjugating on the way.
                     if name[-1] == 'h':
@@ -924,7 +980,7 @@ class TestLinalgEigenSystems(TestLinalgBase):
             # Ensure proper resource management
             with self.assertNoNRTLeak():
                 cfunc(a)
-        
+
         # The main test loop
         for dtype, order in product(self.dtypes, 'FC'):
             a = self.sample_matrix(n, dtype, order)
@@ -932,6 +988,10 @@ class TestLinalgEigenSystems(TestLinalgBase):
 
         # Test both a real and complex type as the impls are different
         for ty in [np.float32, np.complex64]:
+
+            # 0 dimensioned matrix
+            check(np.empty((0, 0), dtype=ty))
+
             # Non square matrices
             self.assert_non_square(cfunc, (np.ones((2, 3), dtype=ty),))
 
@@ -1083,6 +1143,10 @@ class TestLinalgSvd(TestLinalgBase):
         self.assert_no_nan_or_inf(cfunc,
                                   (np.array([[1., 2., ], [np.inf, np.nan]],
                                             dtype=np.float64),))
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            args = (np.empty(sz), True)
+            self.assert_raise_on_empty(cfunc, args)
 
 
 class TestLinalgQr(TestLinalgBase):
@@ -1175,6 +1239,10 @@ class TestLinalgQr(TestLinalgBase):
                                   (np.array([[1., 2., ], [np.inf, np.nan]],
                                             dtype=np.float64),))
 
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            self.assert_raise_on_empty(cfunc, (np.empty(sz),))
+
 
 class TestLinalgSystems(TestLinalgBase):
     """
@@ -1183,8 +1251,9 @@ class TestLinalgSystems(TestLinalgBase):
     """
 
     # check for RHS with dimension > 2 raises
-    def assert_wrong_dimensions_1D(self, name, cfunc, args):
-        msg = "np.linalg.%s() only supported on 1 and 2-D arrays" % name
+    def assert_wrong_dimensions_1D(self, name, cfunc, args, la_prefix=True):
+        prefix = "np.linalg" if la_prefix else "np"
+        msg = "%s.%s() only supported on 1 and 2-D arrays" % (prefix, name)
         self.assert_error(cfunc, args, msg, errors.TypingError)
 
     # check that a dimensionally invalid system raises
@@ -1385,6 +1454,18 @@ class TestLinalgLstsq(TestLinalgSystems):
                 inner_test_loop_fn(A, dt,
                                    rcond=approx_half_rank_rcond)
 
+        # check empty arrays
+        empties = [
+        [(0, 1), (1,)], # empty A, valid b
+        [(1, 0), (1,)], # empty A, valid b
+        [(1, 1), (0,)], # valid A, empty 1D b 
+        [(1, 1), (1, 0)],  # valid A, empty 2D b 
+        ]
+
+        for A, b in empties:
+            args = (np.empty(A), np.empty(b))
+            self.assert_raise_on_empty(cfunc, args)
+
         # Test input validation
         ok = np.array([[1., 2.], [3., 4.]], dtype=np.float64)
 
@@ -1502,6 +1583,9 @@ class TestLinalgSolve(TestLinalgSystems):
                 tmp = B[:, 0].copy(order=b_order)
                 check(A, tmp)
 
+        # check empty
+        cfunc(np.empty((0, 0)), np.empty((0,)))
+
         # Test input validation
         ok = np.array([[1., 0.], [0., 1.]], dtype=np.float64)
 
@@ -1561,6 +1645,11 @@ class TestLinalgPinv(TestLinalgBase):
         cfunc = jit(nopython=True)(pinv_matrix)
 
         def check(a, **kwargs):
+            if self.shape_with_0_input(a):
+                # has shape with 0 on input, numpy will fail,
+                # just make sure Numba runs without error
+                cfunc(a, **kwargs)
+                return
             expected = pinv_matrix(a, **kwargs)
             got = cfunc(a, **kwargs)
 
@@ -1650,6 +1739,10 @@ class TestLinalgPinv(TestLinalgBase):
                 approx_half_rank_rcond = minmn * rcond
                 check(a, rcond=approx_half_rank_rcond)
 
+        # check empty
+        for sz in [(0, 1), (1, 0)]:
+            check(np.empty(sz))
+
         rn = "pinv"
 
         # Wrong dtype
@@ -1675,6 +1768,11 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
     """
 
     def check_det(self, cfunc, a, **kwargs):
+        if self.shape_with_0_input(a):
+            # has shape with 0 on input, numpy will fail,
+            # just make sure Numba runs without error
+            cfunc(a, **kwargs)
+            return
         expected = det_matrix(a, **kwargs)
         got = cfunc(a, **kwargs)
 
@@ -1688,6 +1786,11 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
             cfunc(a, **kwargs)
 
     def check_slogdet(self, cfunc, a, **kwargs):
+        if self.shape_with_0_input(a):
+            # has shape with 0 on input, numpy will fail,
+            # just make sure Numba runs without error
+            cfunc(a, **kwargs)
+            return
         expected = slogdet_matrix(a, **kwargs)
         got = cfunc(a, **kwargs)
 
@@ -1739,6 +1842,9 @@ class TestLinalgDetAndSlogdet(TestLinalgBase):
             a = np.zeros((3, 3), dtype=dtype)
             check(cfunc, a)
 
+        # check empty
+        check(cfunc, np.empty((0, 0)))
+
         # Wrong dtype
         self.assert_wrong_dtype(rn, cfunc,
                                 (np.ones((2, 2), dtype=np.int32),))
@@ -1810,13 +1916,6 @@ class TestLinalgNorm(TestLinalgSystems):
             a = self.sample_vector(10, dtype)[::3]
             check(a, ord=nrm_type)
 
-        # check that numba returns zero for empty arrays. Numpy returns zero
-        # for most norm types and raises ValueError for +/-np.inf.
-        for dtype, nrm_type, order in \
-                product(self.dtypes, nrm_types, 'FC'):
-            a = np.array([], dtype=dtype, order=order)
-            self.assertEqual(cfunc(a, nrm_type), 0.0)
-
         # Check 2D inputs:
         # test: column vector, tall, wide, square, row vector
         # prime sizes
@@ -1846,10 +1945,14 @@ class TestLinalgNorm(TestLinalgSystems):
             check(a[1, 4::3], ord=nrm_type)
 
         # check that numba returns zero for empty arrays. Numpy returns zero
-        # for some norm types and raises a variety of errors for others.
+        # for most norm types and raises ValueError for +/-np.inf.
+        # there is not a great deal of consistency in Numpy's response so
+        # it is not being emulated in Numba
         for dtype, nrm_type, order in \
                 product(self.dtypes, nrm_types, 'FC'):
-            a = np.array([[]], dtype=dtype, order=order)
+            a = np.empty((0,), dtype=dtype, order=order)
+            self.assertEqual(cfunc(a, nrm_type), 0.0)
+            a = np.empty((0, 0), dtype=dtype, order=order)
             self.assertEqual(cfunc(a, nrm_type), 0.0)
 
         rn = "norm"
@@ -1919,6 +2022,10 @@ class TestLinalgCond(TestLinalgBase):
             a = self.specific_sample_matrix(size, dtype, order)
             check(a)
 
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            self.assert_raise_on_empty(cfunc, (np.empty(sz),))
+
         # try an ill-conditioned system with 2-norm, make sure np raises an
         # overflow warning as the result is `+inf` and that the result from
         # numba matches.
@@ -2035,6 +2142,11 @@ class TestLinalgMatrixRank(TestLinalgSystems):
             self.assertEqual(cfunc(a), 1)
             check(a)
 
+        # empty
+        for sz in [(0, 1), (1, 0), (0, 0)]:
+            for tol in [None, 1e-13]:
+                self.assert_raise_on_empty(cfunc, (np.empty(sz), tol))
+
         rn = "matrix_rank"
 
         # Wrong dtype
@@ -2090,6 +2202,8 @@ class TestLinalgMatrixPower(TestLinalgBase):
                 product(sizes, powers, self.dtypes, 'FC'):
             a = self.specific_sample_matrix(size, dtype, order)
             check(a, pwr)
+            a = np.empty((0, 0), dtype=dtype, order=order)
+            check(a, pwr)
 
         rn = "matrix_power"
 
@@ -2101,6 +2215,11 @@ class TestLinalgMatrixPower(TestLinalgBase):
         self.assert_wrong_dtype(rn, cfunc,
                                 (np.ones((2, 2), dtype=np.int32), 1))
 
+        # non square system
+        args = (np.ones((3, 5)), 1)
+        msg = 'input must be a square array'
+        self.assert_error(cfunc, args, msg)
+
         # Dimension issue
         self.assert_wrong_dimensions(rn, cfunc,
                                      (np.ones(10, dtype=np.float64), 1))
@@ -2112,5 +2231,185 @@ class TestLinalgMatrixPower(TestLinalgBase):
         self.assert_raise_on_singular(cfunc, (np.array([[0., 0], [1, 1]]), -1))
 
 
+class TestTrace(TestLinalgBase):
+    """
+    Tests for np.trace.
+    """
+
+    def setUp(self):
+        super(TestTrace, self).setUp()
+        # compile two versions, one with and one without the offset kwarg
+        self.cfunc_w_offset = jit(nopython=True)(trace_matrix)
+        self.cfunc_no_offset = jit(nopython=True)(trace_matrix_no_offset)
+
+    def assert_int_offset(self, cfunc, a, **kwargs):
+        # validate first arg is ok
+        cfunc(a)
+        # pass in kwarg and assert fail
+        with self.assertRaises(errors.TypingError):
+            cfunc(a, **kwargs)
+
+    def test_trace(self):
+
+        def check(a, **kwargs):
+            if 'offset' in kwargs:
+                expected = trace_matrix(a, **kwargs)
+                cfunc = self.cfunc_w_offset
+            else:
+                expected = trace_matrix_no_offset(a, **kwargs)
+                cfunc = self.cfunc_no_offset
+
+            got = cfunc(a, **kwargs)
+
+            res = 5 * np.finfo(a.dtype).resolution
+            np.testing.assert_allclose(got, expected, rtol=res, atol=res)
+
+            # Ensure proper resource management
+            with self.assertNoNRTLeak():
+                cfunc(a, **kwargs)
+
+        # test: column vector, tall, wide, square, row vector
+        # prime sizes
+        sizes = [(7, 1), (11, 5), (5, 11), (3, 3), (1, 7)]
+
+        # offsets to cover the range of the matrix sizes above
+        offsets = [-13, -12, -11] + list(range(-10, 10)) + [11, 12, 13]
+
+        for size, offset, dtype, order in \
+                product(sizes, offsets, self.dtypes, 'FC'):
+            a = self.specific_sample_matrix(size, dtype, order)
+            check(a, offset=offset)
+            if offset == 0:
+                check(a)
+            a = np.empty((0, 0), dtype=dtype, order=order)
+            check(a, offset=offset)
+            if offset == 0:
+                check(a)
+
+        rn = "trace"
+
+        # Dimension issue
+        self.assert_wrong_dimensions(rn, self.cfunc_w_offset,
+                                     (np.ones(10, dtype=np.float64), 1), False)
+        self.assert_wrong_dimensions(rn, self.cfunc_no_offset,
+                                     (np.ones(10, dtype=np.float64),), False)
+
+        # non-integer supplied as exponent
+        self.assert_int_offset(
+            self.cfunc_w_offset, np.ones(
+                (2, 2)), offset=1.2)
+
+    def test_trace_w_optional_input(self):
+        "Issue 2314"
+        @jit("(optional(float64[:,:]),)", nopython=True)
+        def tested(a):
+            return np.trace(a)
+
+        a = np.ones((5, 5), dtype=np.float64)
+        tested(a)
+
+        with self.assertRaises(TypeError) as raises:
+            tested(None)
+
+        errmsg = str(raises.exception)
+        self.assertEqual('expected array(float64, 2d, A), got None', errmsg)
+
+
+class TestBasics(TestLinalgSystems):  # TestLinalgSystems for 1d test
+
+    order1 = cycle(['F', 'C', 'C', 'F'])
+    order2 = cycle(['C', 'F', 'C', 'F'])
+
+    # test: column vector, matrix, row vector, 1d sizes
+    # (7, 1, 3) and two scalars
+    sizes = [(7, 1), (3, 3), (1, 7), (7,), (1,), (3,), 3., 5.]
+
+    def _assert_wrong_dim(self, rn, cfunc):
+        # Dimension issue
+        self.assert_wrong_dimensions_1D(
+            rn, cfunc, (np.array([[[1]]], dtype=np.float64), np.ones(1)), False)
+        self.assert_wrong_dimensions_1D(
+            rn, cfunc, (np.ones(1), np.array([[[1]]], dtype=np.float64)), False)
+
+    def _gen_input(self, size, dtype, order):
+        if not isinstance(size, tuple):
+            return size
+        else:
+            if len(size) == 1:
+                return self.sample_vector(size[0], dtype)
+            else:
+                return self.sample_vector(
+                    size[0] * size[1],
+                    dtype).reshape(
+                    size, order=order)
+
+    def _get_input(self, size1, size2, dtype):
+        a = self._gen_input(size1, dtype, next(self.order1))
+        b = self._gen_input(size2, dtype, next(self.order2))
+        # force domain consistency as underlying ufuncs require it
+        if np.iscomplexobj(a):
+            b = b + 1j
+        if np.iscomplexobj(b):
+            a = a + 1j
+        return (a, b)
+
+    def test_outer(self):
+        cfunc = jit(nopython=True)(outer_matrix)
+
+        def check(a, b, **kwargs):
+
+            # check without kwargs
+            expected = outer_matrix(a, b)
+            got = cfunc(a, b)
+
+            res = 5 * np.finfo(np.asarray(a).dtype).resolution
+            np.testing.assert_allclose(got, expected, rtol=res, atol=res)
+
+            # if kwargs present check with them too
+            if 'out' in kwargs:
+                got = cfunc(a, b, **kwargs)
+                np.testing.assert_allclose(got, expected, rtol=res,
+                                           atol=res)
+                np.testing.assert_allclose(kwargs['out'], expected,
+                                           rtol=res, atol=res)
+
+            # Ensure proper resource management
+            with self.assertNoNRTLeak():
+                cfunc(a, b, **kwargs)
+
+        for size1, size2, dtype in \
+                product(self.sizes, self.sizes, self.dtypes):
+            (a, b) = self._get_input(size1, size2, dtype)
+            check(a, b)
+            if numpy_version >= (1, 9):
+                c = np.empty((np.asarray(a).size, np.asarray(b).size),
+                             dtype=np.asarray(a).dtype)
+                check(a, b, out=c)
+
+        self._assert_wrong_dim("outer", cfunc)
+
+    def test_kron(self):
+        cfunc = jit(nopython=True)(kron_matrix)
+
+        def check(a, b, **kwargs):
+
+            expected = kron_matrix(a, b)
+            got = cfunc(a, b)
+
+            res = 5 * np.finfo(np.asarray(a).dtype).resolution
+            np.testing.assert_allclose(got, expected, rtol=res, atol=res)
+
+            # Ensure proper resource management
+            with self.assertNoNRTLeak():
+                cfunc(a, b)
+
+        for size1, size2, dtype in \
+                product(self.sizes, self.sizes, self.dtypes):
+            (a, b) = self._get_input(size1, size2, dtype)
+            check(a, b)
+
+        self._assert_wrong_dim("kron", cfunc)
+
+
 if __name__ == '__main__':
     unittest.main()
