@@ -11,7 +11,7 @@ from numba import unittest_support as unittest
 from numba.compiler import compile_isolated, Flags, utils
 from numba import jit, typeof, types
 from numba.numpy_support import version as np_version
-from numba.errors import UntypedAttributeError
+from numba.errors import TypingError
 from .support import TestCase, CompilationCache
 
 no_pyobj_flags = Flags()
@@ -42,6 +42,12 @@ def bincount2(a, w):
 def searchsorted(a, v):
     return np.searchsorted(a, v)
 
+def searchsorted_left(a, v):
+    return np.searchsorted(a, v, side='left')
+
+def searchsorted_right(a, v):
+    return np.searchsorted(a, v, side='right')
+
 def digitize(*args):
     return np.digitize(*args)
 
@@ -60,6 +66,12 @@ def finfo(*args):
 def finfo_machar(*args):
     return np.finfo(*args).machar
 
+def correlate(a, v):
+    return np.correlate(a, v)
+
+def convolve(a, v):
+    return np.convolve(a, v)
+
 
 class TestNPFunctions(TestCase):
     """
@@ -312,11 +324,25 @@ class TestNPFunctions(TestCase):
         pyfunc = searchsorted
         cfunc = jit(nopython=True)(pyfunc)
 
+        pyfunc_left = searchsorted_left
+        cfunc_left = jit(nopython=True)(pyfunc_left)
+
+        pyfunc_right = searchsorted_right
+        cfunc_right = jit(nopython=True)(pyfunc_right)
+
         def check(a, v):
             expected = pyfunc(a, v)
             got = cfunc(a, v)
             self.assertPreciseEqual(expected, got)
 
+            expected = pyfunc_left(a, v)
+            got = cfunc_left(a, v)
+            self.assertPreciseEqual(expected, got)
+
+            expected = pyfunc_right(a, v)
+            got = cfunc_right(a, v)
+            self.assertPreciseEqual(expected, got)
+
         # First with integer values (no NaNs)
         bins = np.arange(5) ** 2
         values = np.arange(20) - 1
@@ -345,6 +371,20 @@ class TestNPFunctions(TestCase):
             # Sequence values
             check(a, list(values))
 
+        # nonsense value for 'side' raises TypingError
+        def bad_side(a, v):
+            return np.searchsorted(a, v, side='nonsense')
+        cfunc = jit(nopython=True)(bad_side)
+        with self.assertTypingError():
+            cfunc([1,2], 1)
+
+        # non-constant value for 'side' raises TypingError
+        def nonconst_side(a, v, side='left'):
+            return np.searchsorted(a, v, side=side)
+        cfunc = jit(nopython=True)(nonconst_side)
+        with self.assertTypingError():
+            cfunc([1,2], 1, side='right')
+
     def test_digitize(self):
         pyfunc = digitize
         cfunc = jit(nopython=True)(pyfunc)
@@ -424,6 +464,61 @@ class TestNPFunctions(TestCase):
 
         check_values(values)
 
+    def _test_correlate_convolve(self, pyfunc):
+        cfunc = jit(nopython=True)(pyfunc)
+        # only 1d arrays are accepted, test varying lengths 
+        # and varying dtype
+        lengths = (1, 2, 3, 7)
+        dts = [np.int8, np.int32, np.int64, np.float32, np.float64,
+               np.complex64, np.complex128]
+
+        for dt1, dt2, n, m in itertools.product(dts, dts, lengths, lengths):
+            a = np.arange(n, dtype=dt1)
+            v = np.arange(m, dtype=dt2)
+
+            if np.issubdtype(dt1, np.complexfloating):
+                a = (a + 1j * a).astype(dt1)
+            if np.issubdtype(dt2, np.complexfloating):
+                v = (v + 1j * v).astype(dt2)
+
+            expected = pyfunc(a, v)
+            got = cfunc(a, v)
+            self.assertPreciseEqual(expected, got)
+
+        _a = np.arange(12).reshape(4, 3)
+        _b = np.arange(12)
+        for x, y in [(_a, _b), (_b, _a)]:
+            with self.assertRaises(TypingError) as raises:
+                cfunc(x, y)
+            msg = 'only supported on 1D arrays'
+            self.assertIn(msg, str(raises.exception))
+
+    def test_correlate(self):
+        self._test_correlate_convolve(correlate)
+        # correlate supports 0 dimension arrays
+        _a = np.ones(shape=(0,))
+        _b = np.arange(5)
+        cfunc = jit(nopython=True)(correlate)
+        for x, y in [(_a, _b), (_b, _a), (_a, _a)]:
+            expected = correlate(x, y)
+            got = cfunc(x, y)
+            self.assertPreciseEqual(expected, got)
+
+    def test_convolve(self):
+        self._test_correlate_convolve(convolve)
+        # convolve raises if either array has a 0 dimension
+        _a = np.ones(shape=(0,))
+        _b = np.arange(5)
+        cfunc = jit(nopython=True)(convolve)
+        for x, y in [(_a, _b), (_b, _a)]:
+            with self.assertRaises(ValueError) as raises:
+                cfunc(x, y)
+            if len(x) == 0:
+                self.assertIn("'a' cannot be empty", str(raises.exception))
+            else:
+                self.assertIn("'v' cannot be empty", str(raises.exception))
+
+
 class TestNPMachineParameters(TestCase):
     # tests np.finfo, np.iinfo, np.MachAr
 
@@ -473,7 +568,7 @@ def foo():
             self.check(hc_func, attrs)
 
         # check unsupported attr raises
-        with self.assertRaises(UntypedAttributeError) as raises:
+        with self.assertRaises(TypingError) as raises:
             cfunc = jit(nopython=True)(finfo_machar)
             cfunc(7.)
         msg = "Unknown attribute 'machar' of type finfo"
