@@ -378,6 +378,20 @@ class TestOperators(TestCase):
 
     op = LiteralOperatorImpl
 
+    _bitwise_opnames = {
+        'bitshift_left_usecase': '<<',
+        'bitshift_ileft_usecase': '<<',
+        'bitshift_right_usecase': '>>',
+        'bitshift_iright_usecase': '>>',
+        'bitwise_and_usecase': '&',
+        'bitwise_iand_usecase': '&',
+        'bitwise_or_usecase': '|',
+        'bitwise_ior_usecase': '|',
+        'bitwise_xor_usecase': '^',
+        'bitwise_ixor_usecase': '^',
+        'bitwise_not_usecase_binary': '~',
+    }
+
     def run_test_ints(self, pyfunc, x_operands, y_operands, types_list,
                       flags=force_pyobj_flags):
         if pyfunc is NotImplemented:
@@ -1022,6 +1036,40 @@ class TestOperators(TestCase):
     def test_bitwise_not_npm(self):
         self.test_bitwise_not(flags=Noflags)
 
+    def test_bitwise_float(self):
+        """
+        Make sure that bitwise float operations are not allowed
+        """
+        def assert_reject_compile(pyfunc, argtypes, opname):
+            msg = 'expecting TypingError when compiling {}'.format(pyfunc)
+            with self.assertRaises(errors.TypingError, msg=msg) as raises:
+                compile_isolated(pyfunc, argtypes)
+            # check error message
+            fmt = 'Invalid usage of {}'
+            expecting = fmt.format(opname
+                                   if isinstance(opname, str)
+                                   else 'Function({})'.format(opname))
+            self.assertIn(expecting, str(raises.exception))
+
+        methods = [
+            'bitshift_left_usecase',
+            'bitshift_ileft_usecase',
+            'bitshift_right_usecase',
+            'bitshift_iright_usecase',
+            'bitwise_and_usecase',
+            'bitwise_iand_usecase',
+            'bitwise_or_usecase',
+            'bitwise_ior_usecase',
+            'bitwise_xor_usecase',
+            'bitwise_ixor_usecase',
+            'bitwise_not_usecase_binary',
+        ]
+
+        for name in methods:
+            pyfunc = getattr(self.op, name)
+            assert_reject_compile(pyfunc, (types.float32, types.float32),
+                                  opname=self._bitwise_opnames[name])
+
     def test_not(self):
         pyfunc = self.op.not_usecase
 
@@ -1167,6 +1215,20 @@ class TestOperatorModule(TestOperators):
 
     op = FunctionalOperatorImpl
 
+    _bitwise_opnames = {
+        'bitshift_left_usecase': operator.lshift,
+        'bitshift_ileft_usecase': operator.ilshift,
+        'bitshift_right_usecase': operator.rshift,
+        'bitshift_iright_usecase': operator.irshift,
+        'bitwise_and_usecase': operator.and_,
+        'bitwise_iand_usecase': operator.iand,
+        'bitwise_or_usecase': operator.or_,
+        'bitwise_ior_usecase': operator.ior,
+        'bitwise_xor_usecase': operator.xor,
+        'bitwise_ixor_usecase': operator.ixor,
+        'bitwise_not_usecase_binary': operator.invert,
+    }
+
 
 class TestMixedInts(TestCase):
     """
@@ -1447,6 +1509,37 @@ class TestStaticPower(TestCase):
 
         self._check_pow(exponents, vals)
 
+class TestStringConstComparison(TestCase):
+    """
+    Test comparison of string constants
+    """
+    def test_eq(self):
+        def test_impl1():
+            s = 'test'
+            return s == 'test'
+
+        def test_impl2():
+            s = 'test1'
+            return s == 'test'
+
+        cfunc1 = jit(nopython=True)(test_impl1)
+        cfunc2 = jit(nopython=True)(test_impl2)
+        self.assertEqual(test_impl1(), cfunc1())
+        self.assertEqual(test_impl2(), cfunc2())
+
+    def test_neq(self):
+        def test_impl1():
+            s = 'test'
+            return s != 'test'
+
+        def test_impl2():
+            s = 'test1'
+            return s != 'test'
+
+        cfunc1 = jit(nopython=True)(test_impl1)
+        cfunc2 = jit(nopython=True)(test_impl2)
+        self.assertEqual(test_impl1(), cfunc1())
+        self.assertEqual(test_impl2(), cfunc2())
 
 if __name__ == '__main__':
     unittest.main()
