@@ -3,25 +3,30 @@ from __future__ import print_function
 import collections
 import functools
 import math
+import multiprocessing
 import os
 import random
 import subprocess
 import sys
+import threading
 
 import numpy as np
 
 import numba.unittest_support as unittest
 from numba import jit, _helperlib, types
 from numba.compiler import compile_isolated
-from numba.targets.randomimpl import random_init
 from .support import TestCase, compile_function, tag
 
 
 # State size of the Mersenne Twister
 N = 624
 
-py_state_ptr = _helperlib.c_helpers['py_random_state']
-np_state_ptr = _helperlib.c_helpers['np_random_state']
+
+def get_py_state_ptr():
+    return _helperlib.rnd_get_py_state_ptr()
+
+def get_np_state_ptr():
+    return _helperlib.rnd_get_np_state_ptr()
 
 
 def numpy_randint1(a):
@@ -71,6 +76,21 @@ def numpy_check_randn(seed, a, b):
     got = np.random.randn(a, b)
     return expected, got
 
+# Numba's current PRNG for NumPy follows Python's implementation,
+# however, Python's implementation of shuffle does not work on
+# multidimensional arrays.  Hence, we cannot test np.random.shuffle with
+# Python's shuffle.  Therefore, we implement a shuffle in Python that
+# works on multidimensional arrays and compare against that.
+def python_shuffle(arr, r):
+    if arr.ndim == 1:
+        r.shuffle(arr)
+    else:
+        i = arr.shape[0] - 1
+        while i > 0:
+            j = r.randrange(i + 1)
+            arr[i], arr[j] = np.copy(arr[j]), np.copy(arr[i])
+            i -= 1
+    return arr
 
 def jit_with_args(name, argstring):
     code = """def func(%(argstring)s):
@@ -212,16 +232,16 @@ class TestInternals(BaseTest):
         self.assertEqual(len(set(states)), len(states))
 
     def test_get_set_state(self):
-        self._check_get_set_state(py_state_ptr)
+        self._check_get_set_state(get_py_state_ptr())
 
     def test_shuffle(self):
-        self._check_shuffle(py_state_ptr)
+        self._check_shuffle(get_py_state_ptr())
 
     def test_init(self):
-        self._check_init(py_state_ptr)
+        self._check_init(get_py_state_ptr())
 
     def test_perturb(self):
-        self._check_perturb(py_state_ptr)
+        self._check_perturb(get_py_state_ptr())
 
 
 class TestRandom(BaseTest):
@@ -231,10 +251,6 @@ class TestRandom(BaseTest):
     # iterative algorithms with sensitive exit conditions.
     # Therefore we stick to hardcoded integers for seed values.
 
-    def setUp(self):
-        # Make sure the PRNG is initialized before we set the seed
-        random_init()
-
     def _check_random_seed(self, seedfunc, randomfunc):
         """
         Check seed()- and random()-like functions.
@@ -291,7 +307,7 @@ class TestRandom(BaseTest):
 
     @tag('important')
     def test_random_getrandbits(self):
-        self._check_getrandbits(jit_unary("random.getrandbits"), py_state_ptr)
+        self._check_getrandbits(jit_unary("random.getrandbits"), get_py_state_ptr())
 
     # Explanation for the large ulps value: on 32-bit platforms, our
     # LLVM-compiled functions use SSE but they are compared against
@@ -324,30 +340,30 @@ class TestRandom(BaseTest):
 
     @tag('important')
     def test_random_gauss(self):
-        self._check_gauss(jit_binary("random.gauss"), None, None, py_state_ptr)
+        self._check_gauss(jit_binary("random.gauss"), None, None, get_py_state_ptr())
 
     def test_random_normalvariate(self):
         # normalvariate() is really an alias to gauss() in Numba
         # (not in Python, though - they use different algorithms)
         self._check_gauss(jit_binary("random.normalvariate"), None, None,
-                          py_state_ptr)
+                          get_py_state_ptr())
 
     @tag('important')
     def test_numpy_normal(self):
         self._check_gauss(jit_binary("np.random.normal"),
                           jit_unary("np.random.normal"),
                           jit_nullary("np.random.normal"),
-                          np_state_ptr)
+                          get_np_state_ptr())
 
     @tag('important')
     def test_numpy_standard_normal(self):
         self._check_gauss(None, None, jit_nullary("np.random.standard_normal"),
-                          np_state_ptr)
+                          get_np_state_ptr())
 
     @tag('important')
     def test_numpy_randn(self):
         self._check_gauss(None, None, jit_nullary("np.random.randn"),
-                          np_state_ptr)
+                          get_np_state_ptr())
 
     def _check_lognormvariate(self, func2, func1, func0, ptr):
         """
@@ -366,13 +382,13 @@ class TestRandom(BaseTest):
 
     def test_random_lognormvariate(self):
         self._check_lognormvariate(jit_binary("random.lognormvariate"),
-                                   None, None, py_state_ptr)
+                                   None, None, get_py_state_ptr())
 
     def test_numpy_lognormal(self):
         self._check_lognormvariate(jit_binary("np.random.lognormal"),
                                    jit_unary("np.random.lognormal"),
                                    jit_nullary("np.random.lognormal"),
-                                   np_state_ptr)
+                                   get_np_state_ptr())
 
     def _check_randrange(self, func1, func2, func3, ptr, max_width):
         """
@@ -414,7 +430,7 @@ class TestRandom(BaseTest):
             cr2 = compile_isolated(random_randrange2, (tp, tp))
             cr3 = compile_isolated(random_randrange3, (tp, tp, tp))
             self._check_randrange(cr1.entry_point, cr2.entry_point,
-                                  cr3.entry_point, py_state_ptr, max_width)
+                                  cr3.entry_point, get_py_state_ptr(), max_width)
 
     @tag('important')
     def test_numpy_randint(self):
@@ -422,7 +438,7 @@ class TestRandom(BaseTest):
             cr1 = compile_isolated(numpy_randint1, (tp,))
             cr2 = compile_isolated(numpy_randint2, (tp, tp))
             self._check_randrange(cr1.entry_point, cr2.entry_point,
-                                  None, np_state_ptr, max_width)
+                                  None, get_np_state_ptr(), max_width)
 
     def _check_randint(self, func, ptr, max_width):
         """
@@ -448,7 +464,7 @@ class TestRandom(BaseTest):
     def test_random_randint(self):
         for tp, max_width in [(types.int64, 2**63), (types.int32, 2**31)]:
             cr = compile_isolated(random_randint, (tp, tp))
-            self._check_randint(cr.entry_point, py_state_ptr, max_width)
+            self._check_randint(cr.entry_point, get_py_state_ptr(), max_width)
 
     def _check_uniform(self, func, ptr):
         """
@@ -461,11 +477,11 @@ class TestRandom(BaseTest):
 
     @tag('important')
     def test_random_uniform(self):
-        self._check_uniform(jit_binary("random.uniform"), py_state_ptr)
+        self._check_uniform(jit_binary("random.uniform"), get_py_state_ptr())
 
     @tag('important')
     def test_numpy_uniform(self):
-        self._check_uniform(jit_binary("np.random.uniform"), np_state_ptr)
+        self._check_uniform(jit_binary("np.random.uniform"), get_np_state_ptr())
 
     def _check_triangular(self, func2, func3, ptr):
         """
@@ -481,12 +497,12 @@ class TestRandom(BaseTest):
     def test_random_triangular(self):
         self._check_triangular(jit_binary("random.triangular"),
                                jit_ternary("random.triangular"),
-                               py_state_ptr)
+                               get_py_state_ptr())
 
     def test_numpy_triangular(self):
         triangular = jit_ternary("np.random.triangular")
         fixed_triangular = lambda l, r, m: triangular(l, m, r)
-        self._check_triangular(None, fixed_triangular, np_state_ptr)
+        self._check_triangular(None, fixed_triangular, get_np_state_ptr())
 
     def _check_gammavariate(self, func2, func1, ptr):
         """
@@ -511,15 +527,15 @@ class TestRandom(BaseTest):
 
     def test_random_gammavariate(self):
         self._check_gammavariate(jit_binary("random.gammavariate"), None,
-                                 py_state_ptr)
+                                 get_py_state_ptr())
 
     def test_numpy_gamma(self):
         self._check_gammavariate(jit_binary("np.random.gamma"),
                                  jit_unary("np.random.gamma"),
-                                 np_state_ptr)
+                                 get_np_state_ptr())
         self._check_gammavariate(None,
                                  jit_unary("np.random.standard_gamma"),
-                                 np_state_ptr)
+                                 get_np_state_ptr())
 
     def _check_betavariate(self, func, ptr):
         """
@@ -535,10 +551,10 @@ class TestRandom(BaseTest):
         self.assertRaises(ValueError, func, 1.0, -0.5)
 
     def test_random_betavariate(self):
-        self._check_betavariate(jit_binary("random.betavariate"), py_state_ptr)
+        self._check_betavariate(jit_binary("random.betavariate"), get_py_state_ptr())
 
     def test_numpy_beta(self):
-        self._check_betavariate(jit_binary("np.random.beta"), np_state_ptr)
+        self._check_betavariate(jit_binary("np.random.beta"), get_np_state_ptr())
 
     def _check_vonmisesvariate(self, func, ptr):
         """
@@ -550,11 +566,11 @@ class TestRandom(BaseTest):
 
     def test_random_vonmisesvariate(self):
         self._check_vonmisesvariate(jit_binary("random.vonmisesvariate"),
-                                    py_state_ptr)
+                                    get_py_state_ptr())
 
     def test_numpy_vonmises(self):
         self._check_vonmisesvariate(jit_binary("np.random.vonmises"),
-                                    np_state_ptr)
+                                    get_np_state_ptr())
 
     def _check_expovariate(self, func, ptr):
         """
@@ -569,7 +585,7 @@ class TestRandom(BaseTest):
                                         prec='double')
 
     def test_random_expovariate(self):
-        self._check_expovariate(jit_unary("random.expovariate"), py_state_ptr)
+        self._check_expovariate(jit_unary("random.expovariate"), get_py_state_ptr())
 
     def _check_exponential(self, func1, func0, ptr):
         """
@@ -585,12 +601,12 @@ class TestRandom(BaseTest):
     def test_numpy_exponential(self):
         self._check_exponential(jit_unary("np.random.exponential"),
                                 jit_nullary("np.random.exponential"),
-                                np_state_ptr)
+                                get_np_state_ptr())
 
     def test_numpy_standard_exponential(self):
         self._check_exponential(None,
                                 jit_nullary("np.random.standard_exponential"),
-                                np_state_ptr)
+                                get_np_state_ptr())
 
     def _check_paretovariate(self, func, ptr):
         """
@@ -601,12 +617,12 @@ class TestRandom(BaseTest):
         self._check_dist(func, r.paretovariate, [(0.5,), (3.5,)])
 
     def test_random_paretovariate(self):
-        self._check_paretovariate(jit_unary("random.paretovariate"), py_state_ptr)
+        self._check_paretovariate(jit_unary("random.paretovariate"), get_py_state_ptr())
 
     def test_numpy_pareto(self):
         pareto = jit_unary("np.random.pareto")
         fixed_pareto = lambda a: pareto(a) + 1.0
-        self._check_paretovariate(fixed_pareto, np_state_ptr)
+        self._check_paretovariate(fixed_pareto, get_np_state_ptr())
 
     def _check_weibullvariate(self, func2, func1, ptr):
         """
@@ -623,17 +639,17 @@ class TestRandom(BaseTest):
 
     def test_random_weibullvariate(self):
         self._check_weibullvariate(jit_binary("random.weibullvariate"),
-                                   None, py_state_ptr)
+                                   None, get_py_state_ptr())
 
     def test_numpy_weibull(self):
         self._check_weibullvariate(None, jit_unary("np.random.weibull"),
-                                   np_state_ptr)
+                                   get_np_state_ptr())
 
     @tag('important')
     def test_numpy_binomial(self):
         # We follow Numpy's algorithm up to n*p == 30
         binomial = jit_binary("np.random.binomial")
-        r = self._follow_numpy(np_state_ptr, 0)
+        r = self._follow_numpy(get_np_state_ptr(), 0)
         self._check_dist(binomial, r.binomial, [(18, 0.25)])
         # Sanity check many values
         for n in (100, 1000, 10000):
@@ -658,14 +674,14 @@ class TestRandom(BaseTest):
     @tag('important')
     def test_numpy_chisquare(self):
         chisquare = jit_unary("np.random.chisquare")
-        r = self._follow_cpython(np_state_ptr)
+        r = self._follow_cpython(get_np_state_ptr())
         self._check_dist(chisquare,
                          functools.partial(py_chisquare, r),
                          [(1.5,), (2.5,)])
 
     def test_numpy_f(self):
         f = jit_binary("np.random.f")
-        r = self._follow_cpython(np_state_ptr)
+        r = self._follow_cpython(get_np_state_ptr())
         self._check_dist(f, functools.partial(py_f, r),
                          [(0.5, 1.5), (1.5, 0.8)])
 
@@ -693,13 +709,13 @@ class TestRandom(BaseTest):
 
     def test_numpy_gumbel(self):
         gumbel = jit_binary("np.random.gumbel")
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         self._check_dist(gumbel, r.gumbel, [(0.0, 1.0), (-1.5, 3.5)])
 
     def test_numpy_hypergeometric(self):
         # Our implementation follows Numpy's up to nsamples = 10.
         hg = jit_ternary("np.random.hypergeometric")
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         self._check_dist(hg, r.hypergeometric,
                          [(1000, 5000, 10), (5000, 1000, 10)],
                          niters=30)
@@ -716,7 +732,7 @@ class TestRandom(BaseTest):
         self.assertGreaterEqual(np.mean(r), 90.0)
 
     def test_numpy_laplace(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         self._check_dist(jit_binary("np.random.laplace"), r.laplace,
                          [(0.0, 1.0), (-1.5, 3.5)])
         self._check_dist(jit_unary("np.random.laplace"), r.laplace,
@@ -725,7 +741,7 @@ class TestRandom(BaseTest):
 
     @tag('important')
     def test_numpy_logistic(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         self._check_dist(jit_binary("np.random.logistic"), r.logistic,
                          [(0.0, 1.0), (-1.5, 3.5)])
         self._check_dist(jit_unary("np.random.logistic"), r.logistic,
@@ -733,14 +749,14 @@ class TestRandom(BaseTest):
         self._check_dist(jit_nullary("np.random.logistic"), r.logistic, [()])
 
     def test_numpy_logseries(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         logseries = jit_unary("np.random.logseries")
         self._check_dist(logseries, r.logseries,
                          [(0.1,), (0.99,), (0.9999,)],
                          niters=50)
         # Numpy's logseries overflows on 32-bit builds, so instead
         # hardcode Numpy's (correct) output on 64-bit builds.
-        r = self._follow_numpy(np_state_ptr, seed=1)
+        r = self._follow_numpy(get_np_state_ptr(), seed=1)
         self.assertEqual([logseries(0.9999999999999) for i in range(10)],
                          [2022733531, 77296, 30, 52204, 9341294, 703057324,
                           413147702918, 1870715907, 16009330, 738])
@@ -749,7 +765,7 @@ class TestRandom(BaseTest):
         self.assertRaises(ValueError, logseries, 1.1)
 
     def test_numpy_poisson(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         poisson = jit_unary("np.random.poisson")
         # Our implementation follows Numpy's.
         self._check_dist(poisson, r.poisson,
@@ -758,7 +774,7 @@ class TestRandom(BaseTest):
         self.assertRaises(ValueError, poisson, -0.1)
 
     def test_numpy_negative_binomial(self):
-        self._follow_numpy(np_state_ptr, 0)
+        self._follow_numpy(get_np_state_ptr(), 0)
         negbin = jit_binary("np.random.negative_binomial")
         self.assertEqual([negbin(10, 0.9) for i in range(10)],
                          [2, 3, 1, 5, 2, 1, 0, 1, 0, 0])
@@ -778,7 +794,7 @@ class TestRandom(BaseTest):
 
     @tag('important')
     def test_numpy_power(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         power = jit_unary("np.random.power")
         self._check_dist(power, r.power,
                          [(0.1,), (0.5,), (0.9,), (6.0,)])
@@ -786,7 +802,7 @@ class TestRandom(BaseTest):
         self.assertRaises(ValueError, power, -0.1)
 
     def test_numpy_rayleigh(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         rayleigh1 = jit_unary("np.random.rayleigh")
         rayleigh0 = jit_nullary("np.random.rayleigh")
         self._check_dist(rayleigh1, r.rayleigh,
@@ -796,7 +812,7 @@ class TestRandom(BaseTest):
         self.assertRaises(ValueError, rayleigh1, -0.1)
 
     def test_numpy_standard_cauchy(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         cauchy = jit_nullary("np.random.standard_cauchy")
         self._check_dist(cauchy, r.standard_cauchy, [()])
 
@@ -804,14 +820,14 @@ class TestRandom(BaseTest):
         # We use CPython's algorithm for the gamma dist and numpy's
         # for the normal dist.  Standard T calls both so we can't check
         # against either generator's output.
-        r = self._follow_cpython(np_state_ptr)
+        r = self._follow_cpython(get_np_state_ptr())
         standard_t = jit_unary("np.random.standard_t")
         avg = np.mean([standard_t(5) for i in range(5000)])
         # Sanity check
         self.assertLess(abs(avg), 0.5)
 
     def test_numpy_wald(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         wald = jit_binary("np.random.wald")
         self._check_dist(wald, r.wald, [(1.0, 1.0), (2.0, 5.0)])
         self.assertRaises(ValueError, wald, 0.0, 1.0)
@@ -820,7 +836,7 @@ class TestRandom(BaseTest):
         self.assertRaises(ValueError, wald, 1.0, -0.1)
 
     def test_numpy_zipf(self):
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         zipf = jit_unary("np.random.zipf")
         self._check_dist(zipf, r.zipf, [(1.5,), (2.5,)], niters=100)
         for val in (1.0, 0.5, 0.0, -0.1):
@@ -828,30 +844,34 @@ class TestRandom(BaseTest):
 
     def _check_shuffle(self, func, ptr):
         """
-        Check a shuffle()-like function for 1D arrays.
+        Check a shuffle()-like function for arrays.
         """
         # Our implementation follows Python 3's.
-        a = np.arange(20)
+        arrs = [np.arange(20), np.arange(32).reshape((8, 4))]
         if sys.version_info >= (3,):
             r = self._follow_cpython(ptr)
-            for i in range(3):
-                got = a.copy()
-                expected = a.copy()
-                func(got)
-                r.shuffle(expected)
-                self.assertTrue(np.all(got == expected), (got, expected))
+            for a in arrs:
+                for i in range(3):
+                    got = a.copy()
+                    expected = a.copy()
+                    func(got)
+                    python_shuffle(expected, r)
+                    self.assertPreciseEqual(got, expected)
         else:
             # Sanity check
-            for i in range(3):
-                b = a.copy()
-                func(b)
-                self.assertFalse(np.all(a == b))
-                self.assertEqual(sorted(a), sorted(b))
-                a = b
+            for a in arrs:
+                for i in range(3):
+                    b = a.copy()
+                    func(b)
+                    self.assertFalse(np.array_equal(a, b))
+                    self.assertTrue(np.array_equal(np.sort(a, axis=0),
+                                                   np.sort(b, axis=0)))
+                    a = b
         # Test with an arbitrary buffer-providing object
+        a = arrs[0]
         b = a.copy()
         func(memoryview(b))
-        self.assertFalse(np.all(a == b))
+        self.assertNotEqual(list(a), list(b))
         self.assertEqual(sorted(a), sorted(b))
         # Read-only object
         with self.assertTypingError():
@@ -859,11 +879,11 @@ class TestRandom(BaseTest):
 
     @tag('important')
     def test_random_shuffle(self):
-        self._check_shuffle(jit_unary("random.shuffle"), py_state_ptr)
+        self._check_shuffle(jit_unary("random.shuffle"), get_py_state_ptr())
 
     @tag('important')
     def test_numpy_shuffle(self):
-        self._check_shuffle(jit_unary("np.random.shuffle"), np_state_ptr)
+        self._check_shuffle(jit_unary("np.random.shuffle"), get_np_state_ptr())
 
     def _check_startup_randomness(self, func_name, func_args):
         """
@@ -897,16 +917,61 @@ class TestRandom(BaseTest):
     def test_numpy_gauss_startup(self):
         self._check_startup_randomness("numpy_normal", (1.0, 1.0))
 
+    def test_numpy_random_permutation(self):
+
+        # Numba's current PRNG for NumPy follows Python's implementation, hence,
+        # we cannot test np.random.permutation against NumPy.  Since Python does
+        # not have a permutation implementation, with which we can test
+        # np.random.permutation, we implement one, just for testing purposes.
+        def python_permutation(r, x):
+            if isinstance(x, int):
+                arr = np.arange(x)
+            else:
+                arr = np.array(x)
+            python_shuffle(arr, r)
+            return arr
+
+        # Our implementation follows Python 3's.
+        func = jit_unary("np.random.permutation")
+        if sys.version_info >= (3,):
+            r = self._follow_cpython(get_np_state_ptr())
+            for s in [5, 10, 15, 20]:
+                a = np.arange(s)
+                b = a.copy()
+                # Test array version
+                self.assertPreciseEqual(func(a), python_permutation(r, a))
+                # Test int version
+                self.assertPreciseEqual(func(s), python_permutation(r, s))
+                # Permutation should not modify its argument
+                self.assertPreciseEqual(a, b)
+            # Check multi-dimensional arrays
+            arrs = [np.arange(10).reshape(2, 5),
+                    np.arange(27).reshape(3, 3, 3),
+                    np.arange(36).reshape(2, 3, 3, 2)]
+            for a in arrs:
+                b = a.copy()
+                self.assertPreciseEqual(func(a), python_permutation(r, a))
+                self.assertPreciseEqual(a, b)
+        else:
+            # Sanity check
+            arrs = [np.arange(20), np.arange(20).reshape(5, 2, 2)]
+            for a in arrs:
+                checked = 0
+                while checked < 3:
+                    b = func(a)
+                    # check that permuted arrays are equal when sorted
+                    # account for the possibility of the identity permutation
+                    if not np.array_equal(a, b):
+                        self.assertTrue(np.array_equal(np.sort(a, axis=0),
+                                                       np.sort(b, axis=0)))
+                        checked += 1
+
 
 class TestRandomArrays(BaseTest):
     """
     Test array-producing variants of np.random.* functions.
     """
 
-    def setUp(self):
-        # Make sure the PRNG is initialized before we set the seed
-        random_init()
-
     def _compile_array_dist(self, funcname, nargs):
         qualname = "np.random.%s" % (funcname,)
         argstring = ', '.join('abcd'[:nargs])
@@ -917,7 +982,7 @@ class TestRandomArrays(BaseTest):
         Check returning an array according to a given distribution.
         """
         cfunc = self._compile_array_dist(funcname, len(scalar_args) + 1)
-        r = self._follow_numpy(np_state_ptr)
+        r = self._follow_numpy(get_np_state_ptr())
         pyfunc = getattr(r, funcname)
         for size in (8, (2, 3)):
             args = scalar_args + (size,)
@@ -1037,10 +1102,6 @@ class TestRandomChoice(BaseTest):
     Test np.random.choice.
     """
 
-    def setUp(self):
-        # Make sure the PRNG is initialized before we set the seed
-        random_init()
-
     def _check_results(self, pop, res, replace=True):
         """
         Check basic expectations about a batch of samples.
@@ -1205,10 +1266,6 @@ class TestRandomMultinomial(BaseTest):
     pvals = np.array([1, 1, 1, 2, 3, 1], dtype=np.float64)
     pvals /= pvals.sum()
 
-    def setUp(self):
-        # Make sure the PRNG is initialized before we set the seed
-        random_init()
-
     def _check_sample(self, n, pvals, sample):
         """
         Check distribution of some samples.
@@ -1269,5 +1326,212 @@ class TestRandomMultinomial(BaseTest):
             self._check_sample(n, pvals, sample)
 
 
+
+@jit(nopython=True, nogil=True)
+def py_extract_randomness(seed, out):
+    if seed != 0:
+        random.seed(seed)
+    for i in range(out.size):
+        out[i] = random.getrandbits(32)
+
+_randint_limit = 1 << 32
+
+@jit(nopython=True, nogil=True)
+def np_extract_randomness(seed, out):
+    if seed != 0:
+        np.random.seed(seed)
+    s = 0
+    for i in range(out.size):
+        out[i] = np.random.randint(_randint_limit)
+
+
+
+class ConcurrencyBaseTest(TestCase):
+
+    # Enough iterations for:
+    # 1. Mersenne-Twister state shuffles to occur (once every 624)
+    # 2. Race conditions to be plausible
+    # 3. Nice statistical properties to emerge
+    _extract_iterations = 100000
+
+    def setUp(self):
+        # Warm up, to avoid compiling in the threads
+        args = (42, self._get_output(1))
+        py_extract_randomness(*args)
+        np_extract_randomness(*args)
+
+    def _get_output(self, size):
+        return np.zeros(size, dtype=np.uint32)
+
+    def check_output(self, out):
+        """
+        Check statistical properties of output.
+        """
+        # Output should follow a uniform distribution in [0, 1<<32)
+        expected_avg = 1 << 31
+        expected_std = (1 << 32) / np.sqrt(12)
+        rtol = 0.05  # given enough iterations
+        np.testing.assert_allclose(out.mean(), expected_avg, rtol=rtol)
+        np.testing.assert_allclose(out.std(), expected_std, rtol=rtol)
+
+    def check_several_outputs(self, results, same_expected):
+        # Outputs should have the expected statistical properties
+        # (an unitialized PRNG or a PRNG whose internal state was
+        #  corrupted by a race condition could produce bogus randomness)
+        for out in results:
+            self.check_output(out)
+
+        # Check all threads gave either the same sequence or
+        # distinct sequences
+        if same_expected:
+            expected_distinct = 1
+        else:
+            expected_distinct = len(results)
+
+        heads = {tuple(out[:5]) for out in results}
+        tails = {tuple(out[-5:]) for out in results}
+        sums = {out.sum() for out in results}
+        self.assertEqual(len(heads), expected_distinct, heads)
+        self.assertEqual(len(tails), expected_distinct, tails)
+        self.assertEqual(len(sums), expected_distinct, sums)
+
+
+class TestThreads(ConcurrencyBaseTest):
+    """
+    Check the PRNG behaves well with threads.
+    """
+
+    def extract_in_threads(self, nthreads, extract_randomness, seed):
+        """
+        Run *nthreads* threads extracting randomness with the given *seed*
+        (no seeding if 0).
+        """
+        results = [self._get_output(self._extract_iterations)
+                   for i in range(nthreads + 1)]
+
+        def target(i):
+            # The PRNG will be seeded in thread
+            extract_randomness(seed=seed, out=results[i])
+
+        threads = [threading.Thread(target=target, args=(i,))
+                   for i in range(nthreads)]
+
+        for th in threads:
+            th.start()
+        # Exercise main thread as well
+        target(nthreads)
+        for th in threads:
+            th.join()
+
+        return results
+
+    def check_thread_safety(self, extract_randomness):
+        """
+        When initializing the PRNG the same way, each thread
+        should produce the same sequence of random numbers,
+        using independent states, regardless of parallel
+        execution.
+        """
+        # Note the seed value doesn't matter, as long as it's
+        # the same for all threads
+        results = self.extract_in_threads(15, extract_randomness, seed=42)
+
+        # All threads gave the same sequence
+        self.check_several_outputs(results, same_expected=True)
+
+    def check_implicit_initialization(self, extract_randomness):
+        """
+        The PRNG in new threads should be implicitly initialized with
+        system entropy, if seed() wasn't called.
+        """
+        results = self.extract_in_threads(4, extract_randomness, seed=0)
+
+        # All threads gave a different, valid random sequence
+        self.check_several_outputs(results, same_expected=False)
+
+    def test_py_thread_safety(self):
+        self.check_thread_safety(py_extract_randomness)
+
+    def test_np_thread_safety(self):
+        self.check_thread_safety(np_extract_randomness)
+
+    def test_py_implicit_initialization(self):
+        self.check_implicit_initialization(py_extract_randomness)
+
+    def test_np_implicit_initialization(self):
+        self.check_implicit_initialization(np_extract_randomness)
+
+
+@unittest.skipIf(os.name == 'nt', "Windows is not affected by fork() issues")
+class TestProcesses(ConcurrencyBaseTest):
+    """
+    Check the PRNG behaves well in child processes.
+    """
+
+    # Avoid nested multiprocessing AssertionError
+    # ("daemonic processes are not allowed to have children")
+    _numba_parallel_test_ = False
+
+
+    def extract_in_processes(self, nprocs, extract_randomness):
+        """
+        Run *nprocs* processes extracting randomness
+        without explicit seeding.
+        """
+        q = multiprocessing.Queue()
+        results = []
+
+        def target_inner():
+            out = self._get_output(self._extract_iterations)
+            extract_randomness(seed=0, out=out)
+            return out
+
+        def target():
+            try:
+                out = target_inner()
+                q.put(out)
+            except Exception as e:
+                # Ensure an exception in a child gets reported
+                # in the parent.
+                q.put(e)
+                raise
+
+        procs = [multiprocessing.Process(target=target)
+                 for i in range(nprocs)]
+        for p in procs:
+            p.start()
+        # Need to dequeue before joining, otherwise the large size of the
+        # enqueued objects will lead to deadlock.
+        for i in range(nprocs):
+            results.append(q.get(timeout=5))
+        for p in procs:
+            p.join()
+
+        # Exercise parent process as well; this will detect if the
+        # same state was reused for one of the children.
+        results.append(target_inner())
+        for res in results:
+            if isinstance(res, Exception):
+                self.fail("Exception in child: %s" % (res,))
+
+        return results
+
+    def check_implicit_initialization(self, extract_randomness):
+        """
+        The PRNG in new processes should be implicitly initialized
+        with system entropy, to avoid reproducing the same sequences.
+        """
+        results = self.extract_in_processes(2, extract_randomness)
+
+        # All processes gave a different, valid random sequence
+        self.check_several_outputs(results, same_expected=False)
+
+    def test_py_implicit_initialization(self):
+        self.check_implicit_initialization(py_extract_randomness)
+
+    def test_np_implicit_initialization(self):
+        self.check_implicit_initialization(np_extract_randomness)
+
+
 if __name__ == "__main__":
     unittest.main()
