@@ -92,6 +92,9 @@ def type_usecase(tup, *args):
 def identity(tup):
     return tup
 
+def index_method_usecase(tup, value):
+    return tup.index(value)
+
 
 class TestTupleReturn(TestCase):
 
@@ -314,9 +317,9 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
                 cfunc = jit(nopython=True)(pyfunc)
                 self.assertPreciseEqual(cfunc(p), pyfunc(p))
 
-        # Homogenous
+        # Homogeneous
         check(Rect(4, 5))
-        # Heterogenous
+        # Heterogeneous
         check(Rect(4, 5.5))
 
     def test_len(self):
@@ -325,10 +328,10 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
             cfunc = jit(nopython=True)(pyfunc)
             self.assertPreciseEqual(cfunc(p), pyfunc(p))
 
-        # Homogenous
+        # Homogeneous
         check(Rect(4, 5))
         check(Point(4, 5, 6))
-        # Heterogenous
+        # Heterogeneous
         check(Rect(4, 5.5))
         check(Point(4, 5.5, 6j))
 
@@ -346,9 +349,9 @@ class TestNamedTuple(TestCase, MemoryLeakMixin):
             cfunc = jit(nopython=True)(pyfunc)
             self.assertPreciseEqual(cfunc(p), pyfunc(p))
 
-        # Homogenous
+        # Homogeneous
         check(Rect(4, 5))
-        # Heterogenous
+        # Heterogeneous
         check(Rect(4, 5.5))
         check(Empty())
 
@@ -480,8 +483,21 @@ class TestConversions(TestCase):
 
         with self.assertRaises(errors.TypingError) as raises:
             check(fromty, types.Tuple((types.float32,)), (4, 5))
-        self.assertIn("No conversion from (int32 x 2) to (float32 x 1)",
-                      str(raises.exception))
+        msg = "No conversion from tuple(int32 x 2) to tuple(float32 x 1)"
+        self.assertIn(msg, str(raises.exception))
+
+
+class TestMethods(TestCase):
+
+    def test_index(self):
+        pyfunc = index_method_usecase
+        cfunc = jit(nopython=True)(pyfunc)
+        self.assertEqual(cfunc((1, 2, 3), 2), 1)
+
+        with self.assertRaises(ValueError) as raises:
+            cfunc((1, 2, 3), 4)
+        msg = 'tuple.index(x): x not in tuple'
+        self.assertEqual(msg, str(raises.exception))
 
 
 if __name__ == '__main__':
