@@ -15,6 +15,7 @@ import numpy as np
 import numba.unittest_support as unittest
 from numba import cffi_support, numpy_support, types
 from numba.special import typeof
+from numba.dispatcher import OmittedArg
 from numba._dispatcher import compute_fingerprint
 
 from .support import TestCase, tag
@@ -203,7 +204,8 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         v = set([1.0, 2.0, 3.0])
         self.assertEqual(typeof(v), types.Set(types.float64, reflected=True))
         v = frozenset(v)
-        self.assertIs(typeof(v), None)
+        with self.assertRaises(ValueError):
+            typeof(v)
 
     @tag('important')
     def test_namedtuple(self):
@@ -235,9 +237,9 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertEqual(tp_404, typeof(RequestError.internal_error))
 
         with self.assertRaises(ValueError) as raises:
-            typeof(HeterogenousEnum.red)
+            typeof(HeterogeneousEnum.red)
         self.assertEqual(str(raises.exception),
-                         "Cannot type heterogenous enum: got value types complex128, float64")
+                         "Cannot type heterogeneous enum: got value types complex128, float64")
 
     @tag('important')
     def test_enum_class(self):
@@ -254,9 +256,9 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertNotEqual(tp_error, tp_shape)
 
         with self.assertRaises(ValueError) as raises:
-            typeof(HeterogenousEnum)
+            typeof(HeterogeneousEnum)
         self.assertEqual(str(raises.exception),
-                         "Cannot type heterogenous enum: got value types complex128, float64")
+                         "Cannot type heterogeneous enum: got value types complex128, float64")
 
     @tag('important')
     def test_dtype(self):
@@ -285,7 +287,9 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         mod.init()
         ty_cffi_cos = typeof(mod.cffi_cos)
         ty_cffi_sin = typeof(mod.cffi_sin)
+        ty_cffi_boolean = typeof(mod.cffi_bool)
         self.assertIsInstance(ty_cffi_cos, types.ExternalFunctionPointer)
+        self.assertEqual(ty_cffi_boolean.sig.return_type, types.boolean)
         self.assertEqual(ty_cffi_cos.sig.args, (types.float64,))
         self.assertEqual(ty_cffi_cos.sig.return_type, types.float64)
         self.assertEqual(ty_cffi_cos, ty_cffi_sin)
@@ -300,6 +304,17 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         ty = typeof(Custom())
         self.assertEqual(ty, types.UniTuple(types.boolean, 42))
 
+    def test_omitted_args(self):
+        ty0 = typeof(OmittedArg(0.0))
+        ty1 = typeof(OmittedArg(1))
+        ty2 = typeof(OmittedArg(1.0))
+        ty3 = typeof(OmittedArg(1.0))
+        self.assertEqual(ty0, types.Omitted(0.0))
+        self.assertEqual(ty1, types.Omitted(1))
+        self.assertEqual(ty2, types.Omitted(1.0))
+        self.assertEqual(len({ty0, ty1, ty2}), 3)
+        self.assertEqual(ty3, ty2)
+
 
 class DistinctChecker(object):
 
@@ -528,6 +543,20 @@ class TestFingerprint(TestCase):
         with self.assertRaises(NotImplementedError):
             compute_fingerprint(frozenset([2, 3]))
 
+    def test_omitted_args(self):
+        distinct = DistinctChecker()
+
+        v0 = OmittedArg(0.0)
+        v1 = OmittedArg(1.0)
+        v2 = OmittedArg(1)
+
+        s = compute_fingerprint(v0)
+        self.assertEqual(compute_fingerprint(v1), s)
+        distinct.add(s)
+        distinct.add(compute_fingerprint(v2))
+        distinct.add(compute_fingerprint(0.0))
+        distinct.add(compute_fingerprint(1))
+
     def test_complicated_type(self):
         # Generating a large fingerprint
         t = None
