@@ -531,5 +531,34 @@ class TestRecordDtype(unittest.TestCase):
         self.assertEqual(b, arr[0]['b'])
 
 
+class TestDType(TestCase):
+    def test_type_attr(self):
+        # Test .type attribute of dtype
+        def conv(arr, val):
+            return arr.dtype.type(val)
+
+        jit_conv = jit(nopython=True)(conv)
+
+        def assert_matches(arr, val, exact):
+            expect = conv(arr, val)
+            got = jit_conv(arr, val)
+            self.assertPreciseEqual(expect, exact)
+            self.assertPreciseEqual(typeof(expect), typeof(got))
+            self.assertPreciseEqual(expect, got)
+
+        arr = np.zeros(5)
+        assert_matches(arr.astype(np.intp), 1.2, 1)
+        assert_matches(arr.astype(np.float64), 1.2, 1.2)
+        assert_matches(arr.astype(np.complex128), 1.2, (1.2 + 0j))
+        assert_matches(arr.astype(np.complex128), 1.2j, 1.2j)
+
+    def test_kind(self):
+        def tkind(A):
+            return A.dtype.kind=='f'
+        jit_tkind = jit(nopython=True)(tkind)
+        self.assertEqual(tkind(np.ones(3)), jit_tkind(np.ones(3)))
+        self.assertEqual(tkind(np.ones(3, dtype=np.intp)),
+                            jit_tkind(np.ones(3, dtype=np.intp)))
+
 if __name__ == '__main__':
     unittest.main()
