@@ -106,7 +106,7 @@ class IterNext(AbstractTemplate):
 @infer
 class PairFirst(AbstractTemplate):
     """
-    Given a heterogenous pair, return the first element.
+    Given a heterogeneous pair, return the first element.
     """
     key = "pair_first"
 
@@ -120,7 +120,7 @@ class PairFirst(AbstractTemplate):
 @infer
 class PairSecond(AbstractTemplate):
     """
-    Given a heterogenous pair, return the second element.
+    Given a heterogeneous pair, return the second element.
     """
     key = "pair_second"
 
@@ -953,7 +953,7 @@ def typeof_index(val, c):
 @type_callable(IndexValue)
 def type_index_value(context):
     def typer(ind, mval):
-        if ind == types.intp:
+        if ind == types.intp or ind == types.uintp:
             return IndexValueType(mval)
     return typer
 
